@@ -1,4 +1,4 @@
 #include <config.h>
 
-#define T1LIB_IDENT "0.8-beta-grace"
+#define T1LIB_IDENT "0.9.2p0-grace"
 #define T1LIB_NO_X11_SUPPORT
