@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------------
   ----- File:        t1base.c 
   ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
-  ----- Date:        06/13/1998
+  ----- Date:        1999-08-26
   ----- Description: This file is part of the t1-library. It contains basic
                      basic routines to initialize the data structures used
 		     by the t1-library.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-1998. 
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-1999. 
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -26,6 +26,7 @@
                      Thanks to all people who make free software living!
 --------------------------------------------------------------------------*/
 
+
 #define T1BASE_C
 
 
@@ -82,22 +83,38 @@ void *T1_InitLib( int log)
   else
     pFontBase->bitmap_pad=T1GLYPH_PAD;
 
+  pFontBase->pFontArray = NULL;
+  pFontBase->t1lib_flags=0;
+  /* Chek for AA-caching */
+  if ((log & T1_AA_CACHING)){
+    pFontBase->t1lib_flags |= T1_AA_CACHING;
+  }
+  /* Check for AFM disable */
+  if ((log & T1_NO_AFM)) {
+    pFontBase->t1lib_flags |= T1_NO_AFM;
+  }
+  
+  
   /* Open log-file: */
-  if ((log & LOGFILE)){
+  if ((log & LOGFILE)) {
+    pFontBase->t1lib_flags |= LOGFILE;
     /* Try first opening in current directory: */
-    if ((t1lib_log_file=fopen( T1_LOG_FILE, "w"))==NULL){
-
-      usershome=getenv("HOME");
-      logfilepath=(char *)malloc((strlen(usershome) +
-					   strlen(T1_LOG_FILE) + 2
-					   ) * sizeof(char));
-      strcpy( logfilepath, usershome);
-      strcat( logfilepath, DIRECTORY_SEP);
-      strcat( logfilepath, T1_LOG_FILE);
-      if ((t1lib_log_file=fopen( logfilepath, "w"))==NULL){
-	print_msg( "T1_InitLib()", "Warning: Unable to open log-file");
+    if ((t1lib_log_file=fopen( T1_LOG_FILE, "w"))==NULL) {
+      if ((usershome=getenv("HOME"))!=NULL) {
+	logfilepath=(char *)malloc((strlen(usershome) +
+				    strlen(T1_LOG_FILE) + 2
+				    ) * sizeof(char));
+	strcpy( logfilepath, usershome);
+	strcat( logfilepath, DIRECTORY_SEP);
+	strcat( logfilepath, T1_LOG_FILE);
+	if ((t1lib_log_file=fopen( logfilepath, "w"))==NULL){
+	  print_msg( "T1_InitLib()", "Warning: Unable to open log-file");
+	}
+	free( logfilepath);
+      }
+      else {
+	print_msg( "T1_InitLib()", "Warning: Unable to open log-file in . and $(HOME)");
       }
-      free( logfilepath);
     }
     T1_PrintLog( "T1_InitLib()", "Initialization started",
 	       T1LOG_STATISTIC);
@@ -117,6 +134,10 @@ void *T1_InitLib( int log)
 
   /* Save version identifier */
   sprintf( err_warn_msg_buf, "Version Identifier: %s", T1LIB_IDENT);
+  T1_PrintLog( "T1_InitLib()", err_warn_msg_buf,
+	       T1LOG_DEBUG);
+  /* Save how t1lib is initialized */
+  sprintf( err_warn_msg_buf, "Initialization flags: 0x%X", log);
   T1_PrintLog( "T1_InitLib()", err_warn_msg_buf,
 	       T1LOG_DEBUG);
   /* Save padding value in log file */
@@ -160,6 +181,7 @@ void *T1_InitLib( int log)
   T1_PrintLog( "T1_InitLib()", err_warn_msg_buf, T1LOG_DEBUG);
   
   if (log & IGNORE_CONFIGFILE){
+    pFontBase->t1lib_flags |= IGNORE_CONFIGFILE;
     T1_PrintLog( "T1_InitLib()", "Skipping configuration file search!",
 		 T1LOG_STATISTIC);
   }
@@ -176,6 +198,7 @@ void *T1_InitLib( int log)
   
   /* Check whether to read font database */
   if ((log & IGNORE_FONTDATABASE)){
+    pFontBase->t1lib_flags |= IGNORE_FONTDATABASE;
     /* Initialize the no_fonts... values */
     pFontBase->no_fonts=0;
     pFontBase->no_fonts_ini=pFontBase->no_fonts;
@@ -280,12 +303,17 @@ int scanFontDBase( char *filename)
 	/* We are at the last printable character of a line; let's 
 	   step back (over the [afm/sfm]) to the period and replace it
 	   by an ASCII 0 */
-	while (filebuffer[k]!='.'){
+	while ((filebuffer[k]!='.') && (!isspace((int)filebuffer[k])) ){
 	  k--;
 	}
-	filebuffer[k]=0; /* termination for string reading */
-	while (!isspace((int)filebuffer[k])){
-	  k--;
+	if (filebuffer[k]=='.'){ /* We have a name terminated with . */
+	  filebuffer[k]=0; /* termination for string reading */
+	  while (!isspace((int)filebuffer[k])){
+	    k--;
+	  }
+	}
+	else { /* The filename was without . and / or the first on the line */ 
+	  ;
 	}
 	sscanf( &filebuffer[k+1], "%s", &linebuf[0]);
 	/* We print error string before testing because after the call
@@ -341,11 +369,17 @@ int T1_CloseLib( void)
     for (i=pFontBase->no_fonts; i; i--){
       /* Free filename only if not NULL and if the font is physical!
 	 Do it before removing the font since the physical information
-	 is no more available afterwards
+	 is no more available afterwards. If necessary, an explicitly
+	 specified AFM filename is also freed.
 	 */
       if ((pFontBase->pFontArray[i-1].pFontFileName!=NULL)
 	  && (pFontBase->pFontArray[i-1].physical==1)){
 	free( pFontBase->pFontArray[i-1].pFontFileName);
+	pFontBase->pFontArray[i-1].pFontFileName=NULL;
+	if (pFontBase->pFontArray[i-1].pAfmFileName!=NULL){
+	  free( pFontBase->pFontArray[i-1].pAfmFileName);
+	  pFontBase->pFontArray[i-1].pAfmFileName=NULL;
+	}
       }
       
       /* Now, remove font: */
@@ -380,6 +414,9 @@ int T1_CloseLib( void)
 	free( T1_FDB_ptr);
     T1_FDB_ptr=T1_fontdatabase;
 
+    /* Reset the flags */
+    pFontBase->t1lib_flags=0;
+    
     /* Indicate Library is no more initialized */
     pFontBase=NULL;
     T1_Up=0;
@@ -424,26 +461,35 @@ int T1_AddFont( char *fontfilename)
      realloc memory some amount larger */
   save_ptr=pFontBase->pFontArray;
   if (pFontBase->no_fonts==pFontBase->no_fonts_limit){
-    pFontBase->pFontArray=(FONTPRIVATE *)realloc(pFontBase->pFontArray,
-						 (pFontBase->no_fonts_limit
-						  + ADVANCE_FONTPRIVATE)
-						 * sizeof(FONTPRIVATE));
-    if (pFontBase->pFontArray==NULL){
-      /* Restore pointer */
-      pFontBase->pFontArray=save_ptr;
-      T1_errno=T1ERR_ALLOC_MEM;
-      return(-2); /* No memory available */
+    if (pFontBase->pFontArray == NULL) {
+      /* In case this is the first font */
+      pFontBase->pFontArray=(FONTPRIVATE *)calloc(pFontBase->no_fonts_limit 
+						  + ADVANCE_FONTPRIVATE,
+						  sizeof(FONTPRIVATE));
+    }
+    else {
+      /* We already have some fonts */
+      pFontBase->pFontArray=(FONTPRIVATE *)realloc(pFontBase->pFontArray,
+						   (pFontBase->no_fonts_limit
+						    + ADVANCE_FONTPRIVATE)
+						   * sizeof(FONTPRIVATE));
+      if (pFontBase->pFontArray==NULL){
+	/* Restore pointer */
+	pFontBase->pFontArray=save_ptr;
+	T1_errno=T1ERR_ALLOC_MEM;
+	return(-2); /* No memory available */
+      }
     }
     pFontBase->no_fonts_limit += ADVANCE_FONTPRIVATE;
     /* First, initialize newly allocated to be not used */
     for ( i=pFontBase->no_fonts;
 	  i<pFontBase->no_fonts+ADVANCE_FONTPRIVATE; i++){
       pFontBase->pFontArray[i].pFontFileName=NULL;
+      pFontBase->pFontArray[i].pAfmFileName=NULL;
       pFontBase->pFontArray[i].pAFMData=NULL;
       pFontBase->pFontArray[i].pType1Data=NULL;
       pFontBase->pFontArray[i].pEncMap=NULL;
       pFontBase->pFontArray[i].pKernMap=NULL;
-      pFontBase->pFontArray[i].pIndexMap=NULL;
       pFontBase->pFontArray[i].pFontEnc=NULL;
       pFontBase->pFontArray[i].pFontSizeDeps=NULL;
       pFontBase->pFontArray[i].vm_base=NULL;
@@ -488,7 +534,6 @@ int T1_AddFont( char *fontfilename)
   
 }
 
-		  
 
 /* This function prints a message to stderr and places an entry in the log
    file */
@@ -584,7 +629,13 @@ int test_for_t1_file( char *buffer )
   int i=0;
   char *FullName;
   
-  
+  /* First case: A PostScript Font ASCII File without extension
+     (according to some UNIX-conventions) */
+  if ((FullName=Env_GetCompletePath(buffer,T1_PFAB_ptr))!=NULL){
+    free(FullName);
+    return(0);
+  }
+
   while (buffer[i]!=0){
     i++;
   }
@@ -594,13 +645,13 @@ int test_for_t1_file( char *buffer )
   buffer[i+4]=0;
 
   
-  /* First case: A PostScript Font ASCII File */
+  /* Second case: A PostScript Font ASCII File */
   buffer[i+3]='a';
   if ((FullName=Env_GetCompletePath(buffer,T1_PFAB_ptr))!=NULL){
     free(FullName);
     return(0);
   }
-  /* Second case: A PostScript Font Binary File */
+  /* Third case: A PostScript Font Binary File */
   buffer[i+3]='b';
   if ((FullName=Env_GetCompletePath(buffer,T1_PFAB_ptr))!=NULL){
     free(FullName);
@@ -618,7 +669,6 @@ int test_for_t1_file( char *buffer )
 /* T1_GetFontFileName() returns a pointer to the filename of the font,
    associated with FontID. This filename does not contain a full path.
    */
-
 char *T1_GetFontFileName( int FontID)
 {
 
@@ -640,8 +690,73 @@ char *T1_GetFontFileName( int FontID)
 
 
 
-/* T1_Get_no_fonts(): Return the number of declared fonts */
+/* As suggested by Nicolai Langfeldt, we make it possible to specify
+   a completely independent path for the afm filename. This should
+   make t1lib usable in context with using the kpathsearch-library.
+   We allow setting those paths after initialization, but before a
+   font is loaded.
+   returns  0:   OK
+           -1:   Operation could not be performed
+*/
+int T1_SetAfmFileName( int FontID, char *afm_name)
+{
 
+  if (CheckForFontID(FontID)!=0){
+    /* Operation may not be applied because FontID is invalid
+       or font is loaded */
+    T1_errno=T1ERR_INVALID_FONTID;
+    return(-1);
+  }
+  if (afm_name==NULL) {
+    T1_errno=T1ERR_INVALID_PARAMETER;
+    return(-1);
+  }
+  if (pFontBase->pFontArray[FontID].pAfmFileName!=NULL){
+    /* we first free the current name */
+    free( pFontBase->pFontArray[FontID].pAfmFileName);
+    pFontBase->pFontArray[FontID].pAfmFileName=NULL;
+  }
+  
+  if ((pFontBase->pFontArray[FontID].pAfmFileName=
+       (char *)malloc( (strlen(afm_name)+1)*sizeof( char)))==NULL) {
+    T1_errno=T1ERR_ALLOC_MEM;
+    return( -1);
+  }
+  strcpy( pFontBase->pFontArray[FontID].pAfmFileName, afm_name);
+	  
+  return(0);
+  
+}
+
+		  
+ 
+/* We have a function for querying the name. Returns a pointer
+   to the string or NULL if name was not explicitly set .*/
+char *T1_GetAfmFileName( int FontID)
+{
+
+  static char filename[MAXPATHLEN+1];
+  
+  if (CheckForInit())return(NULL);
+
+  /* Check first for valid FontID */
+  if ((FontID<0) || (FontID>FontBase.no_fonts)){
+    T1_errno=T1ERR_INVALID_FONTID;
+    return(NULL);
+  }
+
+  if (pFontBase->pFontArray[FontID].pAfmFileName==NULL) {
+    return( NULL);
+  }
+  
+  strcpy( filename, pFontBase->pFontArray[FontID].pAfmFileName);
+  return( filename);
+  
+}
+
+
+  
+/* T1_Get_no_fonts(): Return the number of declared fonts */
 int  T1_Get_no_fonts(void)
 {
   if (CheckForInit())return(-1);
@@ -743,6 +858,11 @@ int T1_CopyFont( int FontID)
       T1_errno=T1ERR_ALLOC_MEM;
       return(-3);
     }
+    /* We zero the newly allocated memory */
+    if (pFontBase->pFontArray != NULL) {
+      memset( pFontBase->pFontArray + pFontBase->no_fonts_limit, 0,
+	      ADVANCE_FONTPRIVATE * sizeof(FONTPRIVATE));
+    }
     pFontBase->no_fonts_limit += ADVANCE_FONTPRIVATE;
   }
   /* no_fonts-1 was the largest allowed font ID */
@@ -752,48 +872,49 @@ int T1_CopyFont( int FontID)
   /* (Re)Set some values explicitly, others remain untouched: */  
   pFontBase->pFontArray[new_ID].pFontSizeDeps=NULL;
   pFontBase->pFontArray[new_ID].physical=0;
-  /* AFM-mapping tables are to be setup for logical font separately */
-  k=pFontBase->pFontArray[new_ID].pAFMData->numOfPairs;
-  if (k>0){ /* kern map exists only if kerning pairs exist! */
-    if ((pFontBase->pFontArray[new_ID].pKernMap=
-	 (METRICS_ENTRY *)malloc( k*sizeof( METRICS_ENTRY)))==NULL){
-      sprintf( err_warn_msg_buf, "Error allocating memory for kerning map (new_ID=%d)",
+  /* AFM-mapping tables are to be setup for logical fonts separately
+   (if AFM data is there) */
+  /* first, kerning map */
+  if (pFontBase->pFontArray[new_ID].pAFMData) {
+    k=pFontBase->pFontArray[new_ID].pAFMData->numOfPairs;
+    if (k>0){ /* kern map exists only if kerning pairs exist! */
+      if ((pFontBase->pFontArray[new_ID].pKernMap=
+	   (METRICS_ENTRY *)malloc( k*sizeof( METRICS_ENTRY)))==NULL){
+	sprintf( err_warn_msg_buf, "Error allocating memory for kerning map (new_ID=%d)",
+		 new_ID);
+	T1_PrintLog( "T1_CopyFont()", err_warn_msg_buf,
+		     T1LOG_WARNING);
+	T1_errno=T1ERR_ALLOC_MEM;
+	return(-4);
+      }
+      memcpy( pFontBase->pFontArray[new_ID].pKernMap,
+	      pFontBase->pFontArray[FontID].pKernMap,
+	      k*sizeof( METRICS_ENTRY));
+    } 
+    else { /* no kerning pairs, bu AFM data present */
+      pFontBase->pFontArray[new_ID].pKernMap=NULL;
+    }
+  } 
+  else { /* AFM data not present at all */
+    pFontBase->pFontArray[new_ID].pKernMap=NULL;
+  }
+
+  /* second, encoding map */
+  if (pFontBase->pFontArray[FontID].pEncMap!=NULL) {
+    if ((pFontBase->pFontArray[new_ID].pEncMap=
+	 (int *)calloc(256,sizeof(int)))==NULL){
+      sprintf( err_warn_msg_buf,
+	       "Error allocating memory for encoding map (new_ID=%d)",
 	       new_ID);
       T1_PrintLog( "T1_CopyFont()", err_warn_msg_buf,
 		   T1LOG_WARNING);
       T1_errno=T1ERR_ALLOC_MEM;
       return(-4);
     }
-    memcpy( pFontBase->pFontArray[new_ID].pKernMap,
-	    pFontBase->pFontArray[FontID].pKernMap,
-	    k*sizeof( METRICS_ENTRY));
+    memcpy( pFontBase->pFontArray[new_ID].pEncMap,
+	    pFontBase->pFontArray[FontID].pEncMap,
+	    256*sizeof( int));
   }
-  else
-    pFontBase->pFontArray[new_ID].pKernMap=NULL;
-  if ((pFontBase->pFontArray[new_ID].pIndexMap= 
-       (int *)calloc(256,sizeof(int)))==NULL){ 
-    sprintf( err_warn_msg_buf, "Error allocating memory for flag map (new_ID=%d)",
-	     new_ID);
-    T1_PrintLog( "T1_CopyFont()", err_warn_msg_buf,
-		 T1LOG_WARNING);
-    T1_errno=T1ERR_ALLOC_MEM;
-    return(-4); 
-  } 
-  memcpy( pFontBase->pFontArray[new_ID].pIndexMap,
-	  pFontBase->pFontArray[FontID].pIndexMap,
-	  256*sizeof(int));
-  if ((pFontBase->pFontArray[new_ID].pEncMap=
-       (int *)calloc(256,sizeof(int)))==NULL){
-    sprintf( err_warn_msg_buf, "Error allocating memory for encoding map (new_ID=%d)",
-	     new_ID);
-    T1_PrintLog( "T1_CopyFont()", err_warn_msg_buf,
-		 T1LOG_WARNING);
-    T1_errno=T1ERR_ALLOC_MEM;
-    return(-4);
-  }
-  memcpy( pFontBase->pFontArray[new_ID].pEncMap,
-	  pFontBase->pFontArray[FontID].pEncMap,
-	  256*sizeof( int));
   
   /* New font is logical --> indicate to which physical font it
      refers by setting refcount: */
@@ -834,9 +955,11 @@ int T1_SetBitmapPad( int pad)
     case 16:
       T1_pad=16;
       return(0);
+#ifdef T1_AA_TYPE64
     case 32:
       T1_pad=32;
       return(0);
+#endif
     default:
       T1_errno=T1ERR_INVALID_PARAMETER;
       return(-1);
@@ -893,7 +1016,7 @@ int T1_SetFontDataBase( char *filename)
 
 /* bin_dump(): Print a binary dump of a byte, short and
    long variable (used for debug purposes only): */
-void bin_dump_c(unsigned char value)
+void bin_dump_c(unsigned char value, char space_flag)
 {
   int i,j;
   
@@ -903,11 +1026,12 @@ void bin_dump_c(unsigned char value)
     else
       printf(".");
   }
-  printf(" ");
+  if (space_flag)
+    printf(" ");
 
 }
 
-void bin_dump_s(unsigned short value)
+void bin_dump_s(unsigned short value, char space_flag)
 {
   int i,j;
 
@@ -933,11 +1057,12 @@ void bin_dump_s(unsigned short value)
 	printf(".");
     }
   }
-  printf(" ");
+  if (space_flag)
+    printf(" ");
   
 }
 
-void bin_dump_l(unsigned long value)
+void bin_dump_l(unsigned long value, char space_flag)
 {
   int i,j;
   
@@ -975,7 +1100,8 @@ void bin_dump_l(unsigned long value)
 	printf(".");
     }
   }
-  printf(" ");
+  if (space_flag)
+    printf(" ");
 
 }
 
@@ -1010,6 +1136,7 @@ char *T1_GetLibIdent( void)
   static char buf[15];
 
   sprintf( buf, "%s", T1LIB_IDENT);
+  
   return( (char *)buf);
 }
 
