@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------------
   ----- File:        t1load.c 
   ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
-  ----- Date:        06/15/1998
+  ----- Date:        1999-09-03
   ----- Description: This file is part of the t1-library. It contains
                      functions for loading fonts  and for managing size
 		     dependent data.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-1998. 
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-1999. 
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -59,10 +59,9 @@
 #include "t1afmtool.h"
 
 
-extern char *vm_base; /* from fontfcn.c in initfont()! */
-extern char *vm_used; /* from fontfcn.c in fontfcnA()! */
 
-int fontfcnA( char *env, int *mode, psfont *Font_Ptr);
+extern psobj *StdEncArrayP;       /* For checking of a fonts encoding */
+extern char not_def[];            /* for checking the ".notdef"-string */
 
 
 
@@ -85,7 +84,6 @@ int T1_LoadFont( int FontID)
   char *tmp_ptr;
 #endif
 
-  extern psobj *StdEncArrayP;       /* For checking of a fonts encoding */
 
   /* These are for constructing the kerning lookup table: */
   PairKernData *pkd;
@@ -230,8 +228,11 @@ int T1_LoadFont( int FontID)
 	else{ /* Font-specific encoding */ 
 	  for (k=0; k<256; k++){
 	    ldummy=(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP;
-	    ldummy +=shift;
-	    pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP=(struct ps_obj *)ldummy;
+	    /* The ".notdef" is also static and may not be shifted (Thanks, Derek ;) */
+	    if (ldummy != (unsigned long)not_def) {
+	      ldummy +=shift;
+	      pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP=(struct ps_obj *)ldummy;
+	    }
 	  }
 	}
       } /* end of encoding-handling */
@@ -362,8 +363,11 @@ int T1_LoadFont( int FontID)
 	else{ /* Font-specific encoding */ 
 	  for (k=0; k<256; k++){
 	    ldummy=(long)pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP;
-	    ldummy -=shift;
-	    pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP=(struct ps_obj *)ldummy;
+	    /* The ".notdef" is also static and may not be shifted (Thanks, Derek ;) */
+	    if (ldummy != (unsigned long)not_def) {
+	      ldummy -=shift;
+	      pFontBase->pFontArray[FontID].pType1Data->fontInfoP[j].value.data.arrayP[k].data.arrayP=(struct ps_obj *)ldummy;
+	    }
 	  }
 	}
       } /* end of encoding-handling */
@@ -466,44 +470,54 @@ int T1_LoadFont( int FontID)
   pFontBase->pFontArray[FontID].extend=1.0;
 
   
-  /* Now try to load afm-structures from corresponding .afm-file. */
-  if ((i=openFontMetricsFile( FontID))){
-    /* Try a fallback function: */
-    if ((i=openFontMetricsFileSloppy( FontID))){
-      sprintf( err_warn_msg_buf,
-	       "Alert: Error (%d) sloppy-processing afm-file for Font %d!",
-	       i ,FontID);
-      T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_STATISTIC);
-      if ((pFontBase->pFontArray[FontID].pAFMData=
-	   T1_GenerateAFMFallbackInfo(FontID))==NULL){
+  /* Now try to load afm-structures from corresponding .afm-file (if
+     not suppressed by the user). */
+  if ((pFontBase->t1lib_flags & T1_NO_AFM)!=0) {
+    pFontBase->pFontArray[FontID].pAFMData = NULL;
+    T1_PrintLog( "T1_LoadFont()",
+		 "Suppressing AFM data handling on user request",
+		 T1LOG_STATISTIC);
+  }
+  else {
+    if ((i=openFontMetricsFile( FontID, 0))){
+      /* Try a fallback, opening sloppy: */
+      if ((i=openFontMetricsFile( FontID, 1))) {
 	sprintf( err_warn_msg_buf,
-		 "Ultimately failed to generate metrics information Font %d!",
-		 FontID);
-	T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_WARNING);
-	T1_PrintLog( "T1_LoadFont()", "Segmentation fault might result",T1LOG_WARNING);
-      }
-      else{
-	pFontBase->pFontArray[FontID].info_flags |=AFM_SELFGEN_SUCCESS;
-	T1_PrintLog( "T1_LoadFont()",
-		     "Generating AFM-information from fontfile successful!",
-		     T1LOG_STATISTIC);
+		 "Alert: Error (%d) sloppy-processing afm-file for Font %d!",
+		 i ,FontID);
+	T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_STATISTIC);
+	if ((pFontBase->pFontArray[FontID].pAFMData=
+	     T1_GenerateAFMFallbackInfo(FontID))==NULL){
+	  sprintf( err_warn_msg_buf,
+		   "Ultimately failed to generate metrics information Font %d!",
+		   FontID);
+	  T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_WARNING);
+	}
+	else {
+	  pFontBase->pFontArray[FontID].info_flags |=AFM_SELFGEN_SUCCESS;
+	  T1_PrintLog( "T1_LoadFont()",
+		       "Generating AFM-information from fontfile successful!",
+		       T1LOG_STATISTIC);
+	}
       }
+      else {
+	pFontBase->pFontArray[FontID].info_flags |=AFM_SLOPPY_SUCCESS;
+	sprintf( err_warn_msg_buf,
+		 "Alert: Limited afm-information for Font %d",FontID);
+	T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_STATISTIC);
+      } 
     }
-    else{
-      pFontBase->pFontArray[FontID].info_flags |=AFM_SLOPPY_SUCCESS;
-      sprintf( err_warn_msg_buf,
-	       "Alert: Limited afm-information for Font %d",FontID);
-      T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_STATISTIC);
+    else {
+      pFontBase->pFontArray[FontID].info_flags |=AFM_SUCCESS;
     }
   }
-  else
-    pFontBase->pFontArray[FontID].info_flags |=AFM_SUCCESS;
+  
   
   /* Now, set Encodingvector entry to default if the font's
      internal encoding is "StandardEncoding".
      */
   if (pFontBase->pFontArray[FontID].pType1Data->fontInfoP[ENCODING].value.data.arrayP
-      == StdEncArrayP){
+      == StdEncArrayP) {
     pFontBase->pFontArray[FontID].info_flags |=USES_STANDARD_ENCODING;
     pFontBase->pFontArray[FontID].pFontEnc=pFontBase->default_enc;
     sprintf( err_warn_msg_buf,
@@ -514,11 +528,12 @@ int T1_LoadFont( int FontID)
     sprintf( err_warn_msg_buf,
 	     "Font %d not reencoded to default",FontID);
     T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf, T1LOG_DEBUG);
+    pFontBase->pFontArray[FontID].pFontEnc = NULL;
   }
 
   
   /* If AFM-Info available we try to speed up some things: */
-  if (pFontBase->pFontArray[FontID].pAFMData != NULL){
+  if (pFontBase->pFontArray[FontID].pAFMData != NULL) {
     /* We have to fill the array that maps the current encodings' indices to the
        indices used in afm file */
     if ((pFontBase->pFontArray[FontID].pEncMap=
@@ -530,10 +545,15 @@ int T1_LoadFont( int FontID)
       T1_errno=T1ERR_ALLOC_MEM;
       return(-1);
     }
-    for (i=0; i<256; i++)
-      pFontBase->pFontArray[FontID].pEncMap[i]=-1;
     for (i=0; i<256; i++){
+      pFontBase->pFontArray[FontID].pEncMap[i]=-1;
       charname=T1_GetCharName( FontID, i);
+      if (strcmp( charname, ".notdef")==0) {
+	/* This is because not all AFM files have the .notdef, so we can't
+	   rely on it */
+	pFontBase->pFontArray[FontID].pEncMap[i]=-2;
+	continue;
+      }
       for ( j=0; j<pFontBase->pFontArray[FontID].pAFMData->numOfChars; j++){
 	if (strcmp( charname,
 		    pFontBase->pFontArray[FontID].pAFMData->cmi[j].name)==0){
@@ -542,21 +562,6 @@ int T1_LoadFont( int FontID)
       }
     }
     
-    /* We now introduce an "index map" which gives quickly access to what metric 
-       features a character provides. */
-    if ((pFontBase->pFontArray[FontID].pIndexMap= 
-	 (int *)malloc(256*sizeof(int)))==NULL){ 
-      sprintf( err_warn_msg_buf, "Error allocating memory for index map (FontID=%d)",
-	       FontID);
-      T1_PrintLog( "T1_LoadFont()", err_warn_msg_buf,
-		 T1LOG_WARNING);
-      T1_errno=T1ERR_ALLOC_MEM;
-      return(-1); 
-    }
-    for (i=0; i<256; i++){
-      pFontBase->pFontArray[FontID].pIndexMap[i]=-1;
-    }
-    
     /* We now create an encoding-specific kerning table which will speed up
        looking for kerning pairs! */
     /* First, get number of defined kerning pairs: */
@@ -583,21 +588,14 @@ int T1_LoadFont( int FontID)
 	  /* pair is not relevant in current encoding */
 	  continue;
 	}
-
 	/* Since we get here we have a relevant pair -->
 	   Put char1 in higher byte and char2 in LSB:
 	   */
 	kern_tbl[j].chars=(char1 << 8) | char2;
-
-	/* We save the index into metrics array */ 
-	if ( pFontBase->pFontArray[FontID].pIndexMap[char1] == -1 )
-	  pFontBase->pFontArray[FontID].pIndexMap[char1]=j;
-
+	
 	/* We only make use of horizontal kerning */
 	kern_tbl[j].hkern=pkd[i].xamt;
 	j++;
-	/* Set the kerning flag for this char1: */
-	pFontBase->pFontArray[FontID].pIndexMap[char1] |=KERN_FLAG;
       }
       /* Reset the remaining of the table in case there were
 	 irrelevant pairs: */
@@ -605,10 +603,18 @@ int T1_LoadFont( int FontID)
 	kern_tbl[j].chars=0;
 	kern_tbl[j].hkern=0;
       }
+      /* We now sort the kerning array with respect to char indices */
+      qsort( kern_tbl, (size_t) j, sizeof(METRICS_ENTRY),
+	     &cmp_METRICS_ENTRY );
     }
     else
       pFontBase->pFontArray[FontID].pKernMap=NULL;
-  } /* End of "if (AFM-info ..)" */
+  }
+  else { /* no AFM data */
+    pFontBase->pFontArray[FontID].pKernMap=NULL;
+    pFontBase->pFontArray[FontID].pEncMap=NULL;
+  }
+  /* End of "if (AFM-info ..)" */
   
   
   /* We have just loaded a physical font into memory, thus .... */
@@ -689,40 +695,79 @@ int T1_LoadFont( int FontID)
 }
   
 
-/* openFontMetricsFile( FontID): Gets the fontfilename corresponding to
-   FontID, opens the corresponding afm-file and fills the data structures.
-   return-value is the value returned by the T1lib_parseFile() function: */
-
-int openFontMetricsFile( int FontID)
+/* openFontMetricsFile( FontID, open_sloppy): Gets the fontfilename
+   corresponding to FontID, opens the corresponding afm-file and fills
+   the data structures. return-value is the value returned by the
+   T1lib_parseFile() function. If open_sloppy is set, the minimum
+   information needed is read from AFM file. This can be considered a
+   fallback for problematic AFM files. */
+static int openFontMetricsFile( int FontID, int open_sloppy)
 {
   char *FontFileName;
   char *AFMFileName;
+  char *afm_name;
   char *AFMFileNamePath;
   
-  int i;
+  int i, j;
   FILE *metricsfile;
   
 
-  FontFileName=T1_GetFontFileName( FontID);
-  i=strlen(FontFileName);
-  AFMFileName=(char *)malloc( i+1);
-  strcpy( AFMFileName, FontFileName);
-  /* We now change the extension pfa or pfb to afm: */
-  AFMFileName[i-3]='a';
-  AFMFileName[i-2]='f';
-  AFMFileName[i-1]='m';
-
-  /* Get full path of the afm file */
+  afm_name=T1_GetAfmFileName(FontID);
+  
+  if (afm_name!=NULL) { /* We have name explicitly specified */
+    /* It needs to be freeable */
+    if ((AFMFileName=
+	 (char *)malloc( (strlen(afm_name)+1)*sizeof( char)))==NULL) {
+      T1_errno=T1ERR_ALLOC_MEM;
+      return( -6);
+    }
+    strcpy( AFMFileName, afm_name);
+  }
+  else {
+    FontFileName=T1_GetFontFileName( FontID);
+    i=strlen(FontFileName);
+    j=i;
+    AFMFileName=(char *)malloc( i+5);
+    strcpy( AFMFileName, FontFileName);
+    while ( AFMFileName[i] != '.'){
+      if (i==0) break;
+      else i--;
+    }
+    if (i==0){
+      /* We have a filename without extension -> append extension */
+      AFMFileName[j]='.';
+      AFMFileName[j+1]='a';
+      AFMFileName[j+2]='f';
+      AFMFileName[j+3]='m';
+      AFMFileName[j+4]='\0';
+    }
+    else{
+      /* we found a '.' -> replace extension */
+      AFMFileName[i+1]='a';
+      AFMFileName[i+2]='f';
+      AFMFileName[i+3]='m';
+      AFMFileName[i+4]='\0';
+    }
+  }
+  
+  /* Get full path of the afm file (The case of a full path name
+     name specification is valid */
   AFMFileNamePath=Env_GetCompletePath( AFMFileName, T1_AFM_ptr);
   free( AFMFileName);
   
   /* open afm-file: */
-  if ((metricsfile=fopen(AFMFileNamePath,"r"))==NULL){
-    if (AFMFileNamePath!=NULL)
+  if (AFMFileNamePath!=NULL){
+    if ((metricsfile=fopen(AFMFileNamePath,"r"))==NULL){
       free(AFMFileNamePath);
-    return(-4);
+      return(-4);
+    }
+    else {
+      free(AFMFileNamePath);
+    }
+  }
+  else{
+    return( -5);
   }
-  free(AFMFileNamePath);
   
   /* Call procedure to read afm-file and store the data formatted.
      Flags used here: P_M  All Metrics Information
@@ -735,70 +780,32 @@ int openFontMetricsFile( int FontID)
      There's no way to make a serious decision whether an error has
      occured or not.
      */
-  i=T1lib_parseFile( (FILE *) metricsfile,
-		     (FontInfo **) &(FontBase.pFontArray[FontID].pAFMData),
-		     P_M | P_P );
+  if (open_sloppy)
+    i=T1lib_parseFile( (FILE *) metricsfile,
+		       (FontInfo **) &(FontBase.pFontArray[FontID].pAFMData),
+		       P_M );
+  else
+    i=T1lib_parseFile( (FILE *) metricsfile,
+		       (FontInfo **) &(FontBase.pFontArray[FontID].pAFMData),
+		       P_M | P_P );
   fclose(metricsfile);
   return(i);
 }
 
 
 
-/* openFontMetricsFileSloppy( FontID): Gets the fontfilename corresponding to
-   FontID, opens the corresponding afm-file and tries to fill the data
-   structures only with metric data. This can be considered a fallback!
-   */
-int openFontMetricsFileSloppy( int FontID)
-{
-  char *FontFileName;
-  char *AFMFileName;
-  char *AFMFileNamePath;
-  int i;
-  FILE *metricsfile;
-  
-
-  FontFileName=T1_GetFontFileName( FontID);
-  i=strlen(FontFileName);
-  AFMFileName=(char *)malloc( i+1);
-  strcpy( AFMFileName, FontFileName);
-  /* We now change the extension pfa or pfb to afm: */
-  AFMFileName[i-3]='a';
-  AFMFileName[i-2]='f';
-  AFMFileName[i-1]='m';
-
-  /* Get full path of the afm file */
-  AFMFileNamePath=Env_GetCompletePath( AFMFileName, T1_AFM_ptr);
-  free( AFMFileName);
-  
-  /* open afm-file: */
-  if ((metricsfile=fopen(AFMFileNamePath,"r"))==NULL){
-    /* patch #1 */
-    if (AFMFileNamePath!=NULL)
-      free(AFMFileNamePath);
-    return(-4);
-  }
-  free(AFMFileNamePath);
-  
-  /* Call procedure to read afm-file and store the data formatted.
-     Flags used here: P_M  All Metrics Information
-     */
-  i=T1lib_parseFile( (FILE *) metricsfile,
-	       (FontInfo **) &(FontBase.pFontArray[FontID].pAFMData),
-	       P_M );
-  fclose(metricsfile);
-  return(i);
-}
-
-
-
-
 /* CreateNewFontSize( FontID, size): Create a new size "size" of font
    "FontID" and allocate all data necessary for this. The data
    structure is connected to the linked list of FontSizeDeps for this
    font. Returns a pointer to the newly created FontSizeDeps-struct
    if all went correct and NULL otherwise.
    Since of version 0.3 a member antialias has been added to the
-   FONTSIZEDEPS structure!
+   FONTSIZEDEPS structure! This can be:
+
+   0:     bitmaps are stored in this struct
+   1:     non-antialiased bytemaps are stored in this struct
+   2:     low-antialiased bytemaps are stored in this struct
+   4:     high-antialiased bytemaps are stored in this struct
    */
 FONTSIZEDEPS *CreateNewFontSize( int FontID, float size, int aa)
 {
@@ -835,10 +842,7 @@ FONTSIZEDEPS *CreateNewFontSize( int FontID, float size, int aa)
   /* Put the size into this structure */
   pFontSizeDeps->size=size;
   /* Set the antialias mark: */
-  if (aa)
-    pFontSizeDeps->antialias=1;
-  else
-    pFontSizeDeps->antialias=0;
+  pFontSizeDeps->antialias=aa;
   
   /* Just the current becomes now the last item in the linked list: */
   pFontSizeDeps->pNextFontSizeDeps=NULL;
@@ -858,11 +862,6 @@ FONTSIZEDEPS *CreateNewFontSize( int FontID, float size, int aa)
 	      pFontBase->pFontArray[FontID].FontTransform[1],
 	      pFontBase->pFontArray[FontID].FontTransform[2],
 	      pFontBase->pFontArray[FontID].FontTransform[3]);
-  /* Apply device specific scaling */
-  pFontSizeDeps->pCharSpaceLocal=(struct XYspace *)
-    Scale(pFontSizeDeps->pCharSpaceLocal,
-	  DeviceSpecifics.scale_x,
-	  - DeviceSpecifics.scale_y);
   /* Apply desired scaling factor, and make it Permanent */
   pFontSizeDeps->pCharSpaceLocal=(struct XYspace *) Permanent
     (Scale(pFontSizeDeps->pCharSpaceLocal, size, size));
@@ -936,3 +935,17 @@ FONTSIZEDEPS *GetLastFontSize( int FontID)
   return((FONTSIZEDEPS *)(result_ptr));
 }
 
+
+/* A function for comparing METRICS_ENTRY structs */
+static int cmp_METRICS_ENTRY( const void *entry1, const void *entry2)
+{
+  if (((METRICS_ENTRY *)entry1)->chars <
+      ((METRICS_ENTRY *)entry2)->chars)
+    return(-1);
+  if (((METRICS_ENTRY *)entry1)->chars >
+      ((METRICS_ENTRY *)entry2)->chars)
+    return(1);
+  return(0); /* This should not happen */
+}
+    
+    
