@@ -65,6 +65,8 @@ static int edgemax();
 static int discard();
 static int edgecheck();
 static struct edgelist *NewEdge();
+struct edgelist *swathxsort();  /* 'SortSwath' function               */
+extern struct XYspace *IDENTITY;
  
 /*
 :h3.Functions Provided to the TYPE1IMAGER User
@@ -235,7 +237,7 @@ void KillRegion(area)
         register struct edgelist *next;  /* loop variable                    */
  
         if (area->references < 0)
-               abort("KillRegion:  negative reference count");
+               abort("KillRegion:  negative reference count", 28);
         if ( (--(area->references) > 1) ||
            ( (area->references == 1) && !ISPERMANENT(area->flag) ) )
             return;
@@ -255,7 +257,7 @@ struct region *CopyRegion(area)
         register struct region *area;  /* region to duplicate                */
 {
         register struct region *r;  /* output region built here              */
-        register struct edgelist *last;  /* loop variable                    */
+        register struct edgelist *last=NULL;  /* loop variable                    */
         register struct edgelist *p,*newp;  /* loop variables                */
  
         r = (struct region *)Allocate(sizeof(struct region), area, 0);
@@ -301,7 +303,7 @@ static struct edgelist *NewEdge(xmin, xmax, ymin, ymax, xvalues, isdown)
        IfTrace2((RegionDebug),"....new edge: ymin=%d, ymax=%d ",
                                               (LONG)ymin, (LONG) ymax);
        if (ymin >= ymax)
-               abort("newedge: height not positive");
+               abort("newedge: height not positive", 29);
 /*
 We are going to copy the xvalues into a newly allocated area.  It
 helps performance if the values are all "long" aligned.  We can test
@@ -309,7 +311,7 @@ if the xvalues are long aligned by ANDing the address with the
 (sizeof(long) - 1)--if non zero, the xvalues are not aligned well.  We
 set 'iy' to the ymin value that would give us good alignment:
 */
-       iy = ymin - (((int) xvalues) & (sizeof(LONG) - 1)) / sizeof(pel);
+       iy = ymin - (((unsigned long) xvalues) & (sizeof(LONG) - 1)) / sizeof(pel);
  
        r = (struct edgelist *)Allocate(sizeof(struct edgelist), &template,
                              (ymax - iy) * sizeof(pel));
@@ -400,11 +402,13 @@ user asked, >1: do it regardless).
 /*             if (fillrule != EVENODDRULE)
                else */
                        return((struct region *)UniquePath(p));
-       if (p->type == STROKEPATHTYPE)
-               if (fillrule == WINDINGRULE)
-                       return((struct region *)DoStroke(p));
-               else
-                       p = CoercePath(p);
+       if (p->type == STROKEPATHTYPE){
+	 if (fillrule == WINDINGRULE)
+	   return((struct region *)DoStroke(p));
+	 else
+	   p = CoercePath(p);
+       }
+       
  
        R = (struct region *)Allocate(sizeof(struct region), &EmptyRegion, 0);
  
@@ -562,7 +566,7 @@ MOVETYPE at the end of the path, if it isn't closed:
                        break;
  
                    default:
-                       abort("Interior: path type error");
+                       abort("Interior: path type error", 30);
                }
 /*
 We're done with this segment.  Advance to the next path segment in
@@ -598,7 +602,7 @@ an even number of edges; of course, we abort if we don't.
 static int Unwind(area)
        register struct edgelist *area;  /* input area modified in place      */
 {
-       register struct edgelist *last,*next;  /* struct before and after current one */
+       register struct edgelist *last=NULL,*next;  /* struct before and after current one */
        register int y;       /* ymin of current swath                        */
        register int count,newcount;  /* winding count registers              */
  
@@ -628,7 +632,7 @@ static int Unwind(area)
                } while (area != NULL && area->ymin == y);
  
                if (count != 0)
-                       abort("Unwind:  uneven edges");
+                       abort("Unwind:  uneven edges", 31);
        }
        /* We retunr a value for ANSI-C-compiler */
        return(0);
@@ -689,7 +693,7 @@ void ChangeDirection(type, R, x, y, dy)
                }
  
                if (ymax < ymin)
-                       abort("negative sized edge?");
+                       abort("negative sized edge?", 32);
  
  
                (*R->newedgefcn)(R, R->edgexmin, R->edgexmax, ymin, ymax,
@@ -750,7 +754,6 @@ static int newfilledge(R, xmin, xmax, ymin, ymax, isdown)
        fractpel ymin,ymax;   /* Y range of this edge                         */
        int isdown;           /* flag:  TRUE means edge goes down, else up    */
 {
-       struct edgelist *swathxsort();  /* 'SortSwath' function               */
  
        register pel pelxmin,pelymin,pelxmax,pelymax;  /* pel versions of bounds */
        register struct edgelist *edge;  /* newly created edge                */
@@ -922,7 +925,7 @@ static struct edgelist *splitedge(list, y)
        register pel y;       /* Y value to split list at                     */
 {
        register struct edgelist *new;  /* anchor for newly built list        */
-       register struct edgelist *last;  /* end of newly built list           */
+       register struct edgelist *last=NULL;  /* end of newly built list           */
        register struct edgelist *r;  /* temp pointer to new structure        */
        register struct edgelist *lastlist;  /* temp pointer to last 'list' value */
  
@@ -934,9 +937,9 @@ static struct edgelist *splitedge(list, y)
                if (y < list->ymin)
                        break;
                if (y >= list->ymax)
-                       abort("splitedge: above top of list");
+                       abort("splitedge: above top of list", 33);
                if (y == list->ymin)
-                       abort("splitedge: would be null");
+                       abort("splitedge: would be null", 34);
  
                r = (struct edgelist *)Allocate(sizeof(struct edgelist), list, 0);
 /*
@@ -976,7 +979,7 @@ list at 'lastlist', and add the broken off part to the end of 'new'.
 Then, we return the caller a pointer to 'new':
 */
        if (new == NULL)
-               abort("null splitedge");
+               abort("null splitedge", 35);
        lastlist->link = NULL;
        last->link = list;
        IfTrace1((RegionDebug > 1),"yields %p\n", new);
@@ -993,7 +996,7 @@ static int vertjoin(top, bottom)
        register struct edgelist *bottom;  /* bottommost region               */
 {
        if (BOTTOM(top) > TOP(bottom))
-               abort("vertjoin not disjoint");
+               abort("vertjoin not disjoint", 36);
  
        for (; top->link != NULL; top=top->link) { ; }
  
@@ -1017,7 +1020,7 @@ struct edgelist *swathxsort(before0, edge)
 {
        register struct edgelist *before;
        register struct edgelist *after;
-       register pel y;
+       register pel y=0;
  
        before = before0;
        after = before->link;
@@ -1101,7 +1104,7 @@ struct edgelist *SwathUnion(before0, edge)
  
        h0 = h = edge->ymax - edge->ymin;
        if (h <= 0)
-               abort("SwathUnion:  0 height swath?");
+               abort("SwathUnion:  0 height swath?", 37);
  
        before = before0;
        after = before->link;
@@ -1379,7 +1382,7 @@ static int discard(left, right)
  
        for (p = beg; p != right; p = p->link) {
                if (p->link == NULL && right != NULL)
-                       abort("discard():  ran off end");
+                       abort("discard():  ran off end", 38);
                IfTrace1((RegionDebug > 0),"discarding %p\n", p);
                p->ymin = p->ymax = 32767;
                end = p;
@@ -1469,7 +1472,7 @@ void UnJumble(region)
  
        for (edge=region->anchor; VALIDEDGE(edge); edge=next) {
                if (edge->link == NULL)
-                       abort("UnJumble:  unpaired edge?");
+                       abort("UnJumble:  unpaired edge?", 39);
                next = edge->link->link;
                edge->link->link = NULL;
                anchor = SortSwath(anchor, edge, t1_SwathUnion);
@@ -1509,9 +1512,9 @@ static int OptimizeRegion(R)
                if (xmin != xmax || (xmin != R->xmin && xmax != R->xmax))
                        R->flag &= ~ISRECTANGULAR(ON);
                if (xmin < e->xmin || xmax > e->xmax)
-                       abort("Tighten: existing edge bound was bad");
+                       abort("Tighten: existing edge bound was bad", 40);
                if (xmin < R->xmin || xmax > R->xmax)
-                       abort("Tighten: existing region bound was bad");
+                       abort("Tighten: existing region bound was bad", 41);
                e->xmin = xmin;
                e->xmax = xmax;
        }
@@ -1664,7 +1667,6 @@ struct region *CoerceRegion(tp)
 struct segment *RegionBounds(R)
        register struct region *R;
 {
-       extern struct XYspace *IDENTITY;
  
        register struct segment *path;  /* returned path                      */
  
@@ -1764,13 +1766,13 @@ static int edgecheck(edge, oldmin, oldmax)
        int oldmin,oldmax;
 {
        if (edge->type != EDGETYPE)
-               abort("EDGE ERROR: non EDGETYPE in list");
+               abort("EDGE ERROR: non EDGETYPE in list", 42);
 /*
 The following check is not valid if the region is jumbled so I took it
 out:
 */
 /*     if (edge->ymin < oldmax && edge->ymin != oldmin)
-               abort("EDGE ERROR: overlapping swaths"); */
+               abort("EDGE ERROR: overlapping swaths", 43); */
        return(0);
        
 }
