@@ -34,6 +34,7 @@
 #include <stdlib.h>
 
 #include "as274c.h"
+#include "utils.h"
 #include "protos.h"
 
 int includ(int np, int nrbar, double w, 
@@ -690,28 +691,6 @@ void putdvec(const char *s, double *x, int l, int h)
     }
 }
 
-static double r_mod(float x, float y)    /* used by f2c's version of rand() */
-{
-    double quotient = (double) x/y;
-    quotient = quotient >= 0 ? floor(quotient) : -floor(-quotient);
-    return x - y*quotient;
-}
-
-struct {
-    int ix, iy, iz;
-} randc;
-
-double ranwm()
-{
-    float r;
-    randc.ix = randc.ix * 171 % 30269;
-    randc.iy = randc.iy * 172 % 30307;
-    randc.iz = randc.iz * 170 % 30323;
-    r = (float) randc.ix / (float) 30269. + (float) randc.iy / (float)
-	    30307. + (float) randc.iz / (float) 30323.;
-    return r_mod(r, 1.0);
-}
-
 int dofitcurve(int cnt, double *xd, double *yd, int nd, double *c)
 {
     double **x, *y, *d, *rbar, *thetab, *xrow, sserr, *beta;
