@@ -38,19 +38,18 @@
 
 #include <Xm/Xm.h>
 #include <Xm/DialogS.h>
-#include <Xm/BulletinB.h>
-#include <Xm/Frame.h>
-#include <Xm/FileSB.h>
+#include <Xm/Form.h>
 #include <Xm/Command.h>
-#include <Xm/PushB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/Text.h>
-#include <Xm/Separator.h>
 #include <Xm/List.h>
 
 #include "globals.h"
-#include "protos.h"
+#include "graphs.h"
+#include "utils.h"
+#include "files.h"
 #include "motifinc.h"
+#include "protos.h"
 
 /*
  * Widget item declarations
@@ -76,9 +75,8 @@ void comwin_replace_act(Widget w, XKeyEvent *e, String *p, Cardinal *c);
 void comwin_up_act(Widget w, XKeyEvent *e, String *p, Cardinal *c);
 void create_rhist_popup(Widget w, XtPointer client_data, XtPointer call_data);
 void create_whist_frame(Widget w, XtPointer client_data, XtPointer call_data);
-void do_rhist_proc(Widget w, XtPointer client_data, XtPointer call_data);
-void exec_cmd(char *s);
-void open_command(Widget w, XtPointer client_data, XtPointer call_data);
+static int do_rhist_proc(char *filename, void *data);
+void open_command(void *data);
 
 extern XtAppContext app_con;
 
@@ -122,42 +120,16 @@ static char comwin_table[] = "#override\n\
     Shift ~Ctrl ~Meta ~Alt<Key>osfUp: comwin_up()\n\
     Shift ~Ctrl ~Meta ~Alt<Key>osfDown: comwin_down()";
 
-void exec_cmd(char *s)
-{
-    static int errpos, errorcount;
-    errorcount = errpos = 0;
-    
-    if (s == NULL) {
-	return;
-    }
-    
-    scanner(s, getx(cg, curset), gety(cg, curset),
-            getsetlength(cg, curset), ax, bx, cx, dx, MAXARR, 0, curset, 
-	    &errpos);
-    if (errpos) {
-	errorcount++;
-    }
-}
-
 static void comcall(Widget w, XtPointer cd, XtPointer calld)
 {
-    static int errpos, errorcount;
-    static char val[256];
     char *ts;
+    
     XmCommandCallbackStruct *s = (XmCommandCallbackStruct *) calld;
-    errorcount = errpos = 0;
-    XmStringGetLtoR(s->value, charset, &ts);
-    strcpy(val, ts);
+    ts = GetStringSimple(s->value);
+    scanner(ts);
     XtFree(ts);
-    scanner(val, getx(cg, curset), gety(cg, curset),
-	    getsetlength(cg, curset), ax, bx, cx, dx,
-	    MAXARR, 0, curset, &errpos);
-    if (errpos) {
-	errorcount++;
-    }
 }
 
-
 static void delete_com(Widget w, XtPointer client_data, XtPointer call_data)
 /* delete a entry from the history list */
 {
@@ -225,7 +197,7 @@ static void  add_com(Widget w, XtPointer client_data, XtPointer call_data)
     XmStringFree( comtxt );
 }
 
-static void  replace_com(Widget w, XtPointer client_data, XtPointer call_data)
+static void replace_com(Widget w, XtPointer client_data, XtPointer call_data)
 /*
  * replace a entry in the history list with the command line 
  * without executing it
@@ -263,14 +235,17 @@ static void clear_history(Widget w, XtPointer client_data, XtPointer call_data)
     }
 }
 
+#define MAXERR 5
 static void replay_history(Widget w, XtPointer client_data, XtPointer call_data)
 {
-    static int errpos, errcount;
-    char buf[256], *ts;
+    int errpos;
+    static int errcount;
+    char *ts;
     int i;
     int ac = 0, hc;
     XmStringTable xmstrs;
     Arg al[5];
+    
     ac = 0;
     XtSetArg(al[ac], XmNhistoryItems, &xmstrs);
     ac++;
@@ -279,18 +254,14 @@ static void replay_history(Widget w, XtPointer client_data, XtPointer call_data)
     XtGetValues(command, al, ac);
     errcount = 0;
     for (i = 0; i < hc; i++) {
-        XmStringGetLtoR(xmstrs[i], charset, &ts);
-        strcpy(buf, ts);
+        ts = GetStringSimple(xmstrs[i]);
+        errpos = scanner(ts);
         XtFree(ts);
 
-        errpos = 0;
-        scanner(buf, getx(cg, curset), gety(cg, curset), 
-	        getsetlength(cg, curset), ax, bx, cx, dx, MAXARR, 0, 
-		curset, &errpos);
         if (errpos) {
             errcount++;
         }
-        if (errcount > 3) {
+        if (errcount > MAXERR) {
             if (yesno("Lots of errors, cancel?", NULL, NULL, NULL)) {
             	break;
             } else {
@@ -300,7 +271,7 @@ static void replay_history(Widget w, XtPointer client_data, XtPointer call_data)
     }
 }
 
-void open_command(Widget w, XtPointer client_data, XtPointer call_data)
+void open_command(void *data)
 {
     Widget form, fr1, fr2, but[6];
     char *labrow1[5] = { "Add", "Delete", "Replace", "Up", "Down"};
@@ -308,35 +279,39 @@ void open_command(Widget w, XtPointer client_data, XtPointer call_data)
 			 "Help"};
     set_wait_cursor();
     if (command == NULL) {
-	comshell = XmCreateDialogShell(app_shell, "Commands", NULL, 0);
+	XmString str;
+        comshell = XmCreateDialogShell(app_shell, "Commands", NULL, 0);
 	handle_close(comshell);
 	command = XmCreateCommand(comshell, "command", NULL, 0);
-	hl = XmCommandGetChild( command, XmDIALOG_HISTORY_LIST );
-	XtVaSetValues(command,
-		      XmNpromptString, XmStringCreateLtoR("Command", charset),
-		      NULL);
-
+	hl = XmCommandGetChild(command, XmDIALOG_HISTORY_LIST);
+	str = XmStringCreateLocalized("Command");
+        XtVaSetValues(command, XmNpromptString, str, NULL);
+        XmStringFree(str);
+        
 	form = XmCreateForm(command, "commandform", NULL, 0);
+	XtVaSetValues(form, 
+	    XmNtopAttachment, XmATTACH_WIDGET,
+            XmNtopWidget, hl,
+	    XmNleftAttachment, XmATTACH_FORM,
+	    XmNrightAttachment, XmATTACH_FORM,
+	    XmNbottomAttachment, XmATTACH_WIDGET,
+            XmNbottomWidget, XmCommandGetChild(command, XmDIALOG_PROMPT_LABEL),
+	    NULL);
 
-	fr1 = XmCreateFrame(form, "commandframe1", NULL, 0);
+	fr1 = CreateFrame(form, NULL);
 	XtVaSetValues(fr1, 
 	    XmNtopAttachment, XmATTACH_FORM,
 	    XmNleftAttachment, XmATTACH_FORM,
 	    XmNrightAttachment, XmATTACH_FORM,
 	    NULL);
 	CreateCommandButtonsNoDefault(fr1, 5, but, labrow1);
-	XtAddCallback(but[0], XmNactivateCallback, 
-	              (XtCallbackProc) add_com , (XtPointer) NULL);
-	XtAddCallback(but[1], XmNactivateCallback, 
-	              (XtCallbackProc) delete_com , (XtPointer) NULL);
-	XtAddCallback(but[2], XmNactivateCallback, 
-	              (XtCallbackProc) replace_com , (XtPointer) NULL);
-	XtAddCallback(but[3], XmNactivateCallback, (XtCallbackProc) move_com ,
-			(XtPointer) 0);
-	XtAddCallback(but[4], XmNactivateCallback, 
-	              (XtCallbackProc) move_com, (XtPointer) 1);
+	XtAddCallback(but[0], XmNactivateCallback, add_com, NULL);
+	XtAddCallback(but[1], XmNactivateCallback, delete_com, NULL);
+	XtAddCallback(but[2], XmNactivateCallback, replace_com, NULL);
+	XtAddCallback(but[3], XmNactivateCallback, move_com, (XtPointer) 0);
+	XtAddCallback(but[4], XmNactivateCallback, move_com, (XtPointer) 1);
 
-	fr2 = XmCreateFrame(form, "commandframe2", NULL, 0);
+	fr2 = CreateFrame(form, NULL);
 	XtVaSetValues(fr2, 
 	    XmNtopAttachment, XmATTACH_WIDGET,
 	    XmNtopWidget, fr1,
@@ -345,24 +320,14 @@ void open_command(Widget w, XtPointer client_data, XtPointer call_data)
 	    XmNbottomAttachment, XmATTACH_FORM,
 	    NULL);
 	CreateCommandButtonsNoDefault(fr2, 6, but, labrow2);
-	XtAddCallback(but[0], XmNactivateCallback, 
-	              (XtCallbackProc) create_rhist_popup, (XtPointer) NULL);
-	XtAddCallback(but[1], XmNactivateCallback, 
-	              (XtCallbackProc) create_whist_frame, (XtPointer) NULL);
-	XtAddCallback(but[2], XmNactivateCallback, 
-	              (XtCallbackProc) clear_history, (XtPointer) NULL);
-	XtAddCallback(but[3], XmNactivateCallback, 
-	              (XtCallbackProc) replay_history, (XtPointer) NULL);
-	XtAddCallback(but[4], XmNactivateCallback, 
-	              (XtCallbackProc) destroy_dialog, (XtPointer) comshell);
-	XtAddCallback(but[5], XmNactivateCallback, 
-	              (XtCallbackProc) HelpCB, (XtPointer) "data.html#commands");
+	XtAddCallback(but[0], XmNactivateCallback, create_rhist_popup, NULL);
+	XtAddCallback(but[1], XmNactivateCallback, create_whist_frame, NULL);
+	XtAddCallback(but[2], XmNactivateCallback, clear_history, NULL);
+	XtAddCallback(but[3], XmNactivateCallback, replay_history, NULL);
+	XtAddCallback(but[4], XmNactivateCallback, destroy_dialog, comshell);
+	AddButtonCB(but[5], HelpCB, NULL);
 
-
-	XtAddCallback(command, XmNcommandEnteredCallback, 
-	              (XtCallbackProc) comcall, (XtPointer) NULL);
-	XtManageChild(fr1);
-	XtManageChild(fr2);
+	XtAddCallback(command, XmNcommandEnteredCallback, comcall, NULL);
 	XtManageChild(form);
 	XtManageChild(command);
 	XtManageChild(comshell);
@@ -381,54 +346,46 @@ void close_rhist_popup(Widget w, XtPointer client_data, XtPointer call_data)
     XtUnmanageChild(rhist_dialog);
 }
 
-void do_rhist_proc(Widget w, XtPointer client_data, XtPointer call_data)
+static int do_rhist_proc(char *filename, void *data)
 {
     char buf[512];
-    char *s;
     int sl;
     FILE *fp;
     XmString list_item;
     Widget h = XmCommandGetChild(command, XmDIALOG_HISTORY_LIST);
-    XmFileSelectionBoxCallbackStruct *cbs =
-        (XmFileSelectionBoxCallbackStruct *) call_data;
-    if (!XmStringGetLtoR(cbs->value, charset, &s)) {
-        errwin("Error converting XmString to char string in rdata_proc()");
-        return;
-    }
 
-    strcpy(buf, s);
-    XtFree(s);
-    if ((fp = filter_read(buf)) != NULL) {
-		while (fgets(buf, 255, fp) != NULL) {
-			sl = strlen(buf);
-			buf[sl - 1] = 0;
-			if (strlen(buf) == 0) {
-				continue;
-			}
-			list_item = XmStringCreateLtoR(buf, charset);
-			XmListAddItemUnselected(h, list_item, 0);
-			XmStringFree(list_item);
-		}
-		filter_close(fp);
+    if ((fp = grace_openr(filename, SOURCE_DISK)) != NULL) {
+	while (fgets(buf, 255, fp) != NULL) {
+	    sl = strlen(buf);
+	    buf[sl - 1] = 0;
+	    if (strlen(buf) == 0) {
+                continue;
+	    }
+	    list_item = XmStringCreateLocalized(buf);
+	    XmListAddItemUnselected(h, list_item, 0);
+	    XmStringFree(list_item);
+	}
+	grace_close(fp);
+        return TRUE;
     } else {
-		errwin("Unable to open file");
+        return FALSE;
     }
-    XtUnmanageChild(rhist_dialog);
 }
 
 void create_rhist_popup(Widget w, XtPointer client_data, XtPointer call_data)
 {
+    static FSBStructure *fsb = NULL;
+
     set_wait_cursor();
-    if (rhist_dialog == NULL) {
-	rhist_dialog = XmCreateFileSelectionDialog(app_shell, "Read history", 
-	                                           NULL, 0);
-	XtAddCallback(rhist_dialog, XmNcancelCallback, 
-	              (XtCallbackProc) close_rhist_popup, (XtPointer) NULL);
-	XtAddCallback(rhist_dialog, XmNokCallback, 
-	              (XtCallbackProc) do_rhist_proc, (XtPointer) NULL);
-	XtManageChild(rhist_dialog);
+
+    if (fsb == NULL) {
+        fsb = CreateFileSelectionBox(app_shell, "Read history", "*.com");
+	AddFileSelectionBoxCB(fsb, do_rhist_proc, NULL);
+        XtManageChild(fsb->FSB);
     }
-    XtRaise(rhist_dialog);
+    
+    XtRaise(fsb->dialog);
+
     unset_wait_cursor();
 }
 
@@ -456,7 +413,7 @@ void create_whist_frame(Widget w, XtPointer client_data, XtPointer call_data)
 
 	whist_text_item = CreateTextItem2(whist_panel, 30, "Write history to:");
 
-	XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass, whist_panel, NULL);
+	CreateSeparator(whist_panel);
 
 	CreateCommandButtons(whist_panel, 2, buts, label1);
 	XtAddCallback(buts[0], XmNactivateCallback,
@@ -472,28 +429,25 @@ void create_whist_frame(Widget w, XtPointer client_data, XtPointer call_data)
 static void whist_apply_notify_proc(Widget w, XtPointer client_data, XtPointer call_data)
 {
     int i, ac = 0, hc;
-    char s[256], *ts;
+    char *ts;
     XmStringTable xmstrs;
     Arg al[5];
-    strcpy(s, (char *) xv_getstr(whist_text_item));
-    if (!fexists(s)) {
-	FILE *pp = filter_write(s);
-	if (pp != NULL) {
-	    ac = 0;
-	    XtSetArg(al[ac], XmNhistoryItems, &xmstrs);
-	    ac++;
-	    XtSetArg(al[ac], XmNhistoryItemCount, &hc);
-	    ac++;
-	    XtGetValues(command, al, ac);
-	    for (i = 0; i < hc; i++) {
-	    	XmStringGetLtoR(xmstrs[i], charset, &ts);
-	    	fprintf(pp, "%s\n", ts);
-	    	XtFree(ts);
-	    }
-	    filter_close(pp);
-	} else {
-	    errwin("Unable to open file");
-	}
+    FILE *pp;
+    
+    pp = grace_openw(xv_getstr(whist_text_item));
+    if (pp != NULL) {
+        ac = 0;
+        XtSetArg(al[ac], XmNhistoryItems, &xmstrs);
+        ac++;
+        XtSetArg(al[ac], XmNhistoryItemCount, &hc);
+        ac++;
+        XtGetValues(command, al, ac);
+        for (i = 0; i < hc; i++) {
+            ts = GetStringSimple(xmstrs[i]);
+            fprintf(pp, "%s\n", ts);
+            XtFree(ts);
+        }
+        grace_close(pp);
     }
     XtUnmanageChild(whist_frame);
 }
