@@ -48,42 +48,26 @@
 #define MAX_STRING_LENGTH 512
 
 
-#define MAXGRAPH 10             /* max number of graphs */
-#define MAXPLOT 30              /* max number of sets in a graph */
-#define MAX_SET_COLS 6          /* max number of data columns for a set */
-#define MAXREGION 5             /* max number of regions */
 #define MAXAXES 4               /* max number of axes per graph */
 #define MAX_TICKS 100           /* max number of ticks/labels per axis */
-
-/* max number of different objects */
-#define MAXLINES 50             /* max number of lines */
-#define MAXBOXES 50             /* max number of boxes */
-#define MAXELLIPSES 50          /* max number of ellipses */
-#define MAXSTR 100              /* max number of strings */
-
-#define MAX_LINEWIDTH 10        /* max width of drawn lines */
+#define MAXREGION 5             /* max number of regions */
 
 #define MAX_ZOOM_STACK 20       /* max stack depth for world stack */
+
 #define MAXPARM 10              /* max number of parameters for non-lin fit */
 
 #define MAXFIT 12               /* max degree of polynomial+1 that can be
                                  * fitted */
 
 
+/* number of extra objects of a given type to allocate if not enough */
+#define OBJECT_BUFNUM 10
+
+
 #define MAX_JUST 2
 #define MAX_ARROW 3
 #define MAX_PREC 10
 
-
-#ifndef MAXARR
-#  define MAXARR 20000          /* max elements in an array */
-#endif
-
-#ifndef MAXPICKDIST
-#  define MAXPICKDIST 0.1       /* the maximum distance away from an object */
-#endif                          /* you may be when picking it (in viewport  */
-                                /* coordinates)                             */  
-
 /* symbol types */
 
 #define SYM_NONE    0
@@ -144,10 +128,11 @@
 #define ZX_AXIS 2
 #define ZY_AXIS 3
 
-/*
- * setno == all sets selected
- */
+
+/* setno == all sets selected */
 #define ALL_SETS    -1
+/* setno == new set to be created */
+#define NEW_SET     -2
 
 /*
  * gno == all graphs selected
@@ -159,44 +144,12 @@
 #define SPLINE_CUBIC    1
 #define SPLINE_AKIMA    2
 
-/* Orientation and page size */
+/* Canvas types */
 #define PAGE_FREE       0
 #define PAGE_FIXED      1
 
-/* Graph type */
-#define GRAPH_XY        0
-#define GRAPH_CHART     1
-#define GRAPH_POLAR     2
-#define GRAPH_SMITH     3
-
-/* Set types */
-#define SET_XY          0
-#define SET_BAR         1
-#define SET_XYZ         2
-#define SET_XYDX        3
-#define SET_XYDY        4
-#define SET_XYDXDX      5
-#define SET_XYDYDY      6
-#define SET_XYDXDY      7
-#define SET_XYHILO      8
-#define SET_XYRT        9
-#define SET_NXY        10
-#define SET_POLY       11
-#define SET_BLOCK      12
-#define SET_XYSTRING   13
-
-
-/* Data column names; */
-#define DATA_INDEX      0 /* reserved */
-#define DATA_X          1
-#define DATA_Y          2
-#define DATA_Y1         3
-#define DATA_Y2         4
-#define DATA_Y3         5
-#define DATA_Y4         6
-
-
 /* Strings and things */
+#define OBJECT_NONE    -1
 #define OBJECT_LINE     0
 #define OBJECT_BOX      1
 #define OBJECT_ELLIPSE  2
@@ -218,22 +171,17 @@
 #define LAYOUT_PARALLEL         0
 #define LAYOUT_PERPENDICULAR    1
 
-/* Axis & tick placement */
-#define PLACE_LEFT      0
-#define PLACE_RIGHT     1
-#define PLACE_TOP       2
-#define PLACE_BOTTOM    3
-#define PLACE_BOTH      4
+/* Placement (axis labels, ticks, error bars */
+typedef enum {
+    PLACEMENT_NORMAL,
+    PLACEMENT_OPPOSITE,
+    PLACEMENT_BOTH
+} PlacementType;
 
 /* Tick label placement */
 #define LABEL_ONTICK    0
 #define LABEL_BETWEEN   1
 
-/* Objects to display in status window */
-#define STATUS_SETS     0
-#define STATUS_GRAPHS   1
-#define STATUS_REGIONS  2
-
 /* Coordinates */
 #define COORD_VIEW      0
 #define COORD_WORLD     1
@@ -295,7 +243,6 @@
 /* Data source type */
 #define SOURCE_DISK     0
 #define SOURCE_PIPE     1
-#define SOURCE_STDIN    2
 
 /* Justifications */
 #define JUST_LEFT       0
@@ -332,6 +279,12 @@
 #  define FALSE 0
 #endif
 
+/* types of autscales */
+#define AUTOSCALE_NONE    0
+#define AUTOSCALE_X       1
+#define AUTOSCALE_Y       2
+#define AUTOSCALE_XY      3
+
 /*
  * for set selector gadgets
  */
@@ -351,108 +304,6 @@
 #define SELECTION_TYPE_SINGLE 0
 #define SELECTION_TYPE_MULTIPLE 1
 
-/* for canvas event proc */
-#define ZOOM_1ST 1
-#define ZOOM_2ND 2
-#define VIEW_1ST 3
-#define VIEW_2ND 4
-#define STR_LOC 5
-#define LEG_LOC 6
-#define FIND_POINT 7
-#define DEL_POINT 8
-#define MOVE_POINT1ST 9
-#define MOVE_POINT2ND 10
-#define ADD_POINT 11
-#define DEL_OBJECT 12
-#define MOVE_OBJECT_1ST 13
-#define MOVE_OBJECT_2ND 14
-#define MAKE_BOX_1ST 15
-#define MAKE_BOX_2ND 16
-#define MAKE_LINE_1ST 17
-#define MAKE_LINE_2ND 18
-#define MAKE_CIRC_1ST 19
-#define MAKE_CIRC_2ND 20
-#define MAKE_ARC_1ST 21
-#define MAKE_ARC_2ND 22
-#define MAKE_ELLIP_1ST 23
-#define MAKE_ELLIP_2ND 24
-#define SEL_POINT 25
-#define STR_EDIT 26
-#define COMP_AREA 27
-#define COMP_PERIMETER 28
-#define STR_LOC1ST 29
-#define STR_LOC2ND 30
-#define GRAPH_FOCUS 31
-#define TRACKER 32
-#define INTERP_GRID 33
-#define FIND_GRID 34
-#define FIND_GRIDPT 35
-#define FIND_GRIDRECT 36
-#define SLICE_GRIDDIST 37
-#define SLICE_GRIDX 38
-#define SLICE_GRIDY 39
-#define SLICE_GRIDLINE_1ST 40
-#define SLICE_GRIDLINE_2ND 41
-#define SLICE_GRIDPATH 42
-#define DEF_REGION 43
-#define DEF_REGION1ST 44
-#define DEF_REGION2ND 45
-#define PAINT_POINTS 46
-#define KILL_NEAREST 47
-#define COPY_NEAREST1ST 48
-#define COPY_NEAREST2ND 49
-#define MOVE_NEAREST1ST 50
-#define MOVE_NEAREST2ND 51
-#define REVERSE_NEAREST 52
-#define JOIN_NEAREST1ST 53
-#define JOIN_NEAREST2ND 54
-#define DEACTIVATE_NEAREST 55
-#define EXTRACT_NEAREST1ST 56
-#define EXTRACT_NEAREST2ND 57
-#define DELETE_NEAREST1ST 58
-#define DELETE_NEAREST2ND 59
-#define INSERT_POINTS 60
-#define INSERT_SET 61
-#define EDIT_OBJECT 62
-#define PLACE_TIMESTAMP 63
-#define COPY_OBJECT1ST 64
-#define COPY_OBJECT2ND 65
-#define CUT_OBJECT 66
-#define PASTE_OBJECT 67
-#define AUTO_NEAREST 68
-#define ZOOMX_1ST 69
-#define ZOOMX_2ND 70
-#define ZOOMY_1ST 71
-#define ZOOMY_2ND 72
-#define PICK_SET 73
-#define PICK_SET1 74
-#define PICK_SET2 75
-#define PICK_EXPR 76
-#define PICK_HISTO 77
-#define PICK_FOURIER 78
-#define PICK_RUNAVG 79
-#define PICK_RUNSTD 80
-#define PICK_RUNMIN 81
-#define PICK_RUNMAX 82
-#define PICK_DIFF 83
-#define PICK_INT 84
-#define PICK_REG 85
-#define PICK_XCOR 86
-#define PICK_SAMP 87
-#define PICK_PRUNE 88
-#define PICK_FILTER 89
-#define PICK_EXPR2 90
-#define PICK_SPLINE 91
-#define PICK_INTERP 92
-#define PICK_SAMPLE 93
-#define PICK_SEASONAL 94
-#define PICK_BREAK 95
-#define ADD_POINT1ST 96
-#define ADD_POINT2ND 97
-#define ADD_POINT3RD 98
-#define ADD_POINT_INTERIOR 99
-#define DISLINE1ST 100
-#define DISLINE2ND 101
 
 /* for stufftext() in monwin.c used here and there */
 #define STUFF_TEXT  0
@@ -511,21 +362,24 @@
 #define AVALUE_TYPE_Y           2
 #define AVALUE_TYPE_XY          3
 #define AVALUE_TYPE_STRING      4
+#define AVALUE_TYPE_Z           5
 
 /* ticks */
 #define TICK_TYPE_MAJOR     0
 #define TICK_TYPE_MINOR     1
 
-/* nonlprefs.load possible values */
-#define LOAD_VALUES         0
-#define LOAD_RESIDUALS      1
-#define LOAD_FUNCTION       2
+/* push set direction */
+#define PUSH_SET_TOFRONT    0
+#define PUSH_SET_TOBACK     1
 
-
-/* toolbars */
-#define BAR_TOOLBAR     0
-#define BAR_LOCATORBAR  1
-#define BAR_STATUSBAR   2
+/* restriction types */
+#define RESTRICT_NONE  -1
+#define RESTRICT_WORLD -2
+#define RESTRICT_REG0   0
+#define RESTRICT_REG1   1
+#define RESTRICT_REG2   2
+#define RESTRICT_REG3   3
+#define RESTRICT_REG4   4
 
 
 /*
@@ -545,7 +399,7 @@ typedef struct {
     int bgcolor;
     int pattern;
     int lines;
-    int linew;
+    double linew;
     double charsize;
     int font;
     double symsize;
@@ -559,17 +413,6 @@ typedef struct {
  */
 } Pen;
 
-/* Current drawing properties */
-typedef struct {
-    Pen pen;
-    int bgcolor;
-    int lines;
-    int linew;
-    double charsize;
-    int font;
-    int fillrule;
-} DrawProps;
-
 /* A point in world coordinates */
 typedef struct {
     double x;
@@ -588,6 +431,15 @@ typedef struct {
     double y;
 } VVector;
 
+typedef struct {
+    double xg1, xg2, yg1, yg2;  /* window into world coords */
+} world;
+
+typedef struct {
+    double xv1, xv2, yv1, yv2;  /* viewport */
+} view;
+
+
 /*
  * typedefs for objects
  */
@@ -600,12 +452,20 @@ typedef struct {
     double x2;
     double y2;
     int lines;
-    int linew;
+    double linew;
     int color;
     int fillcolor;
     int fillpattern;
+    view bb;
 } boxtype;
 
+typedef struct {
+    int type;
+    double length;  /* head length (L) */
+    double dL_ff;   /* d/L form factor */
+    double lL_ff;   /* l/L form factor */
+} Arrow;
+
 typedef struct {
     int active;
     int loctype;
@@ -615,11 +475,11 @@ typedef struct {
     double x2;
     double y2;
     int lines;
-    int linew;
+    double linew;
     int color;
-    int arrow;
-    int atype;
-    double asize;
+    int arrow_end;
+    Arrow arrow;
+    view bb;
 } linetype;
 
 typedef struct {
@@ -631,10 +491,11 @@ typedef struct {
     double x2;
     double y2;
     int lines;
-    int linew;
+    double linew;
     int color;
     int fillcolor;
     int fillpattern;
+    view bb;
 } ellipsetype;
 
 typedef struct {
@@ -649,17 +510,10 @@ typedef struct {
     int just;
     double charsize;
     char *s;
+    view bb;
 } plotstr;
 
 
-typedef struct {
-    double xg1, xg2, yg1, yg2;  /* window into world coords */
-} world;
-
-typedef struct {
-    double xv1, xv2, yv1, yv2;  /* device viewport */
-} view;
-
 /*
  * world stack
  */
@@ -694,12 +548,15 @@ typedef struct {
 
 typedef struct {
     int active;          /* on/off */
-    int type;            /* type of error bar */
-    int linew;           /* error bar line width */
+    PlacementType ptype; /* placement type */
+    Pen pen;             /* pen */
+    double linew;        /* error bar line width */
     int lines;           /* error bar line style */
-    int riser_linew;     /* connecting line between error limits line width */
+    double riser_linew;  /* connecting line between error limits line width */
     int riser_lines;     /* connecting line between error limits line style */
-    double length;       /* length of error bar */
+    double barsize;      /* size of error bar */
+    int arrow_clip;      /* draw arrows if clipped */
+    double cliplen;      /* riser clipped length (v.p.) */
 } Errbar;
 
 /* Annotative strings for data values */
@@ -717,66 +574,7 @@ typedef struct {
     VPoint offset;              /* offset related to symbol position */
 } AValue;
 
-typedef struct {
-    int active;                 /* active flag */
-    int type;                   /* dataset type */
-    int deact;                  /* deactivated set */
-    int len;                    /* set length */
-    int nx, ny;                 /* number of pts in X and Y for grids */
-    int setno;                  /* set number */
-    int gno;                    /* graph number */
-    double *ex[MAX_SET_COLS];   /* x, y, dx, z, r, hi depending on dataset type */
-    char **s;                   /* pointer to strings */
 
-    int hotlink;                /* hot linked set */
-    int hotsrc;                 /* source for hot linked file (DISK|PIPE) */
-    char hotfile[GR_MAXPATHLEN];   /* hot linked filename */
-
-    int sym;                    /* set plot symbol type */
-    double symsize;             /* size of symbols */
-    Pen sympen;                 /* pen props of symbol line */
-    Pen symfillpen;             /* pen props of symbol filling */
-    int symlines;               /* Symbol linestyle */
-    int symlinew;               /* Symbol linewidth */
-    int symskip;                /* How many symbols to skip */
-    int symdot;                 /* Symbol dot in center */
-    char symchar;               /* char used if sym == SYM_CHAR */
-    int charfont;               /* font for symchar if sym == SYM_CHAR */
-
-    AValue avalue;              /* Parameters for annotative string */
-    
-    int linet;                  /* set line type */
-    int lines;                  /* set line style */
-    int linew;                  /* line width */
-    Pen linepen;                /* pen for connecting line */
-
-    int baseline_type;          /* type of baseline */
-    int baseline;               /* should the baseline be drawn */
-    int dropline;               /* should the drop lines (from data points to
-                                                      the baseline) be drawn */
-    
-    int filltype;               /* fill type */
-    int fillrule;               /* fill rule (winding/even-odd) */
-    Pen setfillpen;             /* pen props for set fill */
-
-    char comments[MAX_STRING_LENGTH];   /* how did this set originate */
-    char lstr[MAX_STRING_LENGTH];       /* legend for this set */
-
-    Errbar errbar;              /* error bar properties */
-
-    double xmin, xmax;          /* min max for x */
-    double ymin, ymax;          /* min max for y */
-
-    double emin[MAX_SET_COLS];  /* min for each column */
-    double emax[MAX_SET_COLS];  /* max for each column */
-    int imin[MAX_SET_COLS];     /* min loc for each column */
-    int imax[MAX_SET_COLS];     /* max loc for each column */
-
-    Regression *r;              /* coefs from any regression performed on this set */
-    Spline *spl;                /* coefs from any spline performed on this set */
-
-    void *ep;                   /* pointer to EditPoints structure */
-} plotarr;
 
 typedef struct {
     int type;
@@ -787,7 +585,7 @@ typedef struct {
 typedef struct {
     double size;              /* length of tickmarks */
     int color;                /* color of tickmarks */
-    int linew;                /* linewidth of tickmarks */
+    double linew;             /* linewidth of tickmarks */
     int lines;                /* linestyle of tickmarks */
     int gridflag;             /* grid lines at tick marks */
 } tickprops;
@@ -800,12 +598,12 @@ typedef struct {
     plotstr label;              /* graph axis label */
     int label_layout;           /* axis label orientation (h or v) */
     int label_place;            /* axis label placement (specfied or auto) */
-    int label_op;               /* tick labels on opposite side or both */
+    PlacementType label_op;     /* tick labels on opposite side or both */
 
     int t_drawbar;              /* draw a bar connecting tick marks */
     int t_drawbarcolor;         /* color of bar */
     int t_drawbarlines;         /* linestyle of bar */
-    int t_drawbarlinew;         /* line width of bar */
+    double t_drawbarlinew;         /* line width of bar */
 
     double offsx, offsy;        /* offset of axes in viewport coords
                                    (attention: these
@@ -825,7 +623,7 @@ typedef struct {
     tickloc tloc[MAX_TICKS];    /* locations of ticks */
 
     int t_inout;                /* ticks inward, outward or both */
-    int t_op;                   /* ticks on opposite side */
+    PlacementType t_op;         /* ticks on opposite side */
     
     tickprops props;
     tickprops mprops;
@@ -845,7 +643,7 @@ typedef struct {
     double tl_start;            /* value of x to begin tick labels and major ticks */
     double tl_stop;             /* value of x to end tick labels and major ticks */
 
-    int tl_op;                  /* tick labels on opposite side or both */
+    PlacementType tl_op;        /* tick labels on opposite side or both */
 
     int tl_gaptype;             /* tick label placement auto or specified */
     VVector tl_gap;             /* tick label to tickmark distance
@@ -867,7 +665,8 @@ typedef struct {
     int hgap;                   /* horizontal gap(s) between legend string
                                                                   elements */
     int len;                    /* length of line to draw */
-    int box;                    /* box around legend on or off */
+    int invert;                 /* switch between ascending and descending
+                                   order of set legends */
     double legx;                /* location on graph */
     double legy;
     int font;
@@ -875,8 +674,9 @@ typedef struct {
     int color;
     Pen boxpen;
     Pen boxfillpen;
-    int boxlinew;               /* legend frame line width */
+    double boxlinew;            /* legend frame line width */
     int boxlines;               /* legend frame line style */
+    view bb;
 } legend;
 
 typedef struct {
@@ -884,7 +684,7 @@ typedef struct {
     int type;                   /* region type */
     int color;                  /* region color */
     int lines;                  /* region linestyle */
-    int linew;                  /* region line width */
+    double linew;               /* region line width */
     int *linkto;                /* associated with graphs in linkto */
     int n;                      /* number of points if type is POLY */
     double *x, *y;              /* coordinates if type is POLY */
@@ -895,58 +695,11 @@ typedef struct {
     int type;                   /* frame type */
     Pen pen;                    /* frame pen */
     int lines;                  /* frame linestyle */
-    int linew;                  /* frame line width */
+    double linew;                  /* frame line width */
     Pen fillpen;                /* fill pen */
 } framep;
 
 
-/*
- * a graph
- */
-typedef struct {
-    int active;                 /* alive or dead */
-    int hidden;                 /* display or not */
-    int type;                   /* type of graph */
-    int maxplot;                /* max number of sets for this graph */
-
-    int auto_type;              /* TODO: property of axis! */
-    int autoscale;              /* TODO: remove */
-    int noautoscale;            /* TODO: remove */
-    int parmsread;              /* TODO: remove */
-
-    int xscale;                 /* scale mapping of X axes*/
-    int yscale;                 /* scale mapping of Y axes*/
-    int xinvert;                /* X axes inverted, TRUE or FALSE */
-    int yinvert;                /* Y axes inverted, TRUE or FALSE */
-    int xyflip;                 /* whether x and y axes should be flipped */
-
-    int stacked;                /* TRUE if graph is stacked; TODO: a property
-                                   of dataset, not graph!!! */
-
-    plotarr *p;                 /* sets go here */
-
-    legend l;                   /* legends */
-
-    world w;                    /* world */
-    view v;                     /* world/view */
-
-    labels labs;                /* title, subtitle, axes labels */
-
-    tickmarks t[MAXAXES];       /* flags etc. for tickmarks for all axes */
-
-    framep f;                   /* type of box around plot */
-
-    int pointset;               /* if (dsx, dsy) have been set */
-    int pt_type;                /* type of locator display */
-    double dsx, dsy;            /* locator fixed point */
-    int fx, fy;                 /* locator format type */
-    int px, py;                 /* locator precision */
-
-    world_stack ws[MAX_ZOOM_STACK]; /* zoom stack */
-    int ws_top;                 /* stack pointer */
-    int curw;                   /* for cycling through the stack */
-} graph;
-
 /* parameters for non-linear fit */
 typedef struct {
     double value;       /* parameter itself */
@@ -963,68 +716,38 @@ typedef struct {
     double tolerance;   /* tolerance */
 } nonlopts;
 
-/* prefs for non-linear fit */
-typedef struct {
-    int autoload;       /* do autoload */
-    int load;           /* load to... */
-    int npoints;        /* # of points to evaluate function at */
-    double start;       /* start... */
-    double stop;        /* stop ... */
-} nonlprefs;
+/* real time inputs */
+typedef struct _Input_buffer {
+    int           fd;     /* file descriptor */
+    int           errors; /* number of successive parse errors */
+    int           lineno; /* line number */
+    int           zeros;  /* number of successive reads of zero byte */
+    int           reopen; /* non-zero if we should close and reopen */
+                          /* when other side is closed (mainly for fifos) */
+    char         *name;   /* name of the input (filename or symbolic name) */
+    int           size;   /* size of the buffer for already read lines */
+    int           used;   /* number of bytes used in the buffer */
+    char         *buf;    /* buffer for already read lines */
+    unsigned long id;     /* id for X library */
+} Input_buffer;
 
+/* dates formats */
+typedef enum   { FMT_iso,
+                 FMT_european,
+                 FMT_us,
+                 FMT_nohint
+               } Dates_format;
 
-/* target graph & set*/
-typedef struct {
-    int gno;    /* graph # */
-    int setno;  /* set # */
-} target;
+/* rounding types for dates */
+#define ROUND_SECOND 1
+#define ROUND_MINUTE 2
+#define ROUND_HOUR   3
+#define ROUND_DAY    4
+#define ROUND_MONTH  5
 
-#define copyx(gno, setfrom, setto)      copycol(gno, setfrom, setto, 0)
-#define copyy(gno, setfrom, setto)      copycol(gno, setfrom, setto, 1)
-#define getsetlength(gno, set)  (g[gno].p[set].len)
-#define getx(gno, set)          ((double *) g[gno].p[set].ex[0])
-#define gety(gno, set)          ((double *) g[gno].p[set].ex[1])
-#define getcol(gno, set, col)   ((double *) g[gno].p[set].ex[col])
-#define getstring(gno, set)     ((char **) g[gno].p[set].s)
-#define getcomment(gno, set)    ((char *) g[gno].p[set].comments)
-#define getsetlinet(gno, set)   (g[gno].p[set].linet)
-#define getsetlines(gno, set)   (g[gno].p[set].lines)
-#define getsetlinew(gno, set)   (g[gno].p[set].linew)
-#define getsetcolor(gno, set)   (g[gno].p[set].linepen.color)
-#define getsetpattern(gno, set) (g[gno].p[set].linepen.pattern)
-#define getsetplotsym(gno, set) (g[gno].p[set].sym)
-#define getsetplotsymcolor(gno, set)    (g[gno].p[set].sympen.color)
-#define getsetplotsympattern(gno, set)    (g[gno].p[set].sympen.pattern)
-#define getsetplotsymfillcolor(gno, set)    (g[gno].p[set].symfillpen.color)
-#define getsetplotsymfillpattern(gno, set)    (g[gno].p[set].symfillpen.pattern)
-#define getsetplotsymsize(gno, set)     (g[gno].p[set].symsize)
-#define getsetplotsymchar(gno, set)     (g[gno].p[set].symchar)
-#define getsetplotsymskip(gno, set)     (g[gno].p[set].symskip)
-#define dataset_type(gno, set)  (g[gno].p[set].type)
-#define graph_type(gno) (g[gno].type)
-#define setcomment(gno, set, s) (strcpy(g[gno].p[set].comments, s))
-#define settype(gno, i, it)  (g[gno].p[i].type = it)
-#define setplotsym(gno, i, hsym)  (g[gno].p[i].sym = hsym)
-#define setplotsymcolor(gno, i, col)  (g[gno].p[i].sympen.color = col)
-#define setplotsympattern(gno, i, col)  (g[gno].p[i].sympen.pattern = col)
-#define setplotsymfillcolor(gno, i, col)  (g[gno].p[i].symfillpen.color = col)
-#define setplotsymfillpattern(gno, i, col)  (g[gno].p[i].symfillpen.pattern = col)
-#define setplotsymsize(gno, i, hsym)  (g[gno].p[i].symsize = hsym)
-#define setplotsymskip(gno, i, hsym)  (g[gno].p[i].symskip = hsym)
-#define setplotsymchar(gno, i, hsym)  (g[gno].p[i].symchar = hsym)
-#define setplotlinet(gno, i, linet)  (g[gno].p[i].linet = linet)
-#define setplotlines(gno, i, line)  (g[gno].p[i].lines = line)
-#define setplotlinew(gno, i, wid)  (g[gno].p[i].linew = wid)
-#define setplotcolor(gno, i, col)  (g[gno].p[i].linepen.color = col)
-#define setplotpattern(gno, i, col)  (g[gno].p[i].linepen.pattern = col)
-#define ishidden_graph(gno)     (g[gno].hidden)
-#define yes_or_no(x) ((x)?"yes":"no")
-#define isactive_set(gno, set) (g[gno].p[set].active)
-#define isactive_graph(gno) (g[gno].active)
-#define on_or_off(x) ((x)?"on":"off")
-#define true_or_false(x) ((x)?"true":"false")
-#define w_or_v(x) ((x == COORD_WORLD)?"world":"view")
-
-#define autoscale_graph(gno, axis) autoscale_set(gno, ALL_SETS, axis)
+/* tokens for the calendar dates parser */
+typedef struct { int value;
+                 int digits;
+               } Int_token;
 
 #endif /* __DEFINES_H_ */
