@@ -1,10 +1,10 @@
 /*
- * Grace - Graphics for Exploratory Data Analysis
+ * Grace - GRaphing, Advanced Computation and Exploration of data
  * 
  * Home page: http://plasma-gate.weizmann.ac.il/Grace/
  * 
  * Copyright (c) 1991-95 Paul J Turner, Portland, OR
- * Copyright (c) 1996-98 GRACE Development Team
+ * Copyright (c) 1996-99 Grace Development Team
  * 
  * Maintained by Evgeny Stambulchik <fnevgeny@plasma-gate.weizmann.ac.il>
  * 
@@ -36,17 +36,21 @@
 #include <cmath.h>
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "defines.h"
-#include "globals.h"
+#include "device.h"
+#include "graphs.h"
 #include "utils.h"
 #include "draw.h"
 
-#include "protos.h"
+int ReqUpdateColorSel = FALSE;  /* a part of pre-GUI layer; should be in
+                                   a separate module */
 
 void (*devupdatecmap)();        /* update color map */
 
+void (*devdrawpixel) ();        /* device pixel drawing routine */
 void (*devdrawpolyline) ();     /* device polyline drawing routine */
 void (*devfillpolygon) ();      /* device polygon filling routine */
 void (*devdrawarc) ();          /* device arc drawing routine */
@@ -54,6 +58,13 @@ void (*devfillarc) ();          /* device arc filling routine */
 
 void (*devleavegraphics) ();    /* device exit */
 
+static int all_points_inside(VPoint *vps, int n);
+static void purge_dense_points(const VPoint *vps, int n, VPoint *pvps, int *np);
+
+/* Current drawing properties */
+static DrawProps draw_props =
+{{1, 1}, 0, TRUE, 1, 0.0, LINECAP_BUTT, LINEJOIN_MITER, 1.0, 0, FILLRULE_WINDING};
+
 static world worldwin;
 static view viewport;
 static int coordinates;
@@ -69,6 +80,7 @@ static double fyg_med;
 
 static int clipflag = TRUE;        /* whether clipping must be in force */
 
+static int revflag = FALSE;
 
 /*
  * set pen properties
@@ -111,6 +123,19 @@ int getbgcolor(void)
     return (draw_props.bgcolor);
 }
 
+/*
+ * determines whether page background is filled
+ */
+void setbgfill(int flag)
+{
+    draw_props.bgfilled = flag;
+}
+
+int getbgfill(void)
+{
+    return (draw_props.bgfilled);
+}
+
 /*
  * make the current linestyle lines
  */
@@ -125,12 +150,12 @@ int getlinestyle(void)
     return (draw_props.lines);
 }
 
+#define MAGIC_LINEW_SCALE 0.0015
 /*
  * make the current line width linew
  */
-void setlinewidth(int linew)
+void setlinewidth(double linew)
 {
-#define MAGIC_LINEW_SCALE 0.0015
     draw_props.linew = linew;
     return;
 }
@@ -165,6 +190,20 @@ double getcharsize(void)
     return (draw_props.charsize);
 }
 
+/*
+ * setfont - make font the current font to use for writing strings
+ */
+void setfont(int font)
+{
+    draw_props.font = font;
+    return;
+}
+
+int getfont(void)
+{
+    return (draw_props.font);
+}
+
 /*
  * set the current fillrule
  */
@@ -179,6 +218,34 @@ int getfillrule(void)
     return (draw_props.fillrule);
 }
 
+/*
+ * set/get the current linecap parameter
+ */
+void setlinecap(int type)
+{
+    draw_props.linecap = type;
+    return;
+}
+
+int getlinecap(void)
+{
+    return (draw_props.linecap);
+}
+
+/*
+ * set/get the current linejoin type
+ */
+void setlinejoin(int type)
+{
+    draw_props.linejoin = type;
+    return;
+}
+
+int getlinejoin(void)
+{
+    return (draw_props.linejoin);
+}
+
 /*
  * Convert point's world coordinates to viewport
  */
@@ -209,7 +276,7 @@ void symplus(VPoint vp, double s)
 void symx(VPoint vp, double s)
 {
     VPoint vp1, vp2;
-    double side = s * 0.707;
+    double side = M_SQRT1_2*s;
     
     vp1.x = vp.x - side;
     vp1.y = vp.y - side;
@@ -237,6 +304,18 @@ void leavegraphics(void)
     (*devleavegraphics) ();
 }
 
+/*
+ * DrawPixel - put a pixel in the current color at position vp
+ */
+void DrawPixel(VPoint vp)
+{
+     if (is_validVPoint(vp)) {
+         if (get_draw_mode() == TRUE) {
+             (*devdrawpixel)(vp);
+         }
+         update_bboxes(vp);
+     }
+}
 
 /*
  * DrawRect - draw a rectangle using the current color and linestyle
@@ -283,7 +362,7 @@ void FillRect(VPoint vp1, VPoint vp2)
  */
 void DrawPolyline(VPoint *vps, int n, int mode)
 {
-    int i, j, nmax, nc;
+    int i, j, nmax, nc, max_purge, npurged;
     VPoint vp1, vp2;
     VPoint vp1c, vp2c;
     VPoint *vpsc;
@@ -292,13 +371,24 @@ void DrawPolyline(VPoint *vps, int n, int mode)
         return;
     }
     
+    if (n <= 1) {
+        return;
+    }
+    
     if (mode == POLYLINE_CLOSED) {
         nmax = n + 1;
     } else {
         nmax = n;
     }
     
-    if (doclipping()) {
+    max_purge = get_max_path_limit();
+    
+/*
+ *  in most real cases, all points of a set are inside the viewport;
+ *  so we check it prior to going into compilated clipping mode
+ */
+    if (doclipping() && !all_points_inside(vps, n)) {
+        
         vpsc = (VPoint *) malloc((nmax)*sizeof(VPoint));
         if (vpsc == NULL) {
             errmsg ("malloc() failed in DrawPolyline()");
@@ -329,7 +419,13 @@ void DrawPolyline(VPoint *vps, int n, int mode)
                         if (nc != nmax) {
                             mode = POLYLINE_OPEN;
                         }
-                        (*devdrawpolyline)(vpsc, nc, mode);
+                        if (max_purge && nc > max_purge) {
+                            npurged = max_purge;
+                            purge_dense_points(vpsc, nc, vpsc, &npurged);
+                        } else {
+                            npurged = nc;
+                        }
+                        (*devdrawpolyline)(vpsc, npurged, mode);
                     }
                     nc = 0;
                 }
@@ -341,10 +437,21 @@ void DrawPolyline(VPoint *vps, int n, int mode)
             update_bboxes(vps[j]);
         }
         if (get_draw_mode() == TRUE) {
-            (*devdrawpolyline)(vps, n, mode);
+            if (max_purge && n > max_purge) {
+                npurged = max_purge;
+                vpsc = malloc(max_purge*sizeof(VPoint));
+                if (vpsc == NULL) {
+                    errmsg ("malloc() failed in DrawPolyline()");
+                    return;
+                }
+                purge_dense_points(vps, n, vpsc, &npurged);
+                (*devdrawpolyline)(vpsc, npurged, mode);
+                free(vpsc);
+            } else {
+                (*devdrawpolyline)(vps, n, mode);
+            }
         }
     }
-    
 }
 
 /*
@@ -361,23 +468,27 @@ void DrawLine(VPoint vp1, VPoint vp2)
     DrawPolyline(vps, 2, POLYLINE_OPEN);
 }
 
-
 /*
  * DrawPolygon - draw a filled polygon in the current color and pattern
  *      with nodes given by vps[]
  */
 void DrawPolygon(VPoint *vps, int n)
 {
-    int i, nc;
+    int i, nc, max_purge, npurged;
     VPoint *vptmp;
 
     if ((getpen()).pattern == 0) {
         return;
     }
+    if (n < 3) {
+        return;
+    }
+
+    max_purge = get_max_path_limit();
     
-    if (doclipping()) {
+    if (doclipping() && !all_points_inside(vps, n)) {
         /* In the worst case, the clipped polygon may have twice more vertices */
-        vptmp = (VPoint *) malloc((2*n) * sizeof(VPoint));
+        vptmp = malloc((2*n) * sizeof(VPoint));
         if (vptmp == NULL) {
             errmsg("malloc() failed in DrawPolygon");
             return;
@@ -386,7 +497,13 @@ void DrawPolygon(VPoint *vps, int n)
             nc = clip_polygon(vptmp, n);
             if (nc > 2) {
                 if (get_draw_mode() == TRUE) {
-                    (*devfillpolygon) (vptmp, nc);
+                    if (max_purge && nc > max_purge) {
+                        npurged = max_purge;
+                        purge_dense_points(vptmp, nc, vptmp, &npurged);
+                    } else {
+                        npurged = nc;
+                    }
+                    (*devfillpolygon) (vptmp, npurged);
                 }
                 for (i = 0; i < nc; i++) {
                     update_bboxes(vptmp[i]);
@@ -396,7 +513,19 @@ void DrawPolygon(VPoint *vps, int n)
         }
     } else {
         if (get_draw_mode() == TRUE) {
-            (*devfillpolygon) (vps, n);
+            if (max_purge && n > max_purge) {
+                npurged = max_purge;
+                vptmp = malloc(max_purge*sizeof(VPoint));
+                if (vptmp == NULL) {
+                    errmsg ("malloc() failed in DrawPolygon()");
+                    return;
+                }
+                purge_dense_points(vps, n, vptmp, &npurged);
+                (*devfillpolygon) (vptmp, npurged);
+                free(vptmp);
+            } else {
+                (*devfillpolygon) (vps, n);
+            }
         }
         for (i = 0; i < n; i++) {
             update_bboxes(vps[i]);
@@ -430,7 +559,12 @@ void DrawFilledArc(VPoint vp1, VPoint vp2, int angle1, int angle2)
     if ((getpen()).pattern == 0) {
         return;
     }
-    
+
+    if (points_overlap(vp1, vp2)) {
+        DrawPixel(vp1);
+        return;
+    }
+        
     /* TODO: clipping!!!*/
     if (get_draw_mode() == TRUE) {
         (*devfillarc)(vp1, vp2, angle1, angle2);
@@ -508,15 +642,37 @@ int doclipping(void)
     return(clipflag ? TRUE:FALSE);
 }
 
+/*
+ * is_wpoint_inside() checks if point qp is inside of world rectangle w
+ */
+int is_wpoint_inside(WPoint *wp, world *w)
+{
+    return ((wp->x >= w->xg1) && (wp->x <= w->xg2) &&
+            (wp->y >= w->yg1) && (wp->y <= w->yg2));
+}
+
+/* some to avoid round errors due to the finite FP precision */
+#define VP_EPSILON  0.0001
+
 /*
  * is_vpoint_inside() checks if point vp is inside of viewport rectangle v
  */
-int is_vpoint_inside(view v, VPoint vp)
+int is_vpoint_inside(view v, VPoint vp, double epsilon)
 {
-        return ((vp.x >= MIN2(v.xv1, v.xv2)) && 
-                (vp.x <= MAX2(v.xv1, v.xv2)) &&
-                (vp.y >= MIN2(v.yv1, v.yv2)) && 
-                (vp.y <= MAX2(v.yv1, v.yv2)));
+    return ((vp.x >= v.xv1 - epsilon) && (vp.x <= v.xv2 + epsilon) &&
+            (vp.y >= v.yv1 - epsilon) && (vp.y <= v.yv2 + epsilon));
+}
+
+static int all_points_inside(VPoint *vps, int n)
+{
+    int i;
+    
+    for (i = 0; i < n; i++) {
+        if (is_vpoint_inside(viewport, vps[i], VP_EPSILON) != TRUE) {
+            return FALSE;
+        }
+    }
+    return TRUE;
 }
 
 /*
@@ -525,7 +681,7 @@ int is_vpoint_inside(view v, VPoint vp)
 int is_validVPoint(VPoint vp)
 {
     if (doclipping()) {
-        return (is_vpoint_inside(viewport, vp));
+        return (is_vpoint_inside(viewport, vp, VP_EPSILON));
     } else {
         return TRUE;
     }
@@ -632,14 +788,14 @@ int clip_line(VPoint vp1, VPoint vp2, VPoint *vp1c, VPoint *vp2c)
         *vp2c = vp2;
         return (TRUE);
     } else {
-        vpsa[0].x = viewport.xv1;
-        vpsa[0].y = viewport.yv1;
-        vpsa[1].x = viewport.xv2;
-        vpsa[1].y = viewport.yv1;
-        vpsa[2].x = viewport.xv2;
-        vpsa[2].y = viewport.yv2;
-        vpsa[3].x = viewport.xv1;
-        vpsa[3].y = viewport.yv2;
+        vpsa[0].x = viewport.xv1 - VP_EPSILON;
+        vpsa[0].y = viewport.yv1 - VP_EPSILON;
+        vpsa[1].x = viewport.xv2 + VP_EPSILON;
+        vpsa[1].y = viewport.yv1 - VP_EPSILON;
+        vpsa[2].x = viewport.xv2 + VP_EPSILON;
+        vpsa[2].y = viewport.yv2 + VP_EPSILON;
+        vpsa[3].x = viewport.xv1 - VP_EPSILON;
+        vpsa[3].y = viewport.yv2 + VP_EPSILON;
         vpsa[4] = vpsa[0];
         
         na = 0;
@@ -783,15 +939,24 @@ int is_valid_color(RGB rgb)
     }
 }
 
+int compare_rgb(RGB *rgb1, RGB *rgb2)
+{
+    if ((rgb1->red   == rgb2->red)   &&
+        (rgb1->green == rgb2->green) &&
+        (rgb1->blue  == rgb2->blue)) {
+        return TRUE;
+    } else {
+        return FALSE;
+    }
+}
+
 int find_color(RGB rgb)
 {
     int i;
     int cindex = BAD_COLOR;
     
     for (i = 0; i < maxcolors; i++) {
-        if ((cmap_table[i].rgb.red   == rgb.red)   &&
-            (cmap_table[i].rgb.green == rgb.green) &&
-            (cmap_table[i].rgb.blue  == rgb.blue)) {
+        if (compare_rgb(&cmap_table[i].rgb, &rgb) == TRUE) {
             cindex = i;
             break;
         }
@@ -800,6 +965,82 @@ int find_color(RGB rgb)
     return (cindex);
 }
 
+int get_color_by_name(char *cname)
+{
+    int i;
+    int cindex = BAD_COLOR;
+    
+    for (i = 0; i < maxcolors; i++) {
+        if (cmap_table[i].ctype == COLOR_MAIN &&
+            compare_strings(cmap_table[i].cname, cname) == TRUE) {
+            cindex = i;
+            break;
+        }
+    }
+    
+    return (cindex);
+}
+
+int realloc_colors(int n)
+{
+    int i;
+    CMap_entry *cmap_tmp;
+    
+    if (n > MAXCOLORS) {
+        return GRACE_EXIT_FAILURE;
+    } else {
+        for (i = n; i < maxcolors; i++) {
+            cxfree(cmap_table[i].cname);
+        }
+        cmap_tmp = xrealloc(cmap_table, n*sizeof(CMap_entry));
+        if (cmap_tmp == NULL) {
+            return GRACE_EXIT_FAILURE;
+        } else {
+            cmap_table = cmap_tmp;
+            for (i = maxcolors; i < n; i++) {
+                cmap_table[i].rgb.red = 0;
+                cmap_table[i].rgb.green = 0;
+                cmap_table[i].rgb.blue = 0;
+                cmap_table[i].cname = NULL;
+                cmap_table[i].ctype = COLOR_NONE;
+                cmap_table[i].tstamp = 0;
+            }
+        }
+        maxcolors = n;
+    }
+    
+    return GRACE_EXIT_SUCCESS;
+}
+
+int store_color(int n, CMap_entry cmap)
+{
+    if (is_valid_color(cmap.rgb) != TRUE) {
+        return GRACE_EXIT_FAILURE;
+    } else if (n >= maxcolors && realloc_colors(n + 1) == GRACE_EXIT_FAILURE) {
+        return GRACE_EXIT_FAILURE;
+    } else {
+        if (cmap.cname == NULL || strlen(cmap.cname) == 0) {
+            cmap_table[n].cname =
+                copy_string(cmap_table[n].cname, "unnamed");
+        } else {
+            cmap_table[n].cname =
+                copy_string(cmap_table[n].cname, cmap.cname);
+        }
+        cmap_table[n].rgb = cmap.rgb;
+        cmap_table[n].ctype = cmap.ctype;
+        cmap_table[n].tstamp = 1;
+                
+        /* inform current device of changes in the cmap database */
+        if (devupdatecmap != NULL) {
+            (*devupdatecmap)();
+        }
+        if (cmap.ctype == COLOR_MAIN) {
+            ReqUpdateColorSel = TRUE;
+        }
+        return GRACE_EXIT_SUCCESS;
+    }
+}
+
 /*
  * add_color() adds a new entry to the colormap table
  */
@@ -810,30 +1051,15 @@ int add_color(CMap_entry cmap)
     if (is_valid_color(cmap.rgb) != TRUE) {
         cindex = BAD_COLOR;
     } else if ((cindex = find_color(cmap.rgb)) != BAD_COLOR) {
-        if (cmap.ctype == COLOR_MAIN) {
+        if (cmap.ctype == COLOR_MAIN && 
+            cmap_table[cindex].ctype != COLOR_MAIN) {
             cmap_table[cindex].ctype = COLOR_MAIN;
+            ReqUpdateColorSel = TRUE;
         }
-    } else if (maxcolors >= MAXCOLORS) {
+    } else if (store_color(maxcolors, cmap) == GRACE_EXIT_FAILURE) {
         cindex = BAD_COLOR;
     } else {
-        maxcolors++;
         cindex = maxcolors - 1;
-        cmap_table = xrealloc(cmap_table, maxcolors*sizeof(CMap_entry));
-        if (cmap.cname == NULL || strlen(cmap.cname) == 0) {
-            cmap_table[cindex].cname = malloc(8);
-            strcpy(cmap_table[cindex].cname, (char *) "unnamed");
-        } else { 
-            cmap_table[cindex].cname = malloc((strlen(cmap.cname) + 1));
-            strcpy(cmap_table[cindex].cname, cmap.cname);
-        }
-        cmap_table[cindex].rgb = cmap.rgb;
-        cmap_table[cindex].ctype = cmap.ctype;
-        cmap_table[cindex].tstamp = 1;
-        
-        /* inform current device of changes in the cmap database */
-        if (devupdatecmap != NULL) {
-            (*devupdatecmap)();
-        }
     }
     
     return (cindex);
@@ -854,6 +1080,20 @@ RGB *get_rgb(unsigned int cindex)
     }
 }
 
+fRGB *get_frgb(unsigned int cindex)
+{
+    static fRGB frgb;
+    
+    if (cindex < maxcolors) {
+        frgb.red   = (double) cmap_table[cindex].rgb.red   / (MAXCOLORS - 1);
+        frgb.green = (double) cmap_table[cindex].rgb.green / (MAXCOLORS - 1);
+        frgb.blue  = (double) cmap_table[cindex].rgb.blue  / (MAXCOLORS - 1);
+        return &frgb;
+    } else {
+        return NULL;
+    }
+}
+
 CMap_entry *get_cmap_entry(unsigned int cindex)
 {
     if (cindex < maxcolors) {
@@ -955,22 +1195,29 @@ static CMap_entry cmap_init[] = {
  */
 void initialize_cmap(void)
 {
-    int i;
+    int i, n;
     
-    maxcolors = 0;
-    
-    if (revflag == TRUE) {
-        add_color(cmap_init[1]);
-        add_color(cmap_init[0]);
-    } else {
-        add_color(cmap_init[0]);
-        add_color(cmap_init[1]);
-    }
-    for (i = 2; i < sizeof(cmap_init)/sizeof(CMap_entry); i++) {
-        add_color(cmap_init[i]);
+    n = sizeof(cmap_init)/sizeof(CMap_entry);
+    realloc_colors(n);
+    for (i = 0; i < n; i++) {
+        store_color(i, cmap_init[i]);
     }
 }
 
+void reverse_video(void)
+{
+    CMap_entry ctmp;
+    
+    memcpy(&ctmp, &cmap_table[0], sizeof(CMap_entry));
+    memcpy(&cmap_table[0], &cmap_table[1], sizeof(CMap_entry));
+    memcpy(&cmap_table[1], &ctmp, sizeof(CMap_entry));
+    revflag = !revflag;
+}
+
+int is_video_reversed(void)
+{
+    return revflag;
+}
 
 /* 
  * ------------------ Pattern routines ---------------
@@ -994,6 +1241,29 @@ int number_of_linestyles(void)
  * ------------- coordinate conversion routines ------------
  */
 
+char *scale_types(int it)
+{
+    static char s[16];
+
+    switch (it) {
+    case SCALE_NORMAL:
+	strcpy(s, "Normal");
+	break;
+    case SCALE_LOG:
+	strcpy(s, "Logarithmic");
+	break;
+    case SCALE_REC:
+	strcpy(s, "Reciprocal");
+	break;
+    default:
+        strcpy(s, "Unknown");
+	break;
+    }
+    
+    return s;
+}
+
+
 /*
  * axis scaling
  */
@@ -1102,11 +1372,6 @@ void view2world(double xv, double yv, double *xw, double *yw)
     }
 }
 
-view getviewport(void)
-{
-    return (viewport);
-}
-
 /*
  * definewindow - defines the scaling
  *               of the plotting rectangle to be used for clipping
@@ -1115,7 +1380,8 @@ int definewindow(world w, view v, int gtype,
                         int xscale, int yscale,
                         int invx, int invy)
 {
-    if (gtype == GRAPH_POLAR) {
+    switch (gtype) {
+    case GRAPH_POLAR:
         if ((xscale != SCALE_NORMAL) ||
             (yscale != SCALE_NORMAL) ||
             (invy == TRUE)) {
@@ -1137,7 +1403,37 @@ int definewindow(world w, view v, int gtype,
             yv_rc = (MIN2(v.xv2 - v.xv1, v.yv2 - v.yv1)/2.0)/w.yg2;
             return GRACE_EXIT_SUCCESS;
         }
-    } else {
+        break;
+    case GRAPH_FIXED:
+        if ((xscale != SCALE_NORMAL) ||
+            (yscale != SCALE_NORMAL)) {
+            return GRACE_EXIT_FAILURE;
+        } else {
+            coordinates = COORDINATES_XY;
+            worldwin = w;
+            viewport = v;
+
+            scaletypex = xscale;
+            xv_med = (v.xv1 + v.xv2)/2;
+            fxg_med = (w.xg1 + w.xg2)/2;
+            scaletypey = yscale;
+            yv_med = (v.yv1 + v.yv2)/2;
+            fyg_med = (w.yg1 + w.yg2)/2;
+
+            xv_rc = MIN2((v.xv2 - v.xv1)/(w.xg2 - w.xg1),
+                         (v.yv2 - v.yv1)/(w.yg2 - w.yg1));
+            yv_rc = xv_rc;
+            if (invx == TRUE) {
+                xv_rc = -xv_rc;
+            }
+            if (invy == TRUE) {
+                yv_rc = -yv_rc;
+            }
+
+            return GRACE_EXIT_SUCCESS;
+        }
+        break;
+    default:
         if ((xscale == SCALE_LOG && (w.xg1 <= 0.0 || w.xg2 <= 0.0)) ||
             (yscale == SCALE_LOG && (w.yg1 <= 0.0 || w.yg2 <= 0.0)) ||
             (xscale == SCALE_REC && (w.xg1 == 0.0 || w.xg2 == 0.0)) ||
@@ -1168,9 +1464,105 @@ int definewindow(world w, view v, int gtype,
  
            return GRACE_EXIT_SUCCESS;
         }
+        break;
     }
 }
 
+int checkon_world(int gno)
+{
+    double dx, dy;
+    world w;
+    
+    get_graph_world(gno, &w);
+    dx = w.xg2 - w.xg1;
+    dy = w.yg2 - w.yg1;
+
+    if (get_graph_type(gno) == GRAPH_POLAR) {
+        if (w.yg2 <= 0.0) {
+            errmsg("World Rho-max <= 0.0");
+            return FALSE;
+        } else {
+            return TRUE;
+        }
+    } else {
+        if (dx <= 0.0) {
+            errmsg("World DX <= 0.0");
+            return FALSE;
+        }
+        if (dy <= 0.0) {
+            errmsg("World DY <= 0.0");
+            return FALSE;
+        }
+        
+        if (get_graph_type(gno) == GRAPH_FIXED) {
+            if ((get_graph_xscale(gno) != SCALE_NORMAL) ||
+                (get_graph_yscale(gno) != SCALE_NORMAL)) {
+                errmsg("Only linear axis scale is allowed in Fixed graphs");
+                return FALSE;
+            } 
+        }
+        
+        if (get_graph_xscale(gno) == SCALE_LOG) {
+            if (w.xg1 <= 0) {
+                errmsg("World X-min <= 0.0");
+                return FALSE;
+            }
+            if (w.xg2 <= 0) {
+                errmsg("World X-max <= 0.0");
+                return FALSE;
+            }
+        } else if (get_graph_xscale(gno) == SCALE_REC) {
+            if (sign(w.xg1) != sign(w.xg2)) {
+                errmsg("X-axis contains 0");
+                return FALSE;
+            }
+        }
+        
+        if (get_graph_yscale(gno) == SCALE_LOG) {
+            if (w.yg1 <= 0.0) {
+                errmsg("World Y-min <= 0.0");
+                return FALSE;
+            }
+            if (w.yg2 <= 0.0) {
+                errmsg("World Y-max <= 0.0");
+                return FALSE;
+            }
+        } else if (get_graph_yscale(gno) == SCALE_REC) {
+            if (sign(w.yg1) != sign(w.yg2)) {
+                errmsg("Y-axis contains 0");
+                return FALSE;
+            }
+        }
+
+        return TRUE;
+    }
+
+}
+
+int isvalid_viewport(view v)
+{
+    if ((v.xv2 <= v.xv1) || (v.yv2 <= v.yv1)) {
+	return FALSE;
+    } else {
+        return TRUE;
+    }
+}
+
+/* check viewport (min < max) */
+int checkon_viewport(int gno)
+{
+    view v;
+    
+    get_graph_viewport(gno, &v);
+    if (isvalid_viewport(v) == FALSE) {
+        errmsg("Invalid viewport coordinates");
+        return FALSE;
+    } else {
+        return TRUE;
+    }
+}
+
+
 /*
  * ---------------- bbox utilities --------------------
  */
@@ -1293,7 +1685,7 @@ void update_bbox(int type, VPoint vp)
         return;
     }
     if (bbp->active == TRUE) {
-        if (is_vpoint_inside(bbp->v, vp) == FALSE) {
+        if (is_vpoint_inside(bbp->v, vp, 0.0) == FALSE) {
             if (is_valid_bbox(bbp->v)) {
                 bbp->v.xv1 = MIN2(bbp->v.xv1, vp.x);
                 bbp->v.xv2 = MAX2(bbp->v.xv2, vp.x);
@@ -1362,3 +1754,101 @@ int get_draw_mode(void)
 {
     return (draw_mode);
 }
+
+void vpswap(VPoint *vp1, VPoint *vp2)
+{
+    VPoint vptmp;
+
+    vptmp = *vp1;
+    *vp1 = *vp2;
+    *vp2 = vptmp;
+}
+
+int points_overlap(VPoint vp1, VPoint vp2)
+{
+    double delta;
+    
+    delta = 1.0/MIN2(page_width, page_height);
+    if (fabs(vp2.x - vp1.x) < delta || fabs(vp2.y - vp1.y) < delta) {
+        return TRUE;
+    } else {
+        return FALSE;
+    }
+}
+
+
+static max_path_length = MAX_DRAWING_PATH;
+
+void set_max_path_limit(int limit)
+{
+    max_path_length = limit;
+}
+
+int get_max_path_limit(void)
+{
+    return max_path_length;
+}
+
+#define PURGE_INIT_FACTOR   1.0
+#define PURGE_ITER_FACTOR   M_SQRT2
+
+/* Note: vps and pvps may be the same array! */
+static void purge_dense_points(const VPoint *vps, int n, VPoint *pvps, int *np)
+{
+    int i, j, iter;
+    int ok;
+    double eps;
+    VPoint vptmp;
+    
+    if (n <= *np) {
+        memmove(pvps, vps, n*sizeof(VPoint));
+    }
+    
+    if (*np <= 0) {
+        *np = 0;
+        return;
+    }
+    
+    /* Start with 1/np epsilon */
+    eps = PURGE_INIT_FACTOR/(*np);
+    iter = 0;
+    ok = FALSE;
+    while (ok == FALSE) {
+        j = 0;
+        vptmp = vps[0];
+        for (i = 0; i < n - 1; i++) {
+            if (fabs(vps[i].x - vptmp.x) > eps ||
+                fabs(vps[i].y - vptmp.y) > eps) {
+                vptmp = vps[i];
+                j++;
+                if (j >= *np) {
+                    break;
+                }
+            }
+        }
+        if (j < *np - 1) {
+            ok = TRUE;
+        } else {
+            eps *= PURGE_ITER_FACTOR;
+        }
+        iter++;
+    }
+
+    /* actually fill the purged array */
+    pvps[0] = vps[0];
+    j = 0;
+    for (i = 0; i < n - 1; i++) {
+        if (fabs(vps[i].x - pvps[j].x) > eps ||
+            fabs(vps[i].y - pvps[j].y) > eps) {
+            pvps[++j] = vps[i];
+        }
+    }
+    pvps[j++] = vps[n - 1];
+    
+    *np = j;
+#ifdef DEBUG    
+    if (get_debuglevel() == 6) {
+        printf("Purging %d points to %d in %d iteration(s)\n", n, *np, iter);
+    }
+#endif
+}
