@@ -1,10 +1,10 @@
 /*
- * Grace - Graphics for Exploratory Data Analysis
+ * Grace - GRaphing, Advanced Computation and Exploration of data
  * 
  * Home page: http://plasma-gate.weizmann.ac.il/Grace/
  * 
  * Copyright (c) 1991-95 Paul J Turner, Portland, OR
- * Copyright (c) 1996-98 GRACE Development Team
+ * Copyright (c) 1996-99 Grace Development Team
  * 
  * Maintained by Evgeny Stambulchik <fnevgeny@plasma-gate.weizmann.ac.il>
  * 
@@ -39,6 +39,8 @@
 
 #include "globals.h"
 #include "utils.h"
+#include "graphs.h"
+#include "graphutils.h"
 #include "draw.h"
 #include "t1fonts.h"
 #include "protos.h"
@@ -48,6 +50,21 @@ int is_xaxis(int axis)
     return ((axis % 2 == 0));
 }
 
+int is_yaxis(int axis)
+{
+    return ((axis % 2 == 1));
+}
+
+int is_log_axis(int gno, int axis)
+{
+    if ((is_xaxis(axis) && islogx(gno)) ||
+        (is_yaxis(axis) && islogy(gno))) {
+        return TRUE;
+    } else {
+        return FALSE;
+    }
+}
+
 void drawgrid(int gno)
 {
     int caxis;
@@ -69,7 +86,7 @@ void drawgrid(int gno)
     /* TODO: add Pen to ticks and remove the following */
     setpattern(1);
     
-    get_graph_view(gno, &v);
+    get_graph_viewport(gno, &v);
     get_graph_world(gno, &w);
     
     /* graph center; for polar plots */
@@ -132,7 +149,7 @@ void drawgrid(int gno)
 	    	vp_grid_stop = Wpoint2Vpoint(wp_grid_stop);
 
 
-                if (!is_xaxis(caxis) && g[gno].type == GRAPH_POLAR) {
+                if (!is_xaxis(caxis) && get_graph_type(gno) == GRAPH_POLAR) {
                     xy2polar(vp_grid_start.x - vpc.x, vp_grid_start.y - vpc.y,
                              &phi_start, &rho);
                     xy2polar(vp_grid_stop.x - vpc.x, vp_grid_stop.y - vpc.y,
@@ -141,7 +158,7 @@ void drawgrid(int gno)
                     vp1.y = vpc.y + rho;
                     vp2.x = vpc.x + rho;
                     vp2.y = vpc.y - rho;
-                    if (g[gno].xinvert == TRUE) {
+                    if (is_graph_xinvert(gno) == TRUE) {
                         fswap(&phi_start, &phi_stop);
                     }
                     if (phi_stop < phi_start) {
@@ -154,6 +171,7 @@ void drawgrid(int gno)
                 }
 	    }
 	}
+        free_ticklabels(&t);
     }
 }
 
@@ -193,7 +211,7 @@ void drawaxes(int gno)
     /* TODO: add Pen to ticks and remove the following */
     setpattern(1);
     
-    get_graph_view(gno, &v);
+    get_graph_viewport(gno, &v);
     get_graph_world(gno, &w);
 
     /* graph center; for polar plots */
@@ -249,13 +267,13 @@ void drawaxes(int gno)
             vp2_start = Wpoint2Vpoint(wp2_start);
             vp2_stop  = Wpoint2Vpoint(wp2_stop);
             
-            if (g[gno].yinvert == TRUE) {
+            if (is_graph_yinvert(gno) == TRUE) {
                 vpswap(&vp1_start, &vp2_start);
                 vpswap(&vp1_stop, &vp2_stop);
             }
 
             /* TODO axis offset for polar plots */
-            if (g[gno].type != GRAPH_POLAR) {
+            if (get_graph_type(gno) != GRAPH_POLAR) {
                  vp1_start.y -= t.offsx;
                  vp1_stop.y  -= t.offsx;
                  vp2_start.y += t.offsy;
@@ -316,12 +334,12 @@ void drawaxes(int gno)
             vp2_start = Wpoint2Vpoint(wp2_start);
             vp2_stop  = Wpoint2Vpoint(wp2_stop);
 
-            if (g[gno].xinvert == TRUE) {
+            if (is_graph_xinvert(gno) == TRUE) {
                 vpswap(&vp1_start, &vp2_start);
                 vpswap(&vp1_stop, &vp2_stop);
             }
 
-            if (g[gno].type != GRAPH_POLAR) {
+            if (get_graph_type(gno) != GRAPH_POLAR) {
                 vp1_start.x -= t.offsx;
                 vp1_stop.x  -= t.offsx;
                 vp2_start.x += t.offsy;
@@ -352,8 +370,8 @@ void drawaxes(int gno)
 	    setcolor(t.t_drawbarcolor);
 	    setlinewidth(t.t_drawbarlinew);
 	    setlinestyle(t.t_drawbarlines);
-	    if (t.t_op == PLACE_LEFT || t.t_op == PLACE_BOTTOM || t.t_op == PLACE_BOTH) {
-                if (is_xaxis(caxis) && g[gno].type == GRAPH_POLAR) {
+	    if (t.t_op == PLACEMENT_NORMAL || t.t_op == PLACEMENT_BOTH) {
+                if (is_xaxis(caxis) && get_graph_type(gno) == GRAPH_POLAR) {
                     xy2polar(vp1_start.x - vpc.x, vp1_start.y - vpc.y,
                              &phi_start, &rho);
                     xy2polar(vp1_stop.x - vpc.x, vp1_stop.y - vpc.y,
@@ -362,7 +380,7 @@ void drawaxes(int gno)
                     vp1.y = vpc.y + rho;
                     vp2.x = vpc.x + rho;
                     vp2.y = vpc.y - rho;
-                    if (g[gno].xinvert == TRUE) {
+                    if (is_graph_xinvert(gno) == TRUE) {
                         fswap(&phi_start, &phi_stop);
                     }
                     if (phi_stop < phi_start) {
@@ -374,8 +392,8 @@ void drawaxes(int gno)
 	    	    DrawLine(vp1_start, vp1_stop);
                 }
 	    }
-	    if (t.t_op == PLACE_RIGHT || t.t_op == PLACE_TOP || t.t_op == PLACE_BOTH) {
-                if (is_xaxis(caxis) && g[gno].type == GRAPH_POLAR) {
+	    if (t.t_op == PLACEMENT_OPPOSITE || t.t_op == PLACEMENT_BOTH) {
+                if (is_xaxis(caxis) && get_graph_type(gno) == GRAPH_POLAR) {
                     xy2polar(vp2_start.x - vpc.x, vp2_start.y - vpc.y,
                              &phi_start, &rho);
                     xy2polar(vp2_stop.x - vpc.x, vp2_stop.y - vpc.y,
@@ -384,7 +402,7 @@ void drawaxes(int gno)
                     vp1.y = vpc.y + rho;
                     vp2.x = vpc.x + rho;
                     vp2.y = vpc.y - rho;
-                    if (g[gno].xinvert == TRUE) {
+                    if (is_graph_xinvert(gno) == TRUE) {
                         fswap(&phi_start, &phi_stop);
                     }
                     if (phi_stop < phi_start) {
@@ -401,7 +419,7 @@ void drawaxes(int gno)
 
         
         /* TODO ticks, labels and axis labels for polar plots */
-        if (g[gno].type == GRAPH_POLAR) {
+        if (get_graph_type(gno) == GRAPH_POLAR) {
             continue;
         }
 
@@ -459,18 +477,16 @@ void drawaxes(int gno)
 	            }
 	
 	            vtpos = coord_conv(wtpos);
-	            if (t.t_op == PLACE_LEFT ||
-	                t.t_op == PLACE_BOTTOM ||
-	                t.t_op == PLACE_BOTH) {
+	            if (t.t_op == PLACEMENT_NORMAL ||
+	                t.t_op == PLACEMENT_BOTH) {
 	                vp_tick1_start.x = vtpos*ort_para.x + vbase1_start*ort_perp.x;
 	                vp_tick1_start.y = vtpos*ort_para.y + vbase1_start*ort_perp.y;
 	                vp_tick1_stop.x  = vtpos*ort_para.x + vbase1_stop*ort_perp.x;
 	                vp_tick1_stop.y  = vtpos*ort_para.y + vbase1_stop*ort_perp.y;
 	                DrawLine(vp_tick1_start, vp_tick1_stop);
 	            }
-	            if (t.t_op == PLACE_RIGHT ||
-	                t.t_op == PLACE_TOP ||
-	                t.t_op == PLACE_BOTH) {
+	            if (t.t_op == PLACEMENT_OPPOSITE ||
+	                t.t_op == PLACEMENT_BOTH) {
 	                vp_tick2_start.x = vtpos*ort_para.x + vbase2_start*ort_perp.x;
 	                vp_tick2_start.y = vtpos*ort_para.y + vbase2_start*ort_perp.y;
 	                vp_tick2_stop.x  = vtpos*ort_para.x + vbase2_stop*ort_perp.x;
@@ -562,9 +578,8 @@ void drawaxes(int gno)
 
 	        if (itcur % (t.tl_skip + 1) == 0) {
 		  /* Tick labels on normal side */
-	            if (t.tl_op == PLACE_LEFT ||
-	                t.tl_op == PLACE_BOTTOM ||
-	                t.tl_op == PLACE_BOTH) {
+	            if (t.tl_op == PLACEMENT_NORMAL ||
+	                t.tl_op == PLACEMENT_BOTH) {
 	                vbase_tlabel = vbase_tlabel1 - (tl_offset + tlsize)*
 	                                        (itcur % (t.tl_staggered + 1));
 	                vp_tlabel.x = (vtpos + tl_trans)*ort_para.x +
@@ -574,9 +589,8 @@ void drawaxes(int gno)
 	                WriteString(vp_tlabel, t.tl_angle, tlabel1_just, tlabel);
 	            }
 		    /* Tick labels on opposite side */
-	            if (t.tl_op == PLACE_RIGHT ||
-	                t.tl_op == PLACE_TOP ||
-	                t.tl_op == PLACE_BOTH) {
+	            if (t.tl_op == PLACEMENT_OPPOSITE ||
+	                t.tl_op == PLACEMENT_BOTH) {
 	                vbase_tlabel = vbase_tlabel2 + (tl_offset + tlsize)*
 	                                        (itcur % (t.tl_staggered + 1));
 	                vp_tlabel.x = (vtpos + tl_trans)*ort_para.x +
@@ -598,7 +612,7 @@ void drawaxes(int gno)
 	vp_label_offset.x = 0.00; /* parallel */
 	vp_label_offset.y = 0.08; /* perpendicular */
 
-	if (t.label.s[0]) {
+	if (t.label.s && t.label.s[0]) {
 	    
 	    if (t.label_place == TYPE_SPEC) {
 	        vp_label_offset.x = t.label.x;
@@ -613,9 +627,8 @@ void drawaxes(int gno)
 	    setcolor(t.label.color);
 
 	    /* Axis label on normal side */
-	    if (t.label_op == PLACE_LEFT ||
-		t.label_op == PLACE_BOTTOM ||
-		t.label_op == PLACE_BOTH) {
+	    if (t.label_op == PLACEMENT_NORMAL ||
+		t.label_op == PLACEMENT_BOTH) {
 
 	        vp_label.x = (vp1_start.x + vp1_stop.x)/2 + vp_label_offset.x*ort_para.x
                                                         - vp_label_offset.y*ort_perp.x ;
@@ -626,9 +639,8 @@ void drawaxes(int gno)
 	    }
 
 	    /* Axis label on opposite side */
-	    if (t.label_op == PLACE_RIGHT ||
-		t.label_op == PLACE_TOP ||
-		t.label_op == PLACE_BOTH) {
+	    if (t.label_op == PLACEMENT_OPPOSITE ||
+		t.label_op == PLACEMENT_BOTH) {
 
 	        vp_label.x = (vp2_start.x + vp2_stop.x)/2 + vp_label_offset.x*ort_para.x
 		                                        + vp_label_offset.y*ort_perp.x ;
@@ -639,16 +651,14 @@ void drawaxes(int gno)
 	    }
 
 	}
+        
+        free_ticklabels(&t);
 	/* End axis label stuff */
     }
 }
 
 void calculate_tickgrid(int gno)
 {
-    extern double fscale(double wc, int scale);
-    extern double ifscale(double vc, int scale);
-    
-    
     int caxis;
     int itick, imtick, itmaj;
     int nmajor;
@@ -674,7 +684,7 @@ reenter:
         
         
         if (is_xaxis(caxis)) {
-            scale = g[gno].xscale;
+            scale = get_graph_xscale(gno);
             if (scale == SCALE_LOG) {
                 swc_start = fscale(w.xg1, scale);
                 swc_stop  = fscale(w.xg2, scale);
@@ -683,7 +693,7 @@ reenter:
                 swc_stop  = w.xg2;
             }
         } else {
-            scale = g[gno].yscale;
+            scale = get_graph_yscale(gno);
             if (scale == SCALE_LOG) {
                 swc_start = fscale(w.yg1, scale);
                 swc_stop  = fscale(w.yg2, scale);
@@ -697,6 +707,13 @@ reenter:
         } else {
             stmajor = t.tmajor;
         }
+
+        if (stmajor <= 0.0) {
+	    errmsg("Invalid major tick spacing, autoticking");
+	    autotick_axis(gno, caxis);
+            free_ticklabels(&t);
+            goto reenter;
+	}
         
         if (t.t_round == TRUE) {
             swc_start = floor(swc_start/stmajor)*stmajor;
@@ -708,6 +725,7 @@ reenter:
         if (t.nticks > MAX_TICKS) {
 	    errmsg("Too many ticks ( > MAX_TICKS ), autoticking");
 	    autotick_axis(gno, caxis);
+            free_ticklabels(&t);
             goto reenter;
 	}
 
@@ -765,5 +783,6 @@ reenter:
             itmaj++;
         }
         set_graph_tickmarks(gno, &t, caxis);
+        free_ticklabels(&t);
     }
 }
