@@ -38,29 +38,26 @@
 #include <string.h>
 
 #include <Xm/Xm.h>
-#include <Xm/BulletinB.h>
 #include <Xm/DialogS.h>
 #include <Xm/Form.h>
 #include <Xm/Label.h>
-#include <Xm/LabelG.h>
 #include <Xm/PushB.h>
 #include <Xm/RowColumn.h>
-#include <Xm/Separator.h>
 #include <Xm/ToggleB.h>
 #include <Xm/Text.h>
-#include <Xm/List.h>
 
 #include "globals.h"
 #include "utils.h"
+#include "graphs.h"
 #include "protos.h"
 #include "motifinc.h"
 
 typedef struct _Featext_ui {
     Widget top;
-    Widget *tograph;
-    Widget *feature_item;
+    ListStructure *tograph;
+    OptionStructure *feature_item;
     Widget *xval_item;
-    Widget *absic_graph;
+    ListStructure *absic_graph;
     SetChoiceItem absic_set;
     Widget legload_rc;
 } Featext_ui;
@@ -96,40 +93,50 @@ void do_fext_toggle (Widget w, XtPointer client_data, XtPointer call_data)
     }
 }
 
-void do_gto_setsel_update(Widget w, XtPointer client_data, XtPointer call_data)
+void create_featext_frame(void *data)
 {
-	feui.absic_set.gno = (int)client_data;
-	update_save_set_list( feui.absic_set, (int)client_data );
-}
-
-
-void create_featext_frame(Widget w, XtPointer client_data, XtPointer call_data)
-{
-    int i;
-    Widget dialog;
     set_wait_cursor();
+    
     if (feui.top == NULL) {
-	char *label2[3];
+        Widget dialog;
+	char *label2[2];
+        int i;
+        OptionItem feature_option_items[] = {
+            { 0, "Y minimum"         },
+            { 1, "Y maximum"         },
+            { 2, "Y average"         },
+            { 3, "Y std. dev."       },
+            { 4, "Y median"          },
+            { 5, "X minimum"         },
+            { 6, "X maximum"         },
+            { 7, "X average"         },
+            { 8, "X std. dev."       },
+            { 9, "X median"          },
+            {10, "Frequency"         },
+            {11, "Period"            },
+            {12, "Zero crossing"     },
+            {13, "Rise time"         },
+            {14, "Fall time"         },
+            {15, "Slope"             },
+            {16, "Y intercept"       },
+            {17, "Set length"        },
+            {18, "Half maximal width"},
+            {19, "Barycenter X"      },
+            {20, "Barycenter Y"      },
+            {21, "X(Y max)"          },
+            {22, "Y(X max)"          },
+            {23, "Integral"          }
+        };
+
 	label2[0] = "Accept";
 	label2[1] = "Close";
 	feui.top = XmCreateDialogShell(app_shell, "Feature Extraction", NULL, 0);
 	handle_close(feui.top);
 	dialog = XmCreateRowColumn(feui.top, "dialog_rc", NULL, 0);
 
-	feui.tograph = CreateGraphChoice(dialog, "Results to graph: ", maxgraph, 1);
-	feui.feature_item = CreatePanelChoice0(dialog,
-					  "Feature:", 3, 25,
-					  "Y minimum", "Y maximum", "Y average", "Y std. dev.",
-					  "Y median",
-					  "X minimum", "X maximum", "X average", "X std. dev.",
-					  "X median",
-					  "Frequency", "Period", "Zero crossing", 
-					  "Rise time", "Fall time", "Slope", "Y intercept", 
-					  "Set length", "Half maximal width", 
-					  "Barycenter X", "Barycenter Y", 
-					  "X(Y max)", "Y(X max)",
-					  "Integral",
-					  NULL, 0);
+	feui.tograph = CreateGraphChoice(dialog, "Results to graph:", LIST_TYPE_SINGLE);
+	feui.feature_item =
+            CreateOptionChoice(dialog, "Feature:", 3, 24, feature_option_items);
 	feui.xval_item = CreatePanelChoice(dialog,
 						"X values from:", 5,
 						"Index", "Legends", "X from Set", "Y from set",
@@ -140,15 +147,12 @@ void create_featext_frame(Widget w, XtPointer client_data, XtPointer call_data)
 			(XtCallbackProc) do_fext_toggle, (XtPointer) i);
 	}
 	
-	XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass, dialog, NULL);
+	CreateSeparator(dialog);
 
 	feui.legload_rc= XmCreateRowColumn(dialog, "fext_legload_rc", NULL, 0);
 	
 	feui.absic_graph = CreateGraphChoice(feui.legload_rc,
-								"Abscissa from graph: ",maxgraph,0);
-	for(i=0; i<maxgraph; i++ )
-	    XtAddCallback(feui.absic_graph[2 + i], XmNactivateCallback,
-			(XtCallbackProc) do_gto_setsel_update, (XtPointer) i);
+	    "Abscissa from graph:", LIST_TYPE_SINGLE);
 	
 	feui.absic_set = CreateSetSelector(feui.legload_rc, "set:",
 					SET_SELECT_ACTIVE,
@@ -159,7 +163,7 @@ void create_featext_frame(Widget w, XtPointer client_data, XtPointer call_data)
 	
 	XtManageChild(feui.legload_rc);
 	XtSetSensitive(feui.legload_rc, False);
-	XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass, dialog, NULL);
+	CreateSeparator(dialog);
 
 	CreateCommandButtons(dialog, 2, but2, label2);
 	XtAddCallback(but2[0], XmNactivateCallback, (XtCallbackProc)
@@ -180,15 +184,15 @@ void do_fext_proc( Widget w, XtPointer client_data, XtPointer call_data )
 
     Featext_ui *ui = (Featext_ui *) client_data;
 
-    feature = (int) GetChoice(ui->feature_item);
-    gto = (int) GetChoice(ui->tograph )-1;
+    feature = GetOptionChoice(ui->feature_item);
+    GetSingleListChoice(ui->tograph, &gto);
     if( gto == -1 )
-            gto = cg;
+            gto = get_cg();
 
     abs_src = (int) GetChoice(ui->xval_item);
     if( abs_src ==2 || abs_src==3 ) {
         abs_set = GetSelectedSet(ui->absic_set);
-        abs_graph = (int) GetChoice(ui->absic_graph);
+        GetSingleListChoice(ui->absic_graph, &abs_graph);
     }
     fext_routine( gto, feature, abs_src, abs_set, abs_graph ); 
 }
@@ -201,10 +205,13 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 	int iy1, iy2;
 	char tbuf[1024];
 	float *abscissa;
+        double xmin, xmax, ymin, ymax;
+        int cg = get_cg();
+        double *x;
 
-	abscissa = (float *)malloc( maxplot*sizeof(float) );
+	abscissa = malloc(number_of_sets(cg)*SIZEOF_FLOAT);
 	
-	if( !isactive_graph( gto )	){
+	if( !is_graph_active( gto )	){
 		errwin("Graph for results must be active");
 	    return;
 	}
@@ -212,18 +219,18 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 		errwin("Choose a new graph or kill sets!");
 	    return;
 	}
-	ncurves = nactive(cg);
+	ncurves = nactive(get_cg());
 	switch( abs_src ) {
 		case 0:		/* use index */
 			for( i=0; i<ncurves; i++ )
-				abscissa[i] = i+1;
+				abscissa[i] = i;
 			break;	
 		case 1:		/* use legend label */
 			cs = 0;
 			for( i=0; i<ncurves; i++ ){
-				while( !isactive_set( cg, cs ) )
+				while( !is_set_active( get_cg(), cs ) )
 					cs++;
-				if(!sscanf( g[cg].p[cs].lstr, "%f", &abscissa[i]))
+				if(!sscanf( get_legend_string(get_cg(), cs), "%f", &abscissa[i]))
 					break;
 				cs++;
 			}
@@ -233,7 +240,7 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 			}
 			break;
 		case 2:		/* use X from set */
-			if( !isactive_set( abs_graph, abs_set ) ){
+			if( !is_set_active( abs_graph, abs_set ) ){
 	    		errwin("Abscissa set not active");
 	    		return;
 			}
@@ -246,7 +253,7 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 				abscissa[i] = absy[i];
 			break;			
 		case 3:										/* use Y from set */
-			if( !isactive_set( abs_graph, abs_set ) ){
+			if( !is_set_active( abs_graph, abs_set ) ){
 	    		errwin("Abscissa set not active");
 	    		return;
 			}
@@ -263,58 +270,60 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 	cs = 0;
 	tbuf[0] = '\0';
 	for( i=0; i<ncurves; i++ ) {
-		while( !isactive_set( cg, cs ) )
+		while( !is_set_active( get_cg(), cs ) )
 			cs++;
 		extract_err = 0;
 			
-		switch( feature ) {
+		getsetminmax(get_cg(), cs, &xmin, &xmax, &ymin, &ymax);
+                switch( feature ) {
 			case 0:			/* Y minimum */
-				datum = g[cg].p[cs].ymin;		
+				datum = ymin;		
 				break;
 			case 1: 		/* Y maximum */
-				datum = g[cg].p[cs].ymax;		
+				datum = ymax;		
 				break;
 			case 2: 		/* Y mean    */
-				stasum(gety(cg, cs), getsetlength(cg, cs), &datum, &dummy, 0);
+				stasum(gety(cg, cs), getsetlength(cg, cs), &datum, &dummy);
 				break;
 			case 3:			/* Y std dev */
-				stasum(gety(cg, cs), getsetlength(cg, cs), &dummy, &datum, 0);
+				stasum(gety(cg, cs), getsetlength(cg, cs), &dummy, &datum);
 				break;
 			case 4: 		/* Y median  */
 				getmedian( cg, cs, DATA_Y, &datum );
 				break;
 			case 5:			/* X minimum */
-				datum = g[cg].p[cs].xmin;		
+				datum = xmin;		
 				break;
 			case 6: 		/* X maximum */
-				datum = g[cg].p[cs].xmax;		
+				datum = xmax;		
 				break;
 			case 7: 		/* X mean    */
-				stasum(getx(cg, cs), getsetlength(cg, cs), &datum, &dummy, 0);
+				stasum(getx(cg, cs), getsetlength(cg, cs), &datum, &dummy);
 				break;
 			case 8:			/* X std dev */
-				stasum(getx(cg, cs), getsetlength(cg, cs), &dummy, &datum, 0);
+				stasum(getx(cg, cs), getsetlength(cg, cs), &dummy, &datum);
 				break;
 			case 9:			/* X median  */
 				getmedian( cg, cs, DATA_X, &datum );
 				break;
 			case 10: 		/* frequency and period */
 			case 11:
-				if ( ilog2(getsetlength(cg, cs)) <= 0)    /* only DFT */
-					do_fourier(0, cs, 0, 1, 0, 0, 0);
-				else							/* FFT      */
-					do_fourier(1, cs, 0, 1, 0, 0, 0);
+				if ( ilog2(getsetlength(cg, cs)) <= 0)    /* only FFT */
+					do_fourier(cg, cs, 1, 0, 1, 0, 0, 0);
+				else		                                        /* DFT      */
+					do_fourier(cg, cs, 0, 0, 1, 0, 0, 0);
 
 				sprintf( tbuf, "FT of set %d", cs );
 				fts = 0;
-				while( strcmp( tbuf, g[cg].p[fts].comments+1 ) )
+				while( strcmp( tbuf, getcomment(cg, fts)+1 ) )
 					fts++;
 					
 				minmax(gety(cg, fts), getsetlength(cg, fts),&y1,&y2,&iy1,&iy2);
-				if( feature == 8 )
-					datum = g[cg].p[fts].ex[0][iy2-1];
+				x = getx(cg, fts);
+				if( feature == 10 )
+                                        datum = x[iy2];
 				else
-					datum = 1./g[cg].p[fts].ex[0][iy2-1];
+					datum = 1./x[iy2];
 				killset( cg, fts );				/* get rid of Fourier set */
 				break;
 			case 12:		/* first zero crossing */
@@ -327,8 +336,9 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 				}
 				break;
 			case 13:		/* rise time   */
-				if( get_rise_time( getsetlength(cg,cs), getx(cg,cs), 
-					gety(cg,cs), g[cg].p[cs].ymin, g[cg].p[cs].ymax, &datum ) ){
+				getsetminmax(cg, cs, &xmin, &xmax, &ymin, &ymax);
+                                if( get_rise_time( getsetlength(cg,cs), getx(cg,cs), 
+					gety(cg,cs), ymin, ymax, &datum ) ){
 					sprintf( tbuf+strlen(tbuf), 
 							"Unable to find rise time of set %d\n", cs );
 					errwin( tbuf );
@@ -336,8 +346,9 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 				}
 				break;
 			case 14: 		/* fall time   */
-				if( get_fall_time( getsetlength(cg,cs), getx(cg,cs), 
-					gety(cg,cs), g[cg].p[cs].ymin, g[cg].p[cs].ymax, &datum ) ){
+				getsetminmax(cg, cs, &xmin, &xmax, &ymin, &ymax);
+                                if( get_fall_time( getsetlength(cg,cs), getx(cg,cs), 
+					gety(cg,cs), ymin, ymax, &datum ) ){
 					sprintf( tbuf+strlen(tbuf), 
 									"Unable to find fall time of set %d\n", cs );
 					extract_err = 1;
@@ -366,8 +377,9 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 				datum = getsetlength( cg, cs );
 				break;
 			case 18:		/* half maximal widths */
-				if( get_half_max_width(getsetlength( cg, cs ), getx(cg,cs), 
-					   gety(cg,cs), g[cg].p[cs].ymin, g[cg].p[cs].ymax,&datum) ) {
+				getsetminmax(cg, cs, &xmin, &xmax, &ymin, &ymax);
+                                if( get_half_max_width(getsetlength( cg, cs ), getx(cg,cs), 
+					   gety(cg,cs), ymin, ymax,&datum) ) {
 					sprintf( tbuf+strlen(tbuf), 
 						"Unable to find half maximal width of set %d\n", cs );
 					extract_err = 1;
@@ -383,19 +395,21 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 									gety(cg,cs), &datum );
 				break;
 			case 21:		/* X of Maximum Y */
-				get_max_pos( gety(cg, cs), getx( cg, cs ),
-							getsetlength( cg, cs ), g[cg].p[cs].ymax, &datum ); 
+				getsetminmax(cg, cs, &xmin, &xmax, &ymin, &ymax);
+                                get_max_pos( gety(cg, cs), getx( cg, cs ),
+							getsetlength( cg, cs ), ymax, &datum ); 
 				break;
 			case 22:		/* Y of Maximum X */
-				get_max_pos( getx(cg, cs), gety( cg, cs ),
-							getsetlength( cg, cs ), g[cg].p[cs].xmax, &datum ); 
+				getsetminmax(cg, cs, &xmin, &xmax, &ymin, &ymax);
+                                get_max_pos( getx(cg, cs), gety( cg, cs ),
+							getsetlength( cg, cs ), xmax, &datum ); 
 				break;
 			case 23:		/* cumulative sum */
-				datum = do_int( cs, 1 );
+				datum = do_int(cg, cs, 1);
 				break;
 		}
 		if( !extract_err )
-			add_point( gto, ns, abscissa[i], datum, 0, 0, SET_XY );
+			add_point(gto, ns, abscissa[i], datum);
 		cs++;
 	}
 
@@ -474,9 +488,9 @@ void fext_routine( int gto, int feature, int abs_src, int abs_set, int abs_graph
 			sprintf(tbuf,"integrals of sets of graph %d",cg);
 			break;
 	}
-	setcomment( gto, ns, tbuf );
+	set_set_hidden(gto, ns, FALSE);
+        setcomment( gto, ns, tbuf );
 	free( abscissa );
-	plotone( gto );
 }
 
 
@@ -636,7 +650,7 @@ int getmedian( int grno, int setno, int sorton, double *median )
 	int setlen;
 	double *setdata;
 	
-	setlen = getsetlength( cg, setno );
+	setlen = getsetlength( get_cg(), setno );
 	setdata = (double *)malloc( setlen*sizeof(double) );
 	if( sorton == DATA_X )
 		memcpy( setdata, getx( grno, setno ), setlen*sizeof(double) );
