@@ -32,7 +32,6 @@
  *
  * Contents:
  *
- * void gauss() - simple gauss elimination for least squares poly fit
  * void stasum() - compute mean and variance
  * double leasev() - evaluate least squares polynomial
  * int  fitcurve() - compute coefficients for a polynomial fit of degree >1
@@ -62,55 +61,34 @@ static char buf[256];
 
 int dofitcurve(int cnt, double *xd, double *yd, int nd, double *c);
 
-/*
-	simple gauss elimination - no pivoting or scaling strategies
-	all matrices are sym-pos-def
-*/
-void gauss(int n, double *a, int adim, double *b, double *x)
-{
-
-    int i, k, j;
-    double mult;
-
-    for (k = 0; k < n - 1; k++) {
-	for (i = k + 1; i < n; i++) {
-	    mult = a[adim * i + k] / a[adim * k + k];
-	    for (j = k + 1; j < n; j++) {
-		a[adim * i + j] = a[adim * i + j] - mult * a[adim * k + j];
-	    }
-	    b[i] = b[i] - mult * b[k];
-	}
-    }
-    for (i = n - 1; i >= 0; i--) {
-	x[i] = b[i];
-	for (j = i + 1; j < n; j++)
-	    x[i] = x[i] - a[adim * i + j] * x[j];
-	x[i] = x[i] / a[adim * i + i];
-    }
-}
 
 /*
 	compute mean and standard dev
 */
-void stasum(double *x, int n, double *xbar, double *sd, int flag)
+void stasum(double *x, int n, double *xbar, double *sd)
 {
     int i;
 
     *xbar = 0;
     *sd = 0;
-    if (n < 1)
-	return;
-    for (i = 0; i < n; i++)
-	*xbar = (*xbar) + x[i];
-    *xbar = (*xbar) / n;
-    for (i = 0; i < n; i++)
-	*sd = (*sd) + (x[i] - *xbar) * (x[i] - *xbar);
-    if (n - flag)
-	*sd = sqrt(*sd / (n - flag));
-    else {
-	errmsg("compmean: (n-flag)==0");
-	*sd = 0;
+    
+    if (x == NULL) {
+        return;
     }
+    
+    if (n < 1) {
+	return;
+    }
+    
+    for (i = 0; i < n; i++) {
+        *xbar = (*xbar) + x[i];
+    }
+    *xbar = (*xbar)/n;
+    
+    for (i = 0; i < n; i++) {
+        *sd = (*sd) + (x[i] - *xbar) * (x[i] - *xbar);
+    }
+    *sd = sqrt(*sd/n);
 }
 
 
@@ -215,10 +193,10 @@ void runstddev(double *x, double *y, double *ax, double *ay, int n, int ilen)
 	sumx = sumx + x[i];
     }
     ax[0] = sumx / ilen;
-    stasum(y, ilen, &ybar, &ysd, 0);
+    stasum(y, ilen, &ybar, &ysd);
     ay[0] = ysd;
     for (i = 1; i < (n - ilen + 1); i++) {
-	stasum(y + i, ilen, &ybar, &ysd, 0);
+	stasum(y + i, ilen, &ybar, &ysd);
 	sumx = x[i + ilen - 1] - x[i - 1] + sumx;
 	ax[i] = sumx / ilen;
 	ay[i] = ysd;
@@ -364,10 +342,10 @@ int crosscorr(double *x, double *y, int n, int maxlag, double *xcov, double *xco
     if (maxlag + 2 > n)
 	return 1;
     for (i = 0; i <= maxlag; i++) {
-        stasum(&x[i], n - i, &xbar, &xsd, 0);
+        stasum(&x[i], n - i, &xbar, &xsd);
         if (xsd == 0.0)
 	    return 2;
-        stasum(y, n - i, &ybar, &ysd, 0);
+        stasum(y, n - i, &ybar, &ysd);
         if (ysd == 0.0)
 	    return 3;
 	xcov[i] = 0.0;
@@ -453,8 +431,10 @@ int linear_regression(int n, double *x, double *y, double *coeff)
     stufftext(buf, 0);
 
     if (n == 2) {
-        coeff[1] = (y[1]-y[0])/(x[1]-x[0]);
-        coeff[0] = y[0]-coeff[0]*x[0];
+        coeff[1] = (y[1] - y[0])/(x[1] - x[0]);
+        coeff[0] = y[0] - coeff[1]*x[0];
+        sprintf(buf, "Regression constant (INTERCEPT)\t\t = %.7g\n", intercept);
+        stufftext(buf, 0);
         return 0;
     } 
     
