@@ -4,7 +4,7 @@
  * Home page: http://plasma-gate.weizmann.ac.il/Grace/
  * 
  * Copyright (c) 1991-95 Paul J Turner, Portland, OR
- * Copyright (c) 1996-98 GRACE Development Team
+ * Copyright (c) 1996-99 Grace Development Team
  * 
  * Maintained by Evgeny Stambulchik <fnevgeny@plasma-gate.weizmann.ac.il>
  * 
@@ -35,14 +35,19 @@
 #include <config.h>
 
 #include <Xm/Xm.h>
+#include <Xm/Form.h>
 #include <Xm/DialogS.h>
 #include <Xm/RowColumn.h>
+#include <Xm/PanedW.h>
 
 #include "Tab.h"
 
-#include "globals.h"
-#include "protos.h"
+#include "graphs.h"
+#include "graphutils.h"
+#include "utils.h"
+#include "plotone.h"
 #include "motifinc.h"
+#include "protos.h"
 
 static Widget graphapp_dialog = NULL;
 static Widget graphapp_panel;
@@ -50,6 +55,8 @@ static Widget graphapp_panel;
 /*
  * Widget item declarations
  */
+static ListStructure *graph_selector;
+
 static Widget define_view_xv1;
 static Widget define_view_xv2;
 static Widget define_view_yv1;
@@ -57,25 +64,28 @@ static Widget define_view_yv2;
 
 static Widget *graph_type_choice_item;
 
-static Widget label_title_text_item;
-static Widget label_subtitle_text_item;
-static Widget *title_color_item;
-static OptionStructure title_font_item;
+static Widget stacked_item;
+
+static TextStructure *label_title_text_item;
+static TextStructure *label_subtitle_text_item;
+static OptionStructure *title_color_item;
+static OptionStructure *title_font_item;
 static Widget title_size_item;
-static Widget *stitle_color_item;
-static OptionStructure stitle_font_item;
+static OptionStructure *stitle_color_item;
+static OptionStructure *stitle_font_item;
 static Widget stitle_size_item;
 
 static Widget graph_flipxy_item;
 
+static SpinStructure *bargap_item;
+
 static Widget *frame_framestyle_choice_item;
-static Widget *frame_color_choice_item;
-static Widget *frame_pattern_choice_item;
-static Widget *frame_lines_choice_item;
-static Widget *frame_linew_choice_item;
-static Widget *frame_fillcolor_choice_item;
-static Widget *frame_fillpattern_choice_item;
-static Widget *frame_applyto_choice_item;
+static OptionStructure *frame_color_choice_item;
+static OptionStructure *frame_pattern_choice_item;
+static OptionStructure *frame_lines_choice_item;
+static SpinStructure *frame_linew_choice_item;
+static OptionStructure *frame_fillcolor_choice_item;
+static OptionStructure *frame_fillpattern_choice_item;
 
 static Widget legend_x_item;
 static Widget legend_y_item;
@@ -84,46 +94,67 @@ static Widget *toggle_legendloc_item;
 static Widget *legends_vgap_item;
 static Widget *legends_hgap_item;
 static Widget *legends_len_item;
-static OptionStructure legend_font_item;
+static Widget legends_invert_item;
+static OptionStructure *legend_font_item;
 static Widget legend_charsize_item;
-static Widget *legend_color_item;
-static Widget *legend_boxfillcolor_item;
-static Widget *legend_boxfillpat_item;
-static Widget *legend_boxlinew_item;
-static Widget *legend_boxlines_item;
-static Widget *legend_boxcolor_item;
-static Widget *legend_boxpattern_item;
+static OptionStructure *legend_color_item;
+static OptionStructure *legend_boxfillcolor_item;
+static OptionStructure *legend_boxfillpat_item;
+static SpinStructure *legend_boxlinew_item;
+static OptionStructure *legend_boxlines_item;
+static OptionStructure *legend_boxcolor_item;
+static OptionStructure *legend_boxpattern_item;
 
 /*
  * Event and Notify proc declarations
  */
-static void graphapp_aac_cb(Widget w, XtPointer client_data, XtPointer call_data);
+static void graphapp_aac_cb(void *data);
 static void updatelegends(int gno);
+static void update_view(int gno);
+static void update_frame_items(int gno);
+
+void update_graphapp_items(int n, int *values, void *data);
+
+void create_graphapp_frame_cb(void *data)
+{
+    create_graphapp_frame((int) data);
+}
 
 /*
  * 
  */
-void create_graphapp_frame(Widget w, XtPointer client_data, XtPointer call_data)
+void create_graphapp_frame(int gno)
 {
     Widget graphapp_tab, graphapp_frame, graphapp_main, graphapp_titles,
-         graphapp_legends, graphapp_legendbox, rc_head;
+         graphapp_legends, graphapp_legendbox;
     Widget rc, rc1, rc2, fr;
 
     set_wait_cursor();
     
+    if (is_valid_gno(gno) == FALSE) {
+        gno = get_cg();
+    }
+    
     if (graphapp_dialog == NULL) {
 	graphapp_dialog = XmCreateDialogShell(app_shell, "GraphAppearance", NULL, 0);
 	handle_close(graphapp_dialog);
         graphapp_panel = XtVaCreateWidget("graphapp_panel", xmFormWidgetClass, 
                                           graphapp_dialog, NULL, 0);
+/*
+ *         graphapp_panel = XtVaCreateWidget("graphapp_panel", xmPanedWindowWidgetClass, 
+ *                                           graphapp_dialog, NULL, 0);
+ */
 
-        rc_head = XmCreateRowColumn(graphapp_panel, "rc_head", NULL, 0);
-        rc = XmCreateRowColumn(rc_head, "rc", NULL, 0);
-        XtVaSetValues(rc, XmNorientation, XmHORIZONTAL, NULL);
-        CreateGraphChoice(rc, "Graph: (N/I)", maxgraph, 0);
-        XtManageChild(rc);
-        XtManageChild(rc_head);
-        XtVaSetValues(rc_head,
+/*
+ *         rc_head = XmCreateRowColumn(graphapp_panel, "rc_head", NULL, 0);
+ */
+        graph_selector = CreateGraphChoice(graphapp_panel, "Graph:",
+                            LIST_TYPE_MULTIPLE);
+        AddListChoiceCB(graph_selector, update_graphapp_items, NULL);
+/*
+ *         XtManageChild(rc_head);
+ */
+        XtVaSetValues(graph_selector->rc,
                       XmNtopAttachment, XmATTACH_FORM,
                       XmNleftAttachment, XmATTACH_FORM,
                       XmNrightAttachment, XmATTACH_FORM,
@@ -139,23 +170,32 @@ void create_graphapp_frame(Widget w, XtPointer client_data, XtPointer call_data)
         
         graphapp_main = CreateTabPage(graphapp_tab, "Main");
 
-	fr = CreateFrame(graphapp_main, NULL);
+	fr = CreateFrame(graphapp_main, "Presentation");
 	rc = XtVaCreateWidget("rc", xmRowColumnWidgetClass, fr, NULL);
 	graph_type_choice_item = CreatePanelChoice(rc, 
-                                                   "Graph type:",
-						   5,
+                                                   "Type:",
+						   6,
 						   "XY graph",
-						   "Bar chart (N/I)",
+						   "XY chart",
 						   "Polar graph",
 						   "Smith chart (N/I)",
+						   "Fixed",
 						   NULL,
 						   NULL);
+
+        rc1 = XtVaCreateWidget("rc", xmRowColumnWidgetClass, rc, NULL);
+	XtVaSetValues(rc1, XmNorientation, XmHORIZONTAL, NULL);
+	stacked_item = CreateToggleButton(rc1, "Stacked chart");
+        bargap_item = CreateSpinChoice(rc1, "Bar gap:", 5,
+            SPIN_TYPE_FLOAT, 0.0, 1.0, 0.005);
+        XtManageChild(rc1);
+        
         XtManageChild(rc);
 
 	fr = CreateFrame(graphapp_main, "Titles");
 	rc = XtVaCreateWidget("rc", xmRowColumnWidgetClass, fr, NULL);
-	label_title_text_item = CreateTextItem2(rc, 25, "Title:");
-	label_subtitle_text_item = CreateTextItem2(rc, 22, "Subtitle:");
+	label_title_text_item = CreateCSText(rc, "Title: ");
+	label_subtitle_text_item = CreateCSText(rc, "Subtitle: ");
         XtManageChild(rc);
 
         fr = CreateFrame(graphapp_main, "Viewport");
@@ -309,25 +349,19 @@ void create_graphapp_frame(Widget w, XtPointer client_data, XtPointer call_data)
 				             "0", "1", "2", "3", "4",
                                              "5", "6", "7", "8",
 					     0, 0);
+	legends_invert_item = CreateToggleButton(rc, "Put in reverse order");
 	XtManageChild(rc);
         
         
         SelectTabPage(graphapp_tab, graphapp_main);
 
 
-        fr = XmCreateFrame(graphapp_panel, "fr", NULL, 0); 
+        fr = CreateFrame(graphapp_panel, NULL); 
         rc = XmCreateRowColumn(fr, "rc", NULL, 0);
 
-	frame_applyto_choice_item = CreatePanelChoice(rc, "Apply to:",
-						      3,
-						      "Current graph",
-						      "All active graphs",
-						      NULL,
-						      NULL);
         CreateAACButtons(rc, graphapp_panel, graphapp_aac_cb);
         
         XtManageChild(rc);
-        XtManageChild(fr);
         XtVaSetValues(fr,
                       XmNtopAttachment, XmATTACH_NONE,
                       XmNleftAttachment, XmATTACH_FORM,
@@ -336,7 +370,7 @@ void create_graphapp_frame(Widget w, XtPointer client_data, XtPointer call_data)
                       NULL);
         XtVaSetValues(graphapp_tab,
                       XmNtopAttachment, XmATTACH_WIDGET,
-                      XmNtopWidget, rc_head,
+                      XmNtopWidget, graph_selector->rc,
                       XmNleftAttachment, XmATTACH_FORM,
                       XmNrightAttachment, XmATTACH_FORM,
                       XmNbottomAttachment, XmATTACH_WIDGET,
@@ -345,10 +379,16 @@ void create_graphapp_frame(Widget w, XtPointer client_data, XtPointer call_data)
 
         XtManageChild(graphapp_panel);
 
+#ifdef HAVE_LESSTIF
+        /* a kludge against Lesstif geometry calculation bug */
+        SelectTabPage(graphapp_tab, graphapp_legendbox);
+        XtRaise(graphapp_dialog);
+        SelectTabPage(graphapp_tab, graphapp_main);
+#endif
     }
     
+    SelectListChoice(graph_selector, gno);
     XtRaise(graphapp_dialog);
-    update_graphapp_items(cg);
     unset_wait_cursor();
 }
 
@@ -356,140 +396,176 @@ void create_graphapp_frame(Widget w, XtPointer client_data, XtPointer call_data)
  * Notify and event procs
  */
 
-static void graphapp_aac_cb(Widget w, XtPointer client_data, XtPointer call_data)
+static void graphapp_aac_cb(void *data)
 {
-    int i, ming, maxg;
-    int which, aac_mode;
+    int j, gno, n, *values;
+    int aac_mode;
     view v;
+    labels labs;
+    framep f;
+    legend l;
     int graphtype;
-    int flipxy;
-
-    aac_mode = (int) client_data;
+    int stacked;
+    double bargap;
+/*
+ *     int flipxy;
+ */
+    
+    aac_mode = (int) data;
     
     if (aac_mode == AAC_CLOSE) {
         XtUnmanageChild(graphapp_dialog);
         return;
     }
-
-    xv_evalexpr( define_view_xv1, &v.xv1 );  
-    xv_evalexpr( define_view_xv2, &v.xv2 );  
-    xv_evalexpr( define_view_yv1, &v.yv1 );  
-    xv_evalexpr( define_view_yv2, &v.yv2 );  
     
+    graphtype = GetChoice(graph_type_choice_item);
+    
+    xv_evalexpr(define_view_xv1, &v.xv1);  
+    xv_evalexpr(define_view_xv2, &v.xv2);  
+    xv_evalexpr(define_view_yv1, &v.yv1);  
+    xv_evalexpr(define_view_yv2, &v.yv2);  
     if (isvalid_viewport(v) == FALSE) {
         errmsg("Invalid viewport coordinates");
         return;
     } 
+
+    set_default_string(&labs.title);
+    set_plotstr_string(&labs.title, GetTextString(label_title_text_item));
+    labs.title.charsize = GetCharSizeChoice(title_size_item);
+    labs.title.color = GetOptionChoice(title_color_item);
+    labs.title.font = GetOptionChoice(title_font_item);
+
+    set_default_string(&labs.stitle);
+    set_plotstr_string(&labs.stitle, GetTextString(label_subtitle_text_item));
+    labs.stitle.charsize = GetCharSizeChoice(stitle_size_item);
+    labs.stitle.color = GetOptionChoice(stitle_color_item);
+    labs.stitle.font = GetOptionChoice(stitle_font_item);
     
-    graphtype = GetChoice(graph_type_choice_item);
+    f.type = GetChoice(frame_framestyle_choice_item);
+    f.pen.color = GetOptionChoice(frame_color_choice_item);
+    f.pen.pattern = GetOptionChoice(frame_pattern_choice_item);
+    f.linew = GetSpinChoice(frame_linew_choice_item);
+    f.lines = GetOptionChoice(frame_lines_choice_item);
+    f.fillpen.color = GetOptionChoice(frame_fillcolor_choice_item);
+    f.fillpen.pattern = GetOptionChoice(frame_fillpattern_choice_item);
+
+    l.charsize = GetCharSizeChoice(legend_charsize_item);
+    l.active = GetToggleButtonState(toggle_legends_item);
+    l.vgap = GetChoice(legends_vgap_item);
+    l.hgap = GetChoice(legends_hgap_item);
+    l.len = GetChoice(legends_len_item);
+    l.invert = GetToggleButtonState(legends_invert_item);
+    l.loctype = GetChoice(toggle_legendloc_item) ? COORD_VIEW : COORD_WORLD;
+    xv_evalexpr(legend_x_item, &l.legx);
+    xv_evalexpr(legend_y_item, &l.legy);
+    l.font = GetOptionChoice(legend_font_item);
+    l.color = GetOptionChoice(legend_color_item);
+    l.boxfillpen.color = GetOptionChoice(legend_boxfillcolor_item);
+    l.boxfillpen.pattern = GetOptionChoice(legend_boxfillpat_item);
+    l.boxpen.color = GetOptionChoice(legend_boxcolor_item);
+    l.boxpen.pattern = GetOptionChoice(legend_boxpattern_item);
+    l.boxlinew = GetSpinChoice(legend_boxlinew_item);
+    l.boxlines = GetOptionChoice(legend_boxlines_item);
+
+    stacked = GetToggleButtonState(stacked_item);
+
+    bargap = GetSpinChoice(bargap_item);
     
-    flipxy = GetToggleButtonState(graph_flipxy_item);
-
-    
-    which = GetChoice(frame_applyto_choice_item);
-    if (which == 0) {
-	ming = maxg = cg;
-    } else {
-	ming = 0;
-	maxg = maxgraph - 1;
-    }
-    
-    for (i = ming; i <= maxg; i++) {
-	if (isactive_graph(i)) {
-
-            g[i].type = graphtype;
-
-            set_plotstr_string(&g[i].labs.title, xv_getstr(label_title_text_item));
-            set_plotstr_string(&g[i].labs.stitle, xv_getstr(label_subtitle_text_item));
-
-            g[i].labs.title.charsize = GetCharSizeChoice(title_size_item);
-            g[i].labs.stitle.charsize = GetCharSizeChoice(stitle_size_item);
-
-            g[i].labs.title.color = GetChoice(title_color_item);
-            g[i].labs.stitle.color = GetChoice(stitle_color_item);
-            
-            g[i].labs.title.font = GetOptionChoice(title_font_item);
-            g[i].labs.stitle.font = GetOptionChoice(stitle_font_item);
-            
-            g[i].xyflip = flipxy;
-           
-            g[i].v = v;
-
-	    g[i].f.type = GetChoice(frame_framestyle_choice_item);
-	    g[i].f.pen.color = GetChoice(frame_color_choice_item);
-	    g[i].f.pen.pattern = GetChoice(frame_pattern_choice_item);
-	    g[i].f.linew = GetChoice(frame_linew_choice_item);
-	    g[i].f.lines = GetChoice(frame_lines_choice_item);
-	    g[i].f.fillpen.color = GetChoice(frame_fillcolor_choice_item);
-	    g[i].f.fillpen.pattern = GetChoice(frame_fillpattern_choice_item);
-
-	    g[i].l.charsize = GetCharSizeChoice(legend_charsize_item);
-	    g[i].l.active = GetToggleButtonState(toggle_legends_item);
-	    g[i].l.vgap = GetChoice(legends_vgap_item);
-	    g[i].l.hgap = GetChoice(legends_hgap_item);
-	    g[i].l.len = GetChoice(legends_len_item);
-	    g[i].l.loctype = GetChoice(toggle_legendloc_item) ? COORD_VIEW : COORD_WORLD;
-	    xv_evalexpr(legend_x_item, &g[i].l.legx);
-	    xv_evalexpr(legend_y_item, &g[i].l.legy);
-	    g[i].l.font = GetOptionChoice(legend_font_item);
-	    g[i].l.color = GetChoice(legend_color_item);
-	    g[i].l.boxfillpen.color = GetChoice(legend_boxfillcolor_item);
-	    g[i].l.boxfillpen.pattern = GetChoice(legend_boxfillpat_item);
-	    g[i].l.boxpen.color = GetChoice(legend_boxcolor_item);
-	    g[i].l.boxpen.pattern = GetChoice(legend_boxpattern_item);
-	    g[i].l.boxlinew = GetChoice(legend_boxlinew_item);
-	    g[i].l.boxlines = GetChoice(legend_boxlines_item);
+/*
+ *     flipxy = GetToggleButtonState(graph_flipxy_item);
+ */
 
+    n = GetListChoices(graph_selector, &values);
+    for (j = 0; j < n; j++) {
+	gno = values[j];
+        if (is_valid_gno(gno)) {
+            set_graph_type(gno, graphtype);
+            set_graph_stacked(gno, stacked);
+            set_graph_bargap(gno, bargap);
+            set_graph_viewport(gno, v);
+            set_graph_labels(gno, &labs);
+            set_graph_framep(gno, &f);
+            set_graph_legend(gno, &l);
+/*
+ *             g[gno].xyflip = flipxy;
+ */
 	}
     }
+    
+    xfree(values);
 
     if (aac_mode == AAC_ACCEPT) {
         XtUnmanageChild(graphapp_dialog);
     }
     
     drawgraph();
-    set_dirtystate();
 }
 
-void update_graphapp_items(int gno)
+void update_graphapp_items(int n, int *values, void *data)
 {
+    int gno;
+    labels labs;
+    
+    if (n != 1) {
+        return;
+    } else {
+        gno = values[0];
+    }
+
+    if (is_valid_gno(gno) != TRUE) {
+        return;
+    }
+    
     if (graphapp_dialog != NULL) {
+
         update_view(gno);
 
         update_frame_items(gno);
  
         updatelegends(gno);
+        get_graph_labels(gno, &labs);
+            
+        SetChoice(graph_type_choice_item, get_graph_type(gno));
+
+        SetSpinChoice(bargap_item, get_graph_bargap(gno));
+
+        SetToggleButtonState(stacked_item, is_graph_stacked(gno));
+
+        SetTextString(label_title_text_item, labs.title.s);
+        SetTextString(label_subtitle_text_item, labs.stitle.s);
  
-        SetChoice(graph_type_choice_item, g[gno].type);
+        SetCharSizeChoice(title_size_item, labs.title.charsize);
+        SetCharSizeChoice(stitle_size_item, labs.stitle.charsize);
 
-        xv_setstr(label_title_text_item, g[gno].labs.title.s);
-        xv_setstr(label_subtitle_text_item, g[gno].labs.stitle.s);
- 
-        SetCharSizeChoice(title_size_item, g[gno].labs.title.charsize);
-        SetCharSizeChoice(stitle_size_item, g[gno].labs.stitle.charsize);
+        SetOptionChoice(title_color_item, labs.title.color);
+        SetOptionChoice(stitle_color_item, labs.stitle.color);
 
-        SetChoice(title_color_item, g[gno].labs.title.color);
-        SetChoice(stitle_color_item, g[gno].labs.stitle.color);
+        SetOptionChoice(title_font_item, labs.title.font);
+        SetOptionChoice(stitle_font_item, labs.stitle.font);
 
-        SetOptionChoice(title_font_item, g[gno].labs.title.font);
-        SetOptionChoice(stitle_font_item, g[gno].labs.stitle.font);
-
-        SetToggleButtonState(graph_flipxy_item, g[gno].xyflip);
+/*
+ *         SetToggleButtonState(graph_flipxy_item, g[gno].xyflip);
+ */
     }
 }
 /*
  * Viewport update
  */
-void update_view(int gno)
+static void update_view(int gno)
 {
+    view v;
+    char buf[32];
+    
     if (graphapp_dialog) {
-	sprintf(buf, "%.9g", g[gno].v.xv1);
+	get_graph_viewport(gno, &v);
+        
+        sprintf(buf, "%.9g", v.xv1);
 	xv_setstr(define_view_xv1, buf);
-	sprintf(buf, "%.9g", g[gno].v.xv2);
+	sprintf(buf, "%.9g", v.xv2);
 	xv_setstr(define_view_xv2, buf);
-	sprintf(buf, "%.9g", g[gno].v.yv1);
+	sprintf(buf, "%.9g", v.yv1);
 	xv_setstr(define_view_yv1, buf);
-	sprintf(buf, "%.9g", g[gno].v.yv2);
+	sprintf(buf, "%.9g", v.yv2);
 	xv_setstr(define_view_yv2, buf);
     }
 }
@@ -499,56 +575,52 @@ void update_view(int gno)
  */
 static void updatelegends(int gno)
 {
+    legend l;
+    char buf[32];
+    
     if (graphapp_dialog != NULL) {
-	SetCharSizeChoice(legend_charsize_item, g[gno].l.charsize);
+	get_graph_legend(gno, &l);
+        
+        SetCharSizeChoice(legend_charsize_item, l.charsize);
 
-	SetToggleButtonState(toggle_legends_item, g[gno].l.active == TRUE);
+	SetToggleButtonState(toggle_legends_item, l.active == TRUE);
 
-	sprintf(buf, "%.9g", g[gno].l.legx);
+	sprintf(buf, "%.9g", l.legx);
 	xv_setstr(legend_x_item, buf);
-	sprintf(buf, "%.9g", g[gno].l.legy);
+	sprintf(buf, "%.9g", l.legy);
 	xv_setstr(legend_y_item, buf);
 
-	SetChoice(legends_vgap_item, g[gno].l.vgap);
-	SetChoice(legends_hgap_item, g[gno].l.hgap);
-	SetChoice(legends_len_item, g[gno].l.len);
+	SetChoice(legends_vgap_item, l.vgap);
+	SetChoice(legends_hgap_item, l.hgap);
+	SetChoice(legends_len_item, l.len);
+	SetToggleButtonState(legends_invert_item, l.invert);
 
-	SetChoice(toggle_legendloc_item, g[gno].l.loctype == COORD_VIEW);
-	SetOptionChoice(legend_font_item, g[gno].l.font);
-	SetChoice(legend_color_item, g[gno].l.color);
-	SetChoice(legend_boxfillcolor_item, g[gno].l.boxfillpen.color);
-	SetChoice(legend_boxfillpat_item, g[gno].l.boxfillpen.pattern);
-	SetChoice(legend_boxcolor_item, g[gno].l.boxpen.color);
-	SetChoice(legend_boxpattern_item, g[gno].l.boxpen.pattern);
-	SetChoice(legend_boxlinew_item, g[gno].l.boxlinew);
-	SetChoice(legend_boxlines_item, g[gno].l.boxlines);
+	SetChoice(toggle_legendloc_item, l.loctype == COORD_VIEW);
+	SetOptionChoice(legend_font_item, l.font);
+	SetOptionChoice(legend_color_item, l.color);
+	SetOptionChoice(legend_boxfillcolor_item, l.boxfillpen.color);
+	SetOptionChoice(legend_boxfillpat_item, l.boxfillpen.pattern);
+	SetOptionChoice(legend_boxcolor_item, l.boxpen.color);
+	SetOptionChoice(legend_boxpattern_item, l.boxpen.pattern);
+	SetSpinChoice(legend_boxlinew_item, l.boxlinew);
+	SetOptionChoice(legend_boxlines_item, l.boxlines);
     }
 }
 
-void update_frame_items(int gno)
+static void update_frame_items(int gno)
 {
+    framep f;
+    
     if (graphapp_dialog) {
-	SetChoice(frame_framestyle_choice_item, g[gno].f.type);
-	SetChoice(frame_color_choice_item, g[gno].f.pen.color);
-	SetChoice(frame_pattern_choice_item, g[gno].f.pen.pattern);
-	SetChoice(frame_linew_choice_item, g[gno].f.linew);
-	SetChoice(frame_lines_choice_item, g[gno].f.lines);
-	SetChoice(frame_fillcolor_choice_item, g[gno].f.fillpen.color);
-	SetChoice(frame_fillpattern_choice_item, g[gno].f.fillpen.pattern);
+        get_graph_framep(gno, &f);
+    
+	SetChoice(frame_framestyle_choice_item, f.type);
+	SetOptionChoice(frame_color_choice_item, f.pen.color);
+	SetOptionChoice(frame_pattern_choice_item, f.pen.pattern);
+	SetSpinChoice(frame_linew_choice_item, f.linew);
+	SetOptionChoice(frame_lines_choice_item, f.lines);
+	SetOptionChoice(frame_fillcolor_choice_item, f.fillpen.color);
+	SetOptionChoice(frame_fillpattern_choice_item, f.fillpen.pattern);
     }
 }
 
-
-/*
- * activate the legend location flag
- */
-void legend_loc_proc(Widget w, XtPointer client_data, XtPointer call_data)
-{
-    if (graphapp_dialog) {
-	g[cg].l.loctype = GetChoice(toggle_legendloc_item) ? COORD_VIEW : COORD_WORLD;
-    }
-    set_action(0);
-    set_action(LEG_LOC);
-    set_dirtystate();
-}
-
