@@ -4,7 +4,7 @@
  * Home page: http://plasma-gate.weizmann.ac.il/Grace/
  * 
  * Copyright (c) 1991-95 Paul J Turner, Portland, OR
- * Copyright (c) 1996-98 GRACE Development Team
+ * Copyright (c) 1996-99 Grace Development Team
  * 
  * Maintained by Evgeny Stambulchik <fnevgeny@plasma-gate.weizmann.ac.il>
  * 
@@ -38,133 +38,150 @@
 #include <stdlib.h>
 
 #include <Xm/Xm.h>
+#include <Xm/Form.h>
 #include <Xm/DialogS.h>
 #include <Xm/RowColumn.h>
 
 #include "globals.h"
 
+#include "graphs.h"
+#include "utils.h"
+#include "plotone.h"
 #include "motifinc.h"
-
 #include "protos.h"
 
-int cset = 0;                   /* the current set from the symbols panel */
+#define cg get_cg()
+
+#define SETAPP_STRIP_LEGENDS    0
+#define SETAPP_LOAD_COMMENTS    1
+#define SETAPP_ALL_COLORS       2
+#define SETAPP_ALL_SYMBOLS      3
+#define SETAPP_ALL_LINEW        4
+#define SETAPP_ALL_LINES        5
+#define SETAPP_ALL_BW           6
+
+static int cset = 0;            /* the current set from the symbols panel */
 
 static Widget setapp_dialog = NULL;
 
+static Widget duplegs_item;
+
+static OptionStructure *type_item;
 static Widget *toggle_symbols_item;
 static Widget symsize_item;
 static Widget symskip_item;
-static Widget *symcolor_item;
-static Widget *sympattern_item;
-static Widget *symfillcolor_item;
-static Widget *symfillpattern_item;
-static Widget *symlinew_item;
-static Widget *symlines_item;
+static OptionStructure *symcolor_item;
+static OptionStructure *sympattern_item;
+static OptionStructure *symfillcolor_item;
+static OptionStructure *symfillpattern_item;
+static SpinStructure *symlinew_item;
+static OptionStructure *symlines_item;
 static Widget symchar_item;
-static OptionStructure char_font_item;
+static OptionStructure *char_font_item;
 
-static Widget *toggle_color_item;
-static Widget *toggle_pattern_item;
-static Widget *toggle_width_item;
+static OptionStructure *toggle_color_item;
+static OptionStructure *toggle_pattern_item;
+static SpinStructure *toggle_width_item;
 static Widget dropline_item;
-static Widget *toggle_lines_item;
+static OptionStructure *toggle_lines_item;
 static Widget *toggle_linet_item;
 static Widget *toggle_filltype_item;
 static Widget *toggle_fillrule_item;
-static Widget *toggle_fillpat_item;
-static Widget *toggle_fillcol_item;
-static SetChoiceItem toggle_symset_item;
+static OptionStructure *toggle_fillpat_item;
+static OptionStructure *toggle_fillcol_item;
+static ListStructure *toggle_symset_item;
 static Widget baseline_item;
 static Widget *baselinetype_item;
 
-static Widget legend_str_panel;
+static TextStructure *legend_str_item;
 
 static Widget errbar_active_item;
+static Widget *errbar_ptype_item;
+static OptionStructure *errbar_color_item;
+static OptionStructure *errbar_pattern_item;
 static Widget errbar_size_item;
-static Widget *errbar_width_item;
-static Widget *errbar_lines_item;
-static Widget *errbar_type_item;
-static Widget *errbar_riserlinew_item;
-static Widget *errbar_riserlines_item;
+static SpinStructure *errbar_width_item;
+static OptionStructure *errbar_lines_item;
+static SpinStructure *errbar_riserlinew_item;
+static OptionStructure *errbar_riserlines_item;
+static Widget errbar_aclip_item;
+static SpinStructure *errbar_cliplen_item;
 
 static Widget avalue_active_item;
 static Widget *avalue_type_item;
-static OptionStructure avalue_font_item;
-static Widget *avalue_color_item;
+static OptionStructure *avalue_font_item;
+static OptionStructure *avalue_color_item;
 static Widget avalue_charsize_item ;
 static Widget avalue_angle_item;
-static Widget *avalue_format_item;
+static OptionStructure *avalue_format_item;
 static Widget *avalue_precision_item;
 static Widget avalue_offsetx;
 static Widget avalue_offsety;
 static Widget avalue_prestr;
 static Widget avalue_appstr;
 
-static void setapp_aac_cb(Widget w, XtPointer client_data, XtPointer call_data);
-static void setall_colors_proc(Widget w, XtPointer client_data, XtPointer call_data);
-static void setall_sym_proc(Widget w, XtPointer client_data, XtPointer call_data);
-static void setall_linew_proc(Widget w, XtPointer client_data, XtPointer call_data);
-static void set_cset_proc(Widget w, XtPointer client_data, XtPointer call_data);
-static void setall_linesty_proc(Widget w, XtPointer client_data, XtPointer call_data);
-static void set_bw_proc(Widget w, XtPointer client_data, XtPointer call_data);
-static void strip_leg_proc( Widget w, XtPointer client_data, XtPointer call_data);
-
 static void UpdateSymbols(int gno, int value);
+static void set_cset_proc(int n, int *values, void *data);
+static void setapp_aac_cb(void *data);
+static void setapp_data_proc(void *data);
 
 /*
  * create the symbols popup
  */
-void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
+void define_symbols_popup(void *data)
 {
+    int setno;
     Widget setapp_panel, setapp_tab, setapp_main, setapp_symbols, 
            setapp_line, setapp_errbar, setapp_avalue, rc_head, fr, rc, rc1, rc2;
-    Widget menubar, menupane, cascade;
+    Widget menubar, menupane;
 
     set_wait_cursor();
+    
+    setno = (int) data;
+    if (is_valid_setno(cg, setno) == TRUE) {
+        cset = setno;
+    }
+    
     if (setapp_dialog == NULL) {
         setapp_dialog = XmCreateDialogShell(app_shell, "SetAppearance", NULL, 0);
         handle_close(setapp_dialog);
         setapp_panel = XtVaCreateWidget("setapp_panel", xmFormWidgetClass, 
                                           setapp_dialog, NULL, 0);
 
-        menubar = CreateMenuBar(setapp_panel, "setappMenuBar", NULL);
+        menubar = CreateMenuBar(setapp_panel);
         
-        menupane = CreateMenu(menubar, "setappFileMenu", "File", 'F', NULL, NULL);
-        CreateMenuButton(menupane, "close", "Close", 'C',
-            (XtCallbackProc) setapp_aac_cb, (XtPointer) AAC_CLOSE, NULL);
+        menupane = CreateMenu(menubar, "File", 'F', FALSE);
+        CreateMenuButton(menupane, "Close", 'C',
+            setapp_aac_cb, (void *) AAC_CLOSE);
 
-        menupane = CreateMenu(menubar, "setappDataMenu", "Data", 'D', NULL, NULL);
+        menupane = CreateMenu(menubar, "Edit", 'E', FALSE);
 
-        CreateMenuButton(menupane, "allColors", "All colors", 'c',
-            (XtCallbackProc) setall_colors_proc, (XtPointer) NULL, 0);
-        CreateMenuButton(menupane, "allSymbols", "All symbols", 's',
-            (XtCallbackProc) setall_sym_proc, (XtPointer) NULL, 0);
-        CreateMenuButton(menupane, "allLineWidths", "All line widths", 'w',
-            (XtCallbackProc) setall_linew_proc, (XtPointer) NULL, 0);
-        CreateMenuButton(menupane, "allLineStyles", "All line styles", 'y',
-            (XtCallbackProc) setall_linesty_proc, (XtPointer) NULL, 0);
-        CreateMenuButton(menupane, "setBW", "Black & white", 'B',
-            (XtCallbackProc) set_bw_proc, (XtPointer) NULL, 0);
-        CreateMenuSeparator(menupane, "sep");
-        CreateMenuButton(menupane, "loadComments", "Load comments", 'm',
-            (XtCallbackProc) legend_load_proc, (XtPointer) NULL, 0);
-        CreateMenuButton(menupane, "stripLabels", "Strip labels", 'l',
-            (XtCallbackProc) strip_leg_proc, (XtPointer) NULL, 0);
+        CreateMenuButton(menupane, "Set different colors", 'c',
+            setapp_data_proc, (void *) SETAPP_ALL_COLORS);
+        CreateMenuButton(menupane, "Set different symbols", 's',
+            setapp_data_proc, (void *) SETAPP_ALL_SYMBOLS);
+        CreateMenuButton(menupane, "Set different line widths", 'w',
+            setapp_data_proc, (void *) SETAPP_ALL_LINEW);
+        CreateMenuButton(menupane, "Set different line styles", 'y',
+            setapp_data_proc, (void *) SETAPP_ALL_LINES);
+        CreateMenuButton(menupane, "Set black & white", 'B',
+            setapp_data_proc, (void *) SETAPP_ALL_BW);
+        CreateMenuSeparator(menupane);
+        CreateMenuButton(menupane, "Load comments", 'm',
+            setapp_data_proc, (void *) SETAPP_LOAD_COMMENTS);
+        CreateMenuButton(menupane, "Strip legends", 'l',
+            setapp_data_proc, (void *) SETAPP_STRIP_LEGENDS);
         
         
-        menupane = CreateMenu(menubar, "setappOptionsMenu", "Options", 'O', NULL, NULL);
+        menupane = CreateMenu(menubar, "Options", 'O', FALSE);
       
-        /* nonl_autol_item =  */CreateMenuToggle(menupane, "masterSwitch", "Master switch (N/I)", 'M',
-            (XtCallbackProc) NULL, (XtPointer) NULL, NULL);
+        duplegs_item = CreateMenuToggle(menupane,
+            "Duplicate legends", 'D', NULL, NULL);
+        CreateMenuToggle(menupane, "Color sync (N/I)", 's', NULL, NULL);
 
-        menupane = CreateMenu(menubar, "nonlHelpMenu", "Help", 'H', &cascade, NULL);
-        XtVaSetValues(menubar, XmNmenuHelpWidget, cascade, NULL);
+        menupane = CreateMenu(menubar, "Help", 'H', TRUE);
 
-        CreateMenuButton(menupane, "onSetAppearance", "On set appearance", 's',
-            (XtCallbackProc) HelpCB, (XtPointer) NULL, 0);
-
-        CreateMenuButton(menupane, "onContext", "On context", 'x',
-            (XtCallbackProc) ContextHelpCB, (XtPointer) NULL, 0);
+        CreateMenuButton(menupane, "On set appearance", 's', HelpCB, NULL);
         
         XtManageChild(menubar);
         XtVaSetValues(menubar,
@@ -174,17 +191,12 @@ void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
                       NULL);
 
 
-
         rc_head = XmCreateRowColumn(setapp_panel, "rc_head", NULL, 0);
 
-        toggle_symset_item = CreateSetSelector(rc_head, "Select set:",
-                                             SET_SELECT_ACTIVE,
-                                             FILTER_SELECT_NONE,
-                                             GRAPH_SELECT_CURRENT,
-                                             SELECTION_TYPE_MULTIPLE);
+        toggle_symset_item = CreateSetChoice(rc_head, "Select set:",
+                                                LIST_TYPE_MULTIPLE, TRUE);
 
-        XtAddCallback(toggle_symset_item.list, XmNextendedSelectionCallback,
-                      (XtCallbackProc) set_cset_proc, (XtPointer) 0);
+        AddListChoiceCB(toggle_symset_item, set_cset_proc, NULL);
 
         XtManageChild(rc_head);
         XtVaSetValues(rc_head,
@@ -204,9 +216,13 @@ void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
         
         setapp_main = CreateTabPage(setapp_tab, "Main");
 
+        fr = CreateFrame(setapp_main, "Set presentation");
+	type_item = CreateSetTypeChoice(fr, "Type:");
+
         rc2 = XmCreateRowColumn(setapp_main, "rc", NULL, 0);
         XtVaSetValues(rc2, XmNorientation, XmHORIZONTAL, NULL);
 
+
         fr = CreateFrame(rc2, "Symbol properties");
         rc = XtVaCreateWidget("symbolsbb", xmRowColumnWidgetClass, fr, NULL);
         toggle_symbols_item = CreatePanelChoice(rc,
@@ -252,9 +268,7 @@ void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
         XtManageChild(rc2);
 
         fr = CreateFrame(setapp_main, "Legend");
-        rc = XmCreateRowColumn(fr, "rc", NULL, 0);
-        legend_str_panel = CreateTextItem2(rc, 33, "String:");
-        XtManageChild(rc);
+        legend_str_item = CreateCSText(fr, "String:");
 
         fr = CreateFrame(setapp_main, "Display options");
         rc2 = XmCreateRowColumn(fr, "rc", NULL, 0);
@@ -288,8 +302,8 @@ void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
 
         fr = CreateFrame(setapp_symbols, "Extra");
         rc = XtVaCreateWidget("rc", xmRowColumnWidgetClass, fr, NULL);
-        symskip_item = CreateTextItem2(rc, 4, "Sym skip:");
-        char_font_item = CreateFontChoice(rc, "Font:");
+        symskip_item = CreateTextItem2(rc, 4, "Symbol skip:");
+        char_font_item = CreateFontChoice(rc, "Font for char symbol:");
         XtManageChild(rc);
 
 
@@ -380,12 +394,13 @@ void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
 	XtVaSetValues(rc2, XmNorientation, XmHORIZONTAL, NULL);
 	avalue_format_item = CreateFormatChoice(rc, "Format:");
         avalue_type_item = CreatePanelChoice(rc2, "Type:",
-                                             6,
+                                             7,
                                              "None",
                                              "X",
                                              "Y",
                                              "X, Y",
                                              "String",
+                                             "Z",
                                              NULL,
                                              0);
 	avalue_precision_item = CreatePrecisionChoice(rc2, "Precision:");
@@ -409,31 +424,46 @@ void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
         rc2 = XmCreateRowColumn(setapp_errbar, "rc", NULL, 0);
         XtVaSetValues(rc2, XmNorientation, XmHORIZONTAL, NULL);
 
-        fr = CreateFrame(rc2, "Bar line");
+        rc1 = XtVaCreateWidget("rc", xmRowColumnWidgetClass, rc2, NULL);
+
+        fr = CreateFrame(rc1, "Common");
+        rc = XtVaCreateWidget("rc", xmRowColumnWidgetClass, fr, NULL);
+        errbar_ptype_item = CreatePanelChoice(rc,
+                                             "Placement:",
+                                             4,
+                                             "Normal",
+                                             "Opposite",
+                                             "Both",
+                                             NULL,
+                                             0);
+	errbar_color_item = CreateColorChoice(rc, "Color:");
+	errbar_pattern_item = CreatePatternChoice(rc, "Pattern:");
+        XtManageChild(rc);
+
+        fr = CreateFrame(rc1, "Clipping");
+        rc = XtVaCreateWidget("rc", xmRowColumnWidgetClass, fr, NULL);
+	errbar_aclip_item = CreateToggleButton(rc, "Arrow clip");
+	errbar_cliplen_item = CreateSpinChoice(rc, "Max riser length",
+            3, SPIN_TYPE_FLOAT, 0.0, 10.0, 0.1);
+        XtManageChild(rc);
+
+        XtManageChild(rc1);
+
+        rc1 = XtVaCreateWidget("rc", xmRowColumnWidgetClass, rc2, NULL);
+
+        fr = CreateFrame(rc1, "Bar line");
         rc = XtVaCreateWidget("rc", xmRowColumnWidgetClass, fr, NULL);
         errbar_size_item = CreateCharSizeChoice(rc, "Size");
         errbar_width_item = CreateLineWidthChoice(rc, "Width:");
         errbar_lines_item = CreateLineStyleChoice(rc, "Style:");
         XtManageChild(rc);
 
-        rc1 = XtVaCreateWidget("rc", xmRowColumnWidgetClass, rc2, NULL);
-        fr = CreateFrame(rc1, "Placement");
-        rc = XtVaCreateWidget("rc", xmRowColumnWidgetClass, fr, NULL);
-        errbar_type_item = CreatePanelChoice(rc,
-                                             "Type:",
-                                             4,
-                                             "Both",
-                                             "Top/left",
-                                             "Bottom/right",
-                                             NULL,
-                                             0);
-        XtManageChild(rc);
-        
         fr = CreateFrame(rc1, "Riser line");
         rc = XtVaCreateWidget("rc", xmRowColumnWidgetClass, fr, NULL);
         errbar_riserlinew_item = CreateLineWidthChoice(rc, "Width:");
         errbar_riserlines_item = CreateLineStyleChoice(rc, "Style:");
         XtManageChild(rc);
+
         XtManageChild(rc1);
         
         XtManageChild(rc2);
@@ -442,12 +472,8 @@ void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
 
 
         fr = CreateFrame(setapp_panel, NULL); 
-        rc = XmCreateRowColumn(fr, "rc", NULL, 0);
-
-        CreateAACButtons(rc, setapp_panel, setapp_aac_cb);
+        CreateAACButtons(fr, setapp_panel, setapp_aac_cb);
         
-        XtManageChild(rc);
-        XtManageChild(fr);
         XtVaSetValues(fr,
                       XmNtopAttachment, XmATTACH_NONE,
                       XmNleftAttachment, XmATTACH_FORM,
@@ -474,138 +500,131 @@ void define_symbols_popup(Widget w, XtPointer client_data, XtPointer call_data)
 /*
  * define symbols for the current set
  */
-static void setapp_aac_cb(Widget w, XtPointer client_data, XtPointer call_data)
+static void setapp_aac_cb(void *data)
 {
     int aac_mode;
     int i;
-    int sym, symskip, symlinew, symlines;
-    int line, linet, color, pattern, wid;
+    int duplegs;
+    int type;
+    int sym, symskip, symlines;
+    double symlinew;
+    int line, linet, color, pattern;
+    double wid;
     int dropline, filltype, fillrule, fillpat, fillcol;
     int symcolor, sympattern, symfillcolor, symfillpattern;
     double symsize;
-    int etype_tmp, errbar_type;
     int baseline, baselinetype;
     Errbar errbar;
     AValue avalue;
     char symchar;
     int charfont;
+    plotarr p;
     
+    int setno;
     int *selset, cd;
-
-    aac_mode = (int) client_data;
+    
+    aac_mode = (int) data;
     
     if (aac_mode == AAC_CLOSE) {
         XtUnmanageChild(setapp_dialog);
         return;
     }
 
+    duplegs = GetToggleButtonState(duplegs_item);
+
+    type = GetOptionChoice(type_item);
     symsize = GetCharSizeChoice(symsize_item);
     sym = GetChoice(toggle_symbols_item);
-    color = GetChoice(toggle_color_item);
-    pattern = GetChoice(toggle_pattern_item);
-    wid = GetChoice(toggle_width_item);
+    color = GetOptionChoice(toggle_color_item);
+    pattern = GetOptionChoice(toggle_pattern_item);
+    wid = GetSpinChoice(toggle_width_item);
     baseline = GetToggleButtonState(baseline_item);
     baselinetype = GetChoice(baselinetype_item);
     dropline = GetToggleButtonState(dropline_item);
-    line = GetChoice(toggle_lines_item);
+    line = GetOptionChoice(toggle_lines_item);
     linet = GetChoice(toggle_linet_item);
     filltype = GetChoice(toggle_filltype_item);
     fillrule = GetChoice(toggle_fillrule_item);
-    fillpat = GetChoice(toggle_fillpat_item);
-    fillcol = GetChoice(toggle_fillcol_item);
+    fillpat = GetOptionChoice(toggle_fillpat_item);
+    fillcol = GetOptionChoice(toggle_fillcol_item);
     xv_evalexpri(symskip_item, &symskip);
-    symcolor = GetChoice(symcolor_item);
-    sympattern = GetChoice(sympattern_item);
-    symfillcolor = GetChoice(symfillcolor_item);
-    symfillpattern = GetChoice(symfillpattern_item);
-    symlinew = GetChoice(symlinew_item);
-    symlines = GetChoice(symlines_item);
+    symcolor = GetOptionChoice(symcolor_item);
+    sympattern = GetOptionChoice(sympattern_item);
+    symfillcolor = GetOptionChoice(symfillcolor_item);
+    symfillpattern = GetOptionChoice(symfillpattern_item);
+    symlinew = GetSpinChoice(symlinew_item);
+    symlines = GetOptionChoice(symlines_item);
     symchar = atoi(xv_getstr(symchar_item));
     charfont = GetOptionChoice(char_font_item);
     
     errbar.active = GetToggleButtonState(errbar_active_item);
-    errbar.length = GetCharSizeChoice(errbar_size_item);
-    errbar.linew = GetChoice(errbar_width_item);
-    errbar.lines = GetChoice(errbar_lines_item);
-    errbar.riser_linew = GetChoice(errbar_riserlinew_item);
-    errbar.riser_lines = GetChoice(errbar_riserlines_item);
+    errbar.barsize = GetCharSizeChoice(errbar_size_item);
+    errbar.linew = GetSpinChoice(errbar_width_item);
+    errbar.lines = GetOptionChoice(errbar_lines_item);
+    errbar.riser_linew = GetSpinChoice(errbar_riserlinew_item);
+    errbar.riser_lines = GetOptionChoice(errbar_riserlines_item);
         
     avalue.active = GetToggleButtonState(avalue_active_item);
     avalue.type = GetChoice(avalue_type_item);
     avalue.size = GetCharSizeChoice(avalue_charsize_item);
     avalue.font = GetOptionChoice(avalue_font_item);
-    avalue.color = GetChoice(avalue_color_item);
+    avalue.color = GetOptionChoice(avalue_color_item);
     avalue.angle = GetAngleChoice(avalue_angle_item);
-    avalue.format = GetChoice(avalue_format_item);
+    avalue.format = GetOptionChoice(avalue_format_item);
     avalue.prec = GetChoice(avalue_precision_item);
     strcpy(avalue.prestr, xv_getstr(avalue_prestr));
     strcpy(avalue.appstr, xv_getstr(avalue_appstr));
     xv_evalexpr(avalue_offsetx, &avalue.offset.x );
     xv_evalexpr(avalue_offsety, &avalue.offset.y);
                     
-    cd = GetSelectedSets(toggle_symset_item, &selset);
-    if (cd == SET_SELECT_ERROR) {
+    cd = GetListChoices(toggle_symset_item, &selset);
+    if (cd < 1) {
         errwin("No set selected");
         return;
     } else {
         for(i = 0; i < cd; i++) {
-            cset = selset[i];
-            g[cg].p[cset].symskip = symskip;
-            g[cg].p[cset].symsize = symsize;
-            g[cg].p[cset].symlinew = symlinew;
-            g[cg].p[cset].symlines = symlines;
-            g[cg].p[cset].symchar = symchar;
-            g[cg].p[cset].charfont = charfont;
-            g[cg].p[cset].filltype = filltype;
-            g[cg].p[cset].fillrule = fillrule;
-            g[cg].p[cset].setfillpen.pattern = fillpat;
-            g[cg].p[cset].setfillpen.color = fillcol;
-            strcpy(g[cg].p[cset].lstr, xv_getstr(legend_str_panel));
-            setplotsym(cg, cset, sym);
-            setplotlinet(cg, cset, linet);
-            setplotlines(cg, cset, line);
-            setplotlinew(cg, cset, wid);
-            setplotcolor(cg, cset, color);
-            setplotpattern(cg, cset, pattern);
-            setplotsymcolor(cg, cset, symcolor);
-            setplotsympattern(cg, cset, sympattern);
-            setplotsymfillcolor(cg, cset, symfillcolor);
-            setplotsymfillpattern(cg, cset, symfillpattern);
-            g[cg].p[cset].dropline = dropline;
-            g[cg].p[cset].baseline = baseline;
-            g[cg].p[cset].baseline_type = baselinetype;
-
-            etype_tmp = GetChoice(errbar_type_item);
-            switch (dataset_type(cg, cset)) {
-            case SET_XYDX:
-            case SET_XYDXDX:
-                if (etype_tmp == 0) {
-                    errbar_type = PLACE_BOTH;;
-                } else if (etype_tmp == 1) {
-                    errbar_type = PLACE_LEFT;
-                } else {
-                    errbar_type = PLACE_RIGHT;
-                }
-                break;
-            case SET_XYDY:
-            case SET_XYDYDY:
-                if (etype_tmp == 0) {
-                    errbar_type = PLACE_BOTH;;
-                } else if (etype_tmp == 1) {
-                    errbar_type = PLACE_TOP;
-                } else {
-                    errbar_type = PLACE_BOTTOM;
-                }
-                break;
-            default:
-                errbar_type = PLACE_BOTH;
-                break;
+            setno = selset[i];
+            get_graph_plotarr(get_cg(), setno, &p);
+            p.symskip = symskip;
+            p.symsize = symsize;
+            p.symlinew = symlinew;
+            p.symlines = symlines;
+            p.symchar = symchar;
+            p.charfont = charfont;
+            p.filltype = filltype;
+            p.fillrule = fillrule;
+            p.setfillpen.pattern = fillpat;
+            p.setfillpen.color = fillcol;
+            if (cd == 1 || duplegs) {
+                strcpy(p.lstr, GetTextString(legend_str_item));
             }
-            errbar.type = errbar_type;
+            p.sym = sym;
+            p.linet = linet;
+            p.lines = line;
+            p.linew = wid;
+            p.linepen.color = color;
+            p.linepen.pattern = pattern;
+            p.sympen.color = symcolor;
+            p.sympen.pattern = sympattern;
+            p.symfillpen.color = symfillcolor;
+            p.symfillpen.pattern = symfillpattern;
+            p.dropline = dropline;
+            p.baseline = baseline;
+            p.baseline_type = baselinetype;
+
+            errbar.ptype = GetChoice(errbar_ptype_item);
+            errbar.pen.color = GetOptionChoice(errbar_color_item);
+            errbar.pen.pattern = GetOptionChoice(errbar_pattern_item);
+            errbar.arrow_clip = GetToggleButtonState(errbar_aclip_item);
+            errbar.cliplen = GetSpinChoice(errbar_cliplen_item);
     
-            g[cg].p[cset].errbar = errbar;
-            g[cg].p[cset].avalue = avalue;
+            p.errbar = errbar;
+            p.avalue = avalue;
+            
+            set_graph_plotarr(get_cg(), setno, &p);
+            set_dataset_type(get_cg(), setno, type);
         }
+        free(selset);
     } 
 
     if (aac_mode == AAC_ACCEPT) {
@@ -623,234 +642,171 @@ static void setapp_aac_cb(Widget w, XtPointer client_data, XtPointer call_data)
  */
 static void UpdateSymbols(int gno, int value)
 {
-    int itmp;
+    int i;
     char val[24];
+    plotarr p;
 
     if ((cset == value) && (value != -1)) {
+        get_graph_plotarr(gno, cset, &p);
     
-        SetCharSizeChoice(symsize_item, g[gno].p[value].symsize);
-        sprintf(val, "%d", g[gno].p[value].symskip);
+        SetOptionChoice(type_item, p.type);
+        for (i = 0; i < type_item->nchoices; i++) {
+            if (settype_cols(type_item->options[i].value) ==
+                                            settype_cols(p.type)) {
+                XtSetSensitive(type_item->options[i].widget, True);
+            } else {
+                XtSetSensitive(type_item->options[i].widget, False);
+            }
+        }
+
+        SetCharSizeChoice(symsize_item, p.symsize);
+        sprintf(val, "%d", p.symskip);
         xv_setstr(symskip_item, val);
-        sprintf(val, "%d", g[gno].p[value].symchar);
+        sprintf(val, "%d", p.symchar);
         xv_setstr(symchar_item, val);
-        SetChoice(toggle_symbols_item, getsetplotsym(gno, value));
+        SetChoice(toggle_symbols_item, p.sym);
+        
+        SetOptionChoice(symcolor_item, p.sympen.color);
+        SetOptionChoice(sympattern_item, p.sympen.pattern);
+        SetOptionChoice(symfillcolor_item, p.symfillpen.color);
+        SetOptionChoice(symfillpattern_item, p.symfillpen.pattern);
+        SetSpinChoice(symlinew_item, p.symlinew);
+        SetOptionChoice(symlines_item, p.symlines);
+        
+        SetOptionChoice(char_font_item, p.charfont);        
+        
+        SetOptionChoice(toggle_color_item, p.linepen.color);
+        SetOptionChoice(toggle_pattern_item, p.linepen.pattern);
+        SetSpinChoice(toggle_width_item, p.linew);
+        SetToggleButtonState(dropline_item, p.dropline);
+        SetOptionChoice(toggle_lines_item, p.lines);
+        SetChoice(toggle_linet_item, p.linet);
+        SetChoice(toggle_filltype_item, p.filltype);
+        SetChoice(toggle_fillrule_item, p.fillrule);
+        SetOptionChoice(toggle_fillcol_item, p.setfillpen.color);
+        SetOptionChoice(toggle_fillpat_item, p.setfillpen.pattern);
+        
+        SetToggleButtonState(baseline_item, p.baseline);
+        SetChoice(baselinetype_item, p.baseline_type);
 
-        if (symcolorbug || (getsetplotsymcolor(gno, value) == -1)) {
-            setplotsymcolor(gno, value, getsetcolor(gno, value));
-        }
-        if (getsetplotsymfillcolor(gno, value) == -1) {
-            setplotsymfillcolor(gno, value, getsetplotsymcolor(gno, value));
-        }
+        SetTextString(legend_str_item, p.lstr);
         
-        SetChoice(symcolor_item, getsetplotsymcolor(gno, value));
-        SetChoice(sympattern_item, getsetplotsympattern(gno, value));
-        SetChoice(symfillcolor_item, getsetplotsymfillcolor(gno, value));
-        SetChoice(symfillpattern_item, getsetplotsymfillpattern(gno, value));
-        SetChoice(symlinew_item, g[gno].p[value].symlinew);
-        SetChoice(symlines_item, g[gno].p[value].symlines);
-        
-        SetOptionChoice(char_font_item, g[cg].p[value].charfont);        
-        
-        SetChoice(toggle_color_item, getsetcolor(gno, value));
-        SetChoice(toggle_pattern_item, getsetpattern(gno, value));
-        SetChoice(toggle_width_item, getsetlinew(gno, value));
-        SetToggleButtonState(dropline_item, g[gno].p[value].dropline);
-        SetChoice(toggle_lines_item, getsetlines(gno, value));
-        SetChoice(toggle_linet_item, getsetlinet(gno, value));
-        SetChoice(toggle_filltype_item, g[gno].p[value].filltype);
-        SetChoice(toggle_fillrule_item, g[gno].p[value].fillrule);
-        SetChoice(toggle_fillcol_item, g[gno].p[value].setfillpen.color);
-        SetChoice(toggle_fillpat_item, g[gno].p[value].setfillpen.pattern);
-        
-        SetToggleButtonState(baseline_item, g[gno].p[value].baseline);
-        SetChoice(baselinetype_item, g[gno].p[value].baseline_type);
+        SetToggleButtonState(errbar_active_item, p.errbar.active);
+        SetChoice(errbar_ptype_item, p.errbar.ptype);
+        SetOptionChoice(errbar_color_item, p.errbar.pen.color);
+        SetOptionChoice(errbar_pattern_item, p.errbar.pen.pattern);
+        SetToggleButtonState(errbar_aclip_item, p.errbar.arrow_clip);
+        SetSpinChoice(errbar_cliplen_item, p.errbar.cliplen);
+        SetSpinChoice(errbar_width_item, p.errbar.linew);
+        SetOptionChoice(errbar_lines_item, p.errbar.lines);
+        SetSpinChoice(errbar_riserlinew_item, p.errbar.riser_linew);
+        SetOptionChoice(errbar_riserlines_item, p.errbar.riser_lines);
+        SetCharSizeChoice(errbar_size_item, p.errbar.barsize);
 
-        xv_setstr(legend_str_panel, g[gno].p[value].lstr);
+        SetToggleButtonState(avalue_active_item, p.avalue.active);
+        SetChoice(avalue_type_item, p.avalue.type);
+        SetCharSizeChoice(avalue_charsize_item, p.avalue.size);
+        SetOptionChoice(avalue_font_item, p.avalue.font);
+        SetOptionChoice(avalue_color_item, p.avalue.color);
+        SetAngleChoice(avalue_angle_item, p.avalue.angle);
+        SetOptionChoice(avalue_format_item, p.avalue.format);
+        SetChoice(avalue_precision_item, p.avalue.prec);
         
-        switch (g[gno].p[value].errbar.type) {
-        case PLACE_BOTH:
-            itmp = 0;
-            break;
-        case PLACE_TOP:
-        case PLACE_LEFT:
-            itmp = 1;
-            break;
-        case PLACE_BOTTOM:
-        case PLACE_RIGHT:
-            itmp = 2;
-            break;
-        default:
-            itmp = 0;
-            break;
-        }
-        SetToggleButtonState(errbar_active_item, g[cg].p[value].errbar.active);
-        SetChoice(errbar_type_item, itmp);
-        SetChoice(errbar_width_item, g[gno].p[value].errbar.linew);
-        SetChoice(errbar_lines_item, g[gno].p[value].errbar.lines);
-        SetChoice(errbar_riserlinew_item, g[gno].p[value].errbar.riser_linew);
-        SetChoice(errbar_riserlines_item, g[gno].p[value].errbar.riser_lines);
-        SetCharSizeChoice(errbar_size_item, g[gno].p[value].errbar.length);
+        xv_setstr(avalue_prestr, p.avalue.prestr);
+        xv_setstr(avalue_appstr, p.avalue.appstr);
 
-        SetToggleButtonState(avalue_active_item, g[cg].p[value].avalue.active);
-        SetChoice(avalue_type_item, g[cg].p[value].avalue.type);
-        SetCharSizeChoice(avalue_charsize_item, g[cg].p[value].avalue.size);
-        SetOptionChoice(avalue_font_item, g[cg].p[value].avalue.font);
-        SetChoice(avalue_color_item, g[cg].p[value].avalue.color);
-        SetAngleChoice(avalue_angle_item, g[cg].p[value].avalue.angle);
-        SetChoice(avalue_format_item, g[cg].p[value].avalue.format);
-        SetChoice(avalue_precision_item, g[cg].p[value].avalue.prec);
-        
-        xv_setstr(avalue_prestr, g[cg].p[value].avalue.prestr);
-        xv_setstr(avalue_appstr, g[cg].p[value].avalue.appstr);
-
-        sprintf(val, "%f", g[gno].p[value].avalue.offset.x);
+        sprintf(val, "%f", p.avalue.offset.x);
         xv_setstr(avalue_offsetx, val);
-        sprintf(val, "%f", g[gno].p[value].avalue.offset.y);
+        sprintf(val, "%f", p.avalue.offset.y);
         xv_setstr(avalue_offsety, val);
+        
+/*
+ *         set_graph_plotarr(gno, cset, &p);
+ */
    }
 }
 
 
-static void set_cset_proc(Widget w, XtPointer client_data, XtPointer call_data)
+static void set_cset_proc(int n, int *values, void *data)
 {
-    int cd;
-    int *selsets;
-        
-    cd = GetSelectedSets(toggle_symset_item, &selsets);
-    if (cd != SET_SELECT_ERROR) {
-        cset = selsets[0];
+    if (n == 1) {
+        cset = values[0];
         UpdateSymbols(cg, cset);
     }
 }
 
 void updatesymbols(int gno, int setno)
-{
-    int cd;
+{    
+    if (gno != cg) {
+        return;
+    }
     
     if (setapp_dialog != NULL) { 
-        cd = SetSelectedSet(gno, setno, toggle_symset_item);
-        if (cd != SET_SELECT_ERROR) {
+        if (SelectListChoice(toggle_symset_item, setno) == GRACE_EXIT_SUCCESS) {
             cset = setno;
         }
     }
 }
 
-/*
- * legends
- */
 
-/*
- * strip leading pathname from comments
- */
-static void strip_leg_proc( Widget w, XtPointer client_data, XtPointer call_data)
+static void setapp_data_proc(void *data)
 {
-    int i;
+    int proc_type;
+    int *selset, cd;
+    int i, setno;
+    plotarr p;
+    int c = 0, bg = getbgcolor();
+    
+    proc_type = (int) data;
 
-    for (i = 0; i < g[cg].maxplot; i++) {
-        if ( isactive_set(cg, i) ) {
-            strcpy( g[cg].p[i].lstr, mybasename(g[cg].p[i].lstr) );
+    cd = GetListChoices(toggle_symset_item, &selset);
+    if (cd < 1) {
+        errmsg("No set selected");
+        return;
+    } else {
+        for(i = 0; i < cd; i++) {
+            setno = selset[i];
+            switch (proc_type) {
+            case SETAPP_STRIP_LEGENDS:
+                set_legend_string(cg, setno,
+                    mybasename(get_legend_string(cg, setno)));
+                break;
+            case SETAPP_LOAD_COMMENTS:
+                load_comments_to_legend(cg, setno);
+                break;
+            case SETAPP_ALL_COLORS:
+                while (c == bg || get_colortype(c) != COLOR_MAIN) {
+                    c++;
+                    c %= number_of_colors();
+                }
+                set_set_colors(cg, setno, c);
+                c++;
+                break;
+            case SETAPP_ALL_SYMBOLS:
+                get_graph_plotarr(cg, setno, &p);
+                p.sym = (i % (MAXSYM - 2)) + 1;
+                set_graph_plotarr(cg, setno, &p);
+                break;
+            case SETAPP_ALL_LINEW:
+                get_graph_plotarr(cg, setno, &p);
+                p.linew = ((i % (2*((int) MAX_LINEWIDTH) - 1)) + 1)/2.0;
+                set_graph_plotarr(cg, setno, &p);
+                break;
+            case SETAPP_ALL_LINES:
+                get_graph_plotarr(cg, setno, &p);
+                p.lines = (i % (number_of_linestyles() - 1)) + 1;
+                set_graph_plotarr(cg, setno, &p);
+                break;
+            case SETAPP_ALL_BW:
+                set_set_colors(cg, setno, 1);
+                break;
+            }
         }
+        
+        free(selset);
+        
+        UpdateSymbols(cg, cset);
+        set_dirtystate();
+        drawgraph();
     }
 }
-
-/*
- * load legend strings from set comments
- */
-void legend_load_proc(Widget w, XtPointer client_data, XtPointer call_data)
-{
-    int i;
-
-    for (i = 0; i < g[cg].maxplot; i++) {
-        if (isactive_set(cg, i)) {
-            strcpy(g[cg].p[i].lstr, g[cg].p[i].comments);
-        }
-    }
-    set_dirtystate();
-}
-
-
-
-/*
- * define colors incrementally
- */
-void setall_colors_proc(Widget w, XtPointer client_data, XtPointer call_data)
-{
-    int i, c;
-
-    for (i = 0; i < g[cg].maxplot; i++) {
-        if (isactive_set(cg, i)) {
-            c = (i % (number_of_colors() - 1)) + 1;
-            setplotcolor(cg, i, c);
-            setplotsymcolor(cg, i, c);
-            setplotsymfillcolor(cg, i, c);
-        }
-    }
-    UpdateSymbols(cg, cset);
-    drawgraph();
-}
-
-/*
- * define symbols incrementally mod 10
- */
-static void setall_sym_proc(Widget w, XtPointer client_data, XtPointer call_data)
-{
-    int i;
-
-    for (i = 0; i < g[cg].maxplot; i++) {
-        if (isactive_set(cg, i)) {
-            setplotsym(cg, i, (i % 10) + 2);
-        }
-    }
-    UpdateSymbols(cg, cset);
-    drawgraph();
-}
-
-/*
- * define linewidths incrementally mod 7
- */
-static void setall_linew_proc(Widget w, XtPointer client_data, XtPointer call_data)
-{
-    int i;
-
-    for (i = 0; i < g[cg].maxplot; i++) {
-        if (isactive_set(cg, i)) {
-            setplotlinew(cg, i, (i % 7) + 1);
-        }
-    }
-    UpdateSymbols(cg, cset);
-    drawgraph();
-}
-
-/*
- * define line styles incrementally mod 5
- */
-static void setall_linesty_proc(Widget w, XtPointer client_data, XtPointer call_data)
-{
-    int i;
-
-    for (i = 0; i < g[cg].maxplot; i++) {
-        if (isactive_set(cg, i)) {
-            setplotlines(cg, i, (i % 5) + 1);
-        }
-    }
-    UpdateSymbols(cg, cset);
-    drawgraph();
-}
-
-/*
- * make all lines black
- */
-static void set_bw_proc(Widget w, XtPointer client_data, XtPointer call_data)
-{
-    int i;
-
-    for (i = 0; i < g[cg].maxplot; i++) {
-        if (isactive_set(cg, i)) {
-            setplotcolor(cg, i, 1);
-            setplotsymcolor(cg, i, 1);
-            setplotsymfillcolor(cg, i, 1);
-        }
-    }
-    UpdateSymbols(cg, cset);
-    drawgraph();
-}
