@@ -1,10 +1,10 @@
 /*
- * Grace - Graphics for Exploratory Data Analysis
+ * Grace - GRaphing, Advanced Computation and Exploration of data
  * 
  * Home page: http://plasma-gate.weizmann.ac.il/Grace/
  * 
  * Copyright (c) 1991-95 Paul J Turner, Portland, OR
- * Copyright (c) 1996-98 GRACE Development Team
+ * Copyright (c) 1996-99 Grace Development Team
  * 
  * Maintained by Evgeny Stambulchik <fnevgeny@plasma-gate.weizmann.ac.il>
  * 
@@ -29,18 +29,21 @@
 #include <config.h>
 #include <cmath.h>
 
+#include <stdlib.h>
 #include <ctype.h>
 
 #include "defines.h"
-#include "globals.h"
+#include "draw.h"
 
 #include "utils.h"
+#include "files.h"
 #include "device.h"
 #include "t1fonts.h"
 
 #include "protos.h"
 
 static char LastEncodingFile[GR_MAXPATHLEN];
+static char EncodingFile[GR_MAXPATHLEN];
 static float lastExtent;
 static float lastSlant;
 static int bitmap_pad;
@@ -57,20 +60,29 @@ static FontDB *FontDBtable = NULL;
 int init_t1(void)
 {
     int i;
-    char buf[GR_MAXPATHLEN];
+    char buf[GR_MAXPATHLEN], *bufp;
     FILE *fd;
     static char **Encoding = NULL;
     
     /* Set search paths: */
-    sprintf(buf, "%s/fonts/type1", grace_home);
-    T1_SetFileSearchPath(T1_PFAB_PATH, buf);
-    T1_SetFileSearchPath(T1_AFM_PATH, buf);
-    sprintf(buf, "%s/fonts/enc", grace_home);
-    T1_SetFileSearchPath(T1_ENC_PATH, buf);
+    bufp = grace_path("fonts/type1");
+    if (bufp == NULL) {
+        return (GRACE_EXIT_FAILURE);
+    }
+    T1_SetFileSearchPath(T1_PFAB_PATH, bufp);
+    T1_SetFileSearchPath(T1_AFM_PATH, bufp);
+    bufp = grace_path("fonts/enc");
+    if (bufp == NULL) {
+        return (GRACE_EXIT_FAILURE);
+    }
+    T1_SetFileSearchPath(T1_ENC_PATH, bufp);
     
     /* Set font database: */
-    sprintf(buf, "%s/fonts/FontDataBase", grace_home);
-    T1_SetFontDataBase(buf);
+    bufp = grace_path("fonts/FontDataBase");
+    if (bufp == NULL) {
+        return (GRACE_EXIT_FAILURE);
+    }
+    T1_SetFontDataBase(bufp);
 
     /* Set log-level: */
     T1_SetLogLevel(T1LOG_DEBUG);
@@ -91,7 +103,7 @@ int init_t1(void)
         return (GRACE_EXIT_FAILURE);
     }
     
-    fd = fopen(buf, "r");
+    fd = grace_openr(bufp, SOURCE_DISK);
     if (fd == NULL) {
         return (GRACE_EXIT_FAILURE);
     }
@@ -115,12 +127,17 @@ int init_t1(void)
     
 
     Encoding = T1_LoadEncoding(T1_DEFAULT_ENCODING_FILE);
-    strcpy(LastEncodingFile, T1_DEFAULT_ENCODING_FILE);
-    
+    if (Encoding != NULL) {
+        strcpy(EncodingFile, T1_DEFAULT_ENCODING_FILE);
+    } else {
+        Encoding = T1_LoadEncoding(T1_FALLBACK_ENCODING_FILE);
+        strcpy(EncodingFile, T1_FALLBACK_ENCODING_FILE);
+    }
     if (Encoding != NULL) {
         T1_SetDefaultEncoding(Encoding);
+        strcpy(LastEncodingFile, EncodingFile);
     } else {
-        errmsg("Failed loading default encoding vector!");
+        return (GRACE_EXIT_FAILURE);
     }
     
     lastExtent = 1.0;
@@ -138,7 +155,6 @@ void update_t1(void)
     int i;
     
     float Slant = T1_DEFAULT_SLANT, Extent;
-    char EncodingFile[GR_MAXPATHLEN] = T1_DEFAULT_ENCODING_FILE;
     
     static char **Encoding = NULL;
     
@@ -166,7 +182,7 @@ void update_t1(void)
       	lastSlant = Slant;
     }
 
-    Extent = page_dpi_x/page_dpi_y;
+    Extent = 1.0;
     if (Extent != lastExtent) {
       	/* Delete all size dependent data */
       	for (i = 0; i < T1_Get_no_fonts(); i++) {
@@ -180,20 +196,6 @@ void update_t1(void)
     }    
 }
 
-/*
- * setfont - make font the current font to use for writing strings
- */
-void setfont(int font)
-{
-    draw_props.font = font;
-    return;
-}
-
-int getfont(void)
-{
-    return (draw_props.font);
-}
-
 void map_fonts(int map)
 {
     int i;
@@ -298,6 +300,30 @@ char *get_fontfilename(int font)
     return (T1_GetFontFileName(font));
 }
 
+char *get_afmfilename(int font)
+{
+    char *s, *s1;
+    static char buf[64];
+    int len;
+
+    s = T1_GetAfmFileName(font);
+    if (s == NULL) {
+        s = T1_GetFontFileName(font);
+        len = strlen(s);
+        s1 = s + (len - 1);
+        while(s1 && *s1 != '.') {
+            len--;
+            s1--;
+        }
+        strncpy(buf, s, len);
+        buf[len] = '\0';
+        strcat(buf, "afm");
+        return buf;
+    } else {
+        return s;
+    }
+}
+
 char *get_fontname(int font)
 {
     return (T1_GetFontName(font));
@@ -318,6 +344,8 @@ char *get_encodingscheme(int font)
     return (T1_GetEncodingScheme(font));
 }
 
+static T1_TMATRIX UNITY_MATRIX = {1.0, 0.0, 0.0, 1.0};
+
 GLYPH *GetGlyphString(int FontID, double Size, double Angle, int modflag, 
                                                             char *theString)
 {
@@ -330,7 +358,8 @@ GLYPH *GetGlyphString(int FontID, double Size, double Angle, int modflag,
     int LigDetect = 0;
     
     GLYPH *glyph;
-
+    T1_TMATRIX matrix, *matrixP;
+    
     char *ligtheString = '\0';
     char *succs, *ligs;
     char buf_char;
@@ -384,6 +413,13 @@ GLYPH *GetGlyphString(int FontID, double Size, double Angle, int modflag,
     } else {
         strcpy(ligtheString, theString);
     }
+    
+    if (Angle == 0.0){
+        matrixP = NULL;
+    } else {
+        matrix = UNITY_MATRIX;
+        matrixP = T1_RotateMatrix(&matrix, (float) Angle);
+    }
 
     dev = get_curdevice_props();
     if (dev.fontaa == TRUE) {
@@ -409,7 +445,7 @@ GLYPH *GetGlyphString(int FontID, double Size, double Angle, int modflag,
  
     	    delta_rgb.red   = (fg_rgb.red   - bg_rgb.red)   / (T1_AALEVELS - 1);
     	    delta_rgb.green = (fg_rgb.green - bg_rgb.green) / (T1_AALEVELS - 1);
-    	    delta_rgb.blue  = (fg_rgb.blue  - bg_rgb.blue) / (T1_AALEVELS - 1);
+    	    delta_rgb.blue  = (fg_rgb.blue  - bg_rgb.blue)  / (T1_AALEVELS - 1);
  
     	    for (i = 1; i < T1_AALEVELS - 1; i++) {
     		cmap.rgb.red   = bg_rgb.red + i*delta_rgb.red;
@@ -432,20 +468,14 @@ GLYPH *GetGlyphString(int FontID, double Size, double Angle, int modflag,
     			   aacolors[4]);
 
     	glyph = T1_AASetString(FontID, ligtheString, 0,
-    				   Space, modflag, (float) Size, (float) Angle);
+    				   Space, modflag, (float) Size, matrixP);
     } else {
     	glyph = T1_SetString(FontID, ligtheString, 0,
-    				   Space, modflag, (float) Size, (float) Angle);
+    				   Space, modflag, (float) Size, matrixP);
     }
  
     free(ligtheString);
  
-    if ((glyph == NULL) ||
-  	(glyph->metrics.ascent - glyph->metrics.descent == 0) ||
-  	(glyph->metrics.rightSideBearing - glyph->metrics.leftSideBearing == 0)) {
-        return NULL;
-    }
-
     return glyph;
 }
 
@@ -592,6 +622,8 @@ CompositeString *String2Composite(char *string)
                             break;
 	                case 'M':
                             new_gotomark = atoi(buf);
+		            new_vshift = 0.0;
+		            new_hshift = 0.0;
                             break;
                         }
                         i += (j + 2);
@@ -671,11 +703,10 @@ CompositeString *String2Composite(char *string)
 	        csbuf[nss].setmark = setmark;
                 setmark = MARK_NONE;
 	        csbuf[nss].gotomark = gotomark;
-	        csbuf[nss].s = malloc(strlen(ss) + 1);
 	        if (direction == STRING_DIRECTION_RL) {
                     reverse_string(ss);
                 }
-                strcpy(csbuf[nss].s, ss);
+	        csbuf[nss].s = copy_string(NULL, ss);
 	
                 nss++;
             }
@@ -697,7 +728,7 @@ CompositeString *String2Composite(char *string)
             gotomark = new_gotomark;
             if (gotomark >= 0) {
                 /* once a substring is manually advanced, all the following
-                 * subtrings will be advanced as well!
+                 * substrings will be advanced as well!
                  */
                 new_gotomark = MARK_NONE;
             }
@@ -714,16 +745,28 @@ CompositeString *String2Composite(char *string)
 /*
  * Convenience wrapper for T1_ConcatGlyphs()
  */
-GLYPH *CatGlyphs(GLYPH *dest_glyph, GLYPH *src_glyph, int x_off, int y_off)
+GLYPH *CatGlyphs(GLYPH *dest_glyph, GLYPH *src_glyph,
+    int x_off, int y_off, int advancing)
 {
     GLYPH *buf_glyph;
+    int mode;
     
     if (src_glyph == NULL) {
+        /* even if T1lib fails for some reason, don't miss the offsets! */
+        if (dest_glyph != NULL) {
+            dest_glyph->metrics.advanceX += x_off;
+            dest_glyph->metrics.advanceY += y_off;
+        }
         return (dest_glyph);
     }
     
     if (dest_glyph != NULL) {
-        buf_glyph = T1_ConcatGlyphs(dest_glyph, src_glyph, x_off, y_off);
+        if (advancing == TEXT_ADVANCING_RL) {
+            mode = T1_RIGHT_TO_LEFT;
+        } else {
+            mode = T1_DEFAULT;
+        }
+        buf_glyph = T1_ConcatGlyphs(dest_glyph, src_glyph, x_off, y_off, mode);
         if (buf_glyph != NULL) {
             T1_FreeGlyph(dest_glyph);
             dest_glyph = T1_CopyGlyph(buf_glyph);
@@ -742,12 +785,13 @@ void WriteString(VPoint vp, int rot, int just, char *theString)
     int hjust, vjust, just_type;
     float hfudge, vfudge;
     
-    double page_ipv, page_dpv_x, page_dpv_y;
+    double page_ipv, page_dpv;
  
     /* Variables for raster parameters */
     double Size, Angle = 0.0;
     int FontID;
     int modflag;
+    int text_advancing;
 
     int iglyph;
     GLYPH *glyph;
@@ -784,19 +828,13 @@ void WriteString(VPoint vp, int rot, int just, char *theString)
     
     dev = get_curdevice_props();
     
-    /* No patterned texts */
-    setpattern(1);
-    
-    cstring = String2Composite(theString);
-    
     /* inches per 1 unit of viewport */
     page_ipv = MIN2(page_width_in, page_height_in);
 
     /* dots per 1 unit of viewport */
-    page_dpv_x = page_ipv*page_dpi_x;
-    page_dpv_y = page_ipv*page_dpi_y;
+    page_dpv = page_ipv*page_dpi;
 
-    scale_factor = page_dpv_y * MAGIC_FONT_SCALE * getcharsize();
+    scale_factor = page_dpv * MAGIC_FONT_SCALE * getcharsize();
     if (scale_factor <= 0.0) {
         return;
     }
@@ -811,11 +849,14 @@ void WriteString(VPoint vp, int rot, int just, char *theString)
         cs_marks[gotomark].y = 0;
     }
     
+    cstring = String2Composite(theString);
+    
     first = FALSE;
     iglyph = 0;
     while (cstring[iglyph].s != NULL) {
         Size = scale_factor * cstring[iglyph].scale;
   	FontID = cstring[iglyph].font;
+        text_advancing = cstring[iglyph].advancing;
         modflag = T1_UNDERLINE * cstring[iglyph].underline |
                   T1_OVERLINE  * cstring[iglyph].overline;
 	glyph = GetGlyphString(FontID, Size, Angle, modflag, cstring[iglyph].s);
@@ -850,7 +891,7 @@ void WriteString(VPoint vp, int rot, int just, char *theString)
         v_off_last = v_off_buf;
         x_off = (int) rint(h_off*co - v_off*si);
         y_off = (int) rint(v_off*co + h_off*si);
-        CSglyph = CatGlyphs(CSglyph, glyph, x_off, y_off);
+        CSglyph = CatGlyphs(CSglyph, glyph, x_off, y_off, text_advancing);
         setmark = cstring[iglyph].setmark;
         if (CSglyph != NULL && setmark >= 0 && setmark < MAX_MARKS) {
             cs_marks[setmark].x = CSglyph->metrics.advanceX;
@@ -861,6 +902,11 @@ void WriteString(VPoint vp, int rot, int just, char *theString)
     if (CSglyph == NULL) {
         return;
     }
+    if (CSglyph->bits == NULL) {
+        /* a string containing only spaces or unrasterizable chars */
+        T1_FreeGlyph(CSglyph);
+        return;
+    }
     
     pinpoint_x = CSglyph->metrics.leftSideBearing;
     pinpoint_y = CSglyph->metrics.ascent;
@@ -910,15 +956,15 @@ void WriteString(VPoint vp, int rot, int just, char *theString)
     just_type = just & 020;
     switch (just_type) {
     case JUST_OBJECT:
-        justpoint_x = (int) (baseline_start_x + 
+        justpoint_x = (int) rint(baseline_start_x + 
                                 hfudge*(baseline_end_x - baseline_start_x));
-        justpoint_y = (int) (baseline_start_y + 
+        justpoint_y = (int) rint(baseline_start_y + 
                                 hfudge*(baseline_end_y - baseline_start_y));
         break;
     case JUST_BBOX:
-        justpoint_x = (int) (bbox_left_x + 
+        justpoint_x = (int) rint(bbox_left_x + 
                                 hfudge*(bbox_right_x - bbox_left_x));
-        justpoint_y = (int) (bbox_lower_y + 
+        justpoint_y = (int) rint(bbox_lower_y + 
                                 vfudge*(bbox_upper_y - bbox_lower_y));
         break;
     default:
@@ -932,25 +978,28 @@ void WriteString(VPoint vp, int rot, int just, char *theString)
     xshift = pinpoint_x - justpoint_x;
     yshift = pinpoint_y - justpoint_y;
     
-    vptmp.x = vp.x + (double) xshift/page_dpv_x;
-    vptmp.y = vp.y + (double) yshift/page_dpv_y;
+    vptmp.x = vp.x + (double) xshift/page_dpv;
+    vptmp.y = vp.y + (double) yshift/page_dpv;
 
     if (get_draw_mode() == TRUE) {
+        /* No patterned texts */
+        setpattern(1);
+    
         if (dev.devfonts == FONTSRC_BITMAP) {
             (*devputpixmap) (vptmp, pwidth, pheight, CSglyph->bits, 
                                 CSglyph->bpp, bitmap_pad, PIXMAP_TRANSPARENT);
         } else {
             if (devputtext == NULL) {
-                errmsg("Device has no fonts built-in");
+                errmsg("Device has no built-in fonts");
             } else {
                 vp_baseline_start.x = vptmp.x + 
-                    (double) (baseline_start_x - bbox_left_x)/page_dpv_x;
+                    (double) (baseline_start_x - bbox_left_x)/page_dpv;
                 vp_baseline_start.y = vptmp.y +
-                    (double) (baseline_start_y - bbox_upper_y)/page_dpv_y;
+                    (double) (baseline_start_y - bbox_upper_y)/page_dpv;
                 vp_baseline_end.x   = vptmp.x + 
-                    (double) (baseline_end_x - bbox_left_x)/page_dpv_x;
+                    (double) (baseline_end_x - bbox_left_x)/page_dpv;
                 vp_baseline_end.y   = vptmp.y +
-                    (double) (baseline_end_y - bbox_upper_y)/page_dpv_y;
+                    (double) (baseline_end_y - bbox_upper_y)/page_dpv;
                 (*devputtext) (vp_baseline_start, vp_baseline_end, 
                                                 scale_factor, cstring);
             }
@@ -960,7 +1009,7 @@ void WriteString(VPoint vp, int rot, int just, char *theString)
     T1_FreeGlyph(CSglyph);
     
     update_bboxes(vptmp);
-    vptmp.x += (double) pwidth/page_dpv_x;
-    vptmp.y -= (double) pheight/page_dpv_y;
+    vptmp.x += (double) pwidth/page_dpv;
+    vptmp.y -= (double) pheight/page_dpv;
     update_bboxes(vptmp);
 }
