@@ -1,10 +1,10 @@
 /*
- * Grace - Graphics for Exploratory Data Analysis
+ * Grace - GRaphing, Advanced Computation and Exploration of data
  * 
  * Home page: http://plasma-gate.weizmann.ac.il/Grace/
  * 
  * Copyright (c) 1991-95 Paul J Turner, Portland, OR
- * Copyright (c) 1996-98 GRACE Development Team
+ * Copyright (c) 1996-99 Grace Development Team
  * 
  * Maintained by Evgeny Stambulchik <fnevgeny@plasma-gate.weizmann.ac.il>
  * 
@@ -27,22 +27,8 @@
  */
 
 /*
- *
  * misc utilities
  *
- * Contents:
- *
- * void cxfree() - free and check for NULL pointer
- * void fswap()  - swap doubles
- * void iswap()  - swap ints
- * void lowtoupper() - convert a string to upper case
- * void convertchar() - remove commas and Fortran D format
- * int ilog2() - integer log base 2, for the fft routine
- * double comp_area() - compute the area of a polygon
- * double comp_perimeter() - compute the perimeter
- * double fmin(), fmax()
- * Julian date routines
- *
  */
 
 #include <config.h>
@@ -50,12 +36,38 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <ctype.h>
 #include <string.h>
+#include <pwd.h>
+#ifdef TIME_WITH_SYS_TIME
+#  include <sys/time.h>
+#  include <time.h>
+#else
+#  ifdef HAVE_SYS_TIME_H
+#    include <sys/time.h>
+#  else
+#    include <time.h>
+#  endif
+#endif
 #include <signal.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/resource.h>
+#ifdef HAVE_SYS_SELECT_H
+#  include <sys/select.h>
+#endif
+#include <limits.h>
+
+#ifdef HAVE_SETLOCALE
+#  include <locale.h>
+#endif
+
+#include "buildinfo.h"
 
 #include "globals.h"
 #include "utils.h"
+#include "files.h"
 #include "protos.h"
 
 static void rereadConfig(void);
@@ -65,7 +77,7 @@ static void bugwarn(char *signame);
 /*
  * free and check for NULL pointer
  */
-void cxfree(void *ptr)
+void xfree(void *ptr)
 {
     if (ptr != NULL) {
 	free(ptr);
@@ -74,18 +86,28 @@ void cxfree(void *ptr)
 
 void *xrealloc(void *ptr, size_t size)
 {
+    void *retval;
+
 #if defined(REALLOC_IS_BUGGY)
     if (ptr == NULL) {
-        return malloc(size);
+        retval = malloc(size);
     } else if (size == 0) {
         free(ptr);
-        return NULL;
+        retval = NULL;
     } else {
-        return realloc(ptr, size); 
+        retval = realloc(ptr, size); 
     }
 #else
-    return realloc(ptr, size); 
+    retval = realloc(ptr, size);
+    if (size == 0) {
+        retval = NULL;
+    }
 #endif
+    
+    if (retval == NULL && size != 0) {
+        errmsg("Memory storage exceeded!");
+    }
+    return retval;
 }
 
 /*
@@ -109,15 +131,6 @@ void iswap(int *x, int *y)
     *y = tmp;
 }
 
-void vpswap(VPoint *vp1, VPoint *vp2)
-{
-    VPoint vptmp;
-
-    vptmp = *vp1;
-    *vp1 = *vp2;
-    *vp2 = vptmp;
-}
-
 int isoneof(int c, char *s)
 {
     while (*s) {
@@ -150,19 +163,23 @@ int argmatch(char *s1, char *s2, int atleast)
  */
 void lowtoupper(char *s)
 {
-    int i, quoteon = 0;
+    int i, quoteon = FALSE;
 
     for (i = 0; i < strlen(s); i++) {
 	if (s[i] == '"') {
 	    if (!quoteon) {
-		quoteon = 1;
+		quoteon = TRUE;
 	    } else if ((i > 0) && (s[i-1] != '\\')) {
-		quoteon = 0;
+		quoteon = FALSE;
 	    }
 	}
-	if (s[i] >= 'a' && s[i] <= 'z' && !quoteon) {
-	    s[i] -= ' ';
-	}
+	if (quoteon == FALSE) {
+            if (!isprint(s[i])) {
+                s[i] = ' ';
+            } else if (s[i] >= 'a' && s[i] <= 'z') {
+	        s[i] -= ' ';
+	    }
+        }
     }
 }
 
@@ -223,186 +240,16 @@ double comp_perimeter(int n, double *x, double *y)
     return sum;
 }
 
-/*
- * Time and date routines
- */
-
 char *dayofweekstrs[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
 char *dayofweekstrl[] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
 char *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 char *monthl[] = {"January", "February", "March", "April", "May", "June",
 "July", "August", "September", "October", "November", "December"};
 
-static int days1[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365};
-static int days2[] = {0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366};
-
-/*
- * return the Julian day + hms as a real number
- */
-
-/*
-** Takes a date, and returns a Julian day. A Julian day is the number of
-** days since some base date  (in the very distant past).
-** Handy for getting date of x number of days after a given Julian date
-** (use jdate to get that from the Gregorian date).
-** Author: Robert G. Tantzen, translator: Nat Howard
-** Translated from the algol original in Collected Algorithms of CACM
-** (This and jdate are algorithm 199).
-*/
-double julday(int mon, int day, int year, int h, int mi, double se)
-{
-    long m = mon, d = day, y = year;
-    long c, ya, j;
-    double seconds = h * 3600.0 + mi * 60 + se;
-
-    if (m > 2)
-	m -= 3;
-    else {
-	m += 9;
-	--y;
-    }
-    c = y / 100L;
-    ya = y - (100L * c);
-    j = (146097L * c) / 4L + (1461L * ya) / 4L + (153L * m + 2L) / 5L + d + 1721119L;
-    if (seconds < 12 * 3600.0) {
-	j--;
-	seconds += 12.0 * 3600.0;
-    } else {
-	seconds = seconds - 12.0 * 3600.0;
-    }
-    return (j + (seconds / 3600.0) / 24.0);
-}
-
-/* Julian date converter. Takes a julian date (the number of days since
-** some distant epoch or other), and returns an int pointer to static space.
-** ip[0] = month;
-** ip[1] = day of month;
-** ip[2] = year (actual year, like 1977, not 77 unless it was  77 a.d.);
-** ip[3] = day of week (0->Sunday to 6->Saturday)
-** These are Gregorian.
-** Copied from Algorithm 199 in Collected algorithms of the CACM
-** Author: Robert G. Tantzen, Translator: Nat Howard
-*/
-void calcdate(double jd, int *m, int *d, int *y, int *h, int *mi, double *sec)
-{
-    static int ret[4];
-
-    long j = jd;
-    double tmp, frac = jd - j;
-
-    if (frac >= 0.5) {
-	frac = frac - 0.5;
-	j++;
-    } else {
-	frac = frac + 0.5;
-    }
-
-    ret[3] = (j + 1L) % 7L;
-    j -= 1721119L;
-    *y = (4L * j - 1L) / 146097L;
-    j = 4L * j - 1L - 146097L * *y;
-    *d = j / 4L;
-    j = (4L * *d + 3L) / 1461L;
-    *d = 4L * *d + 3L - 1461L * j;
-    *d = (*d + 4L) / 4L;
-    *m = (5L * *d - 3L) / 153L;
-    *d = 5L * *d - 3 - 153L * *m;
-    *d = (*d + 5L) / 5L;
-    *y = 100L * *y + j;
-    if (*m < 10)
-	*m += 3;
-    else {
-	*m -= 9;
-	*y += 1;
-    }
-    tmp = 3600.0 * (frac * 24.0);
-    *h = (int) (tmp / 3600.0);
-    tmp = tmp - *h * 3600.0;
-    *mi = (int) (tmp / 60.0);
-    *sec = tmp - *mi * 60.0;
-}
-
 int dayofweek(double j)
 {
-    j += 0.5;
-    return (int) (j + 1) % 7;
-}
-
-int leapyear(int year)
-{
-    if (year % 400 == 0)
-        return (TRUE);
-    else if (year % 100 == 0)
-        return (FALSE);
-    else if (year % 4 == 0)
-        return (TRUE);
-    else
-        return (FALSE);
-}
-
-/*
-   get the month and day given the number of days
-   from the beginning of the year 'yr'
-*/
-void getmoday(int days, int yr, int *mo, int *da)
-{
-    int i;
-
-    if (leapyear(yr)) {
-	for (i = 0; i < 13; i++) {
-	    if (days <= days2[i]) {
-		*mo = i;
-		*da = (days - days2[i - 1]);
-		goto out1;
-	    }
-	}
-    } else {
-	for (i = 0; i < 13; i++) {
-	    if (days <= days1[i]) {
-		*mo = i;
-		*da = (days - days1[i - 1]);
-		goto out1;
-	    }
-	}
-    }
-out1:;
-}
-
-/*
-   return the number of days from the beginning of the year 'yr'
-*/
-int getndays(double j)
-{
-    int m, d, y, hh, mm;
-    double ss;
-
-    calcdate(j, &m, &d, &y, &hh, &mm, &ss);
-    if (leapyear(y)) {
-	return days2[m - 1] + d;
-    } else {
-	return days1[m - 1] + d;
-    }
-}
-
-/*
- * strip special chars from a string
- */
-void stripspecial(char *s, char *cs)
-{
-    int i, slen = strlen(s), curcnt = 0;
-
-    for (i = 0; i < slen; i++) {
-	if (s[i] == '\\' && isdigit(s[i + 1])) {
-	    i++;
-	} else if (s[i] == '\\' && isoneof(s[i + 1], "cCbxsSNuU+-")) {
-	    i++;
-	} else if (s[i] == '\\' && s[i + 1] == '\\') {
-	    i++;
-	} else {
-	    cs[curcnt++] = s[i];
-	}
-    }
-    cs[curcnt] = 0;
+    int i = (int) floor(j + 1.5);
+    return (i <= 0) ? 6 - (6 - i)%7 : i%7;
 }
 
 /*
@@ -452,14 +299,23 @@ int sign(double a)
     }
 }
 
+double mytrunc(double a)
+{
+    if (a > 0.0) {
+        return floor(a);
+    } else {
+        return ceil(a);
+    }
+}
+
 /*
  * exit grace
  */
 void bailout(void)
 {
-    if (!is_dirtystate() || yesno("Exit losing unsaved changes?", NULL, NULL, "file.html#exit")) {
+    if (!is_dirtystate() || yesno("Exit losing unsaved changes?", NULL, NULL, NULL)) {
          if (resfp) {
-             filter_close(resfp);
+             grace_close(resfp);
          }
          exit(0);
     }
@@ -470,6 +326,7 @@ void bailout(void)
  */
 static void rereadConfig(void)
 {
+    getparms("gracerc");
 }
 
 /*
@@ -477,16 +334,49 @@ static void rereadConfig(void)
  */
 static void bugwarn(char *signame)
 {
-    fprintf(stderr, "\a\nOops! Got %s. Please use \"Help/Comments\" to report the bug.\n", signame);
-    exit(GRACE_EXIT_FAILURE);
+    static int emergency_save = FALSE;
+/*
+ *  Since we got so far, memory is probably corrupted so it's better to use
+ *  a static storage
+ */
+    static char buf[GR_MAXPATHLEN];
+/* number of interrupts received during the emergency save */
+    static int interrupts;
+    
+    if (emergency_save != FALSE) {
+        /* don't mind signals anymore: we're in emergency save mode already */
+        interrupts++;
+        if (interrupts > 10) {
+            fprintf(stderr, "oh, no luck :-(\n");
+            fprintf(stderr, "\nPlease use \"Help/Comments\" to report the bug.\n");
+            exit(GRACE_EXIT_FAILURE);
+        }
+        return;
+    } else {
+        emergency_save = TRUE;
+        interrupts = 0;
+        fprintf(stderr, "\a\nOops! Got %s\n", signame);
+        if (is_dirtystate()) {
+            strcpy(buf, get_docname());
+            strcat(buf, "$");
+            fprintf(stderr, "Trying to save your work into file \"%s\"... ", buf);
+            fflush(stderr);
+            noask = TRUE;
+            if (save_project(buf) == GRACE_EXIT_SUCCESS) {
+                fprintf(stderr, "ok!\n");
+            } else {
+                fprintf(stderr, "oh, no luck :-(\n");
+            }
+        }
+        fprintf(stderr, "\nPlease use \"Help/Comments\" to report the bug.\n");
+        exit(GRACE_EXIT_FAILURE);
+    }
 }
 
 /*
  * Signal-handling routines
  */
  
-void installSignal();
-
 static RETSIGTYPE actOnSignal(int signo)
 {
     char signame[16];
@@ -571,6 +461,10 @@ void installSignal(void){
 #endif
 #ifdef SIGTERM
     signal(SIGTERM, actOnSignal);   /* software termination signal */
+#endif
+    signal(SIGALRM, actOnSignal);  /* timer */
+#ifdef SIGIO
+    signal(SIGIO, actOnSignal);     /* input/output ready */
 #endif
 }
 
@@ -580,15 +474,13 @@ char *create_fstring(int form, int prec, double loc, int type)
     char format[64], eng_prefix[6];
     static char s[MAX_STRING_LENGTH];
     double tmp;
-    int m, d, y, h, mm;
-    double sec;
-    int itmp;
+    int m, d, y, h, mm, sec;
+    double arcmin, arcsec;
     int exponent;
     double mantissa;
-    extern char *months[];
-    extern char *monthl[];
-    extern char *dayofweekstrs[];
-    extern char *dayofweekstrl[];
+
+    /* for locale decimal points */
+    set_locale_num(TRUE);
 
     strcpy(format, "%.*lf");
     switch (form) {
@@ -718,133 +610,108 @@ char *create_fstring(int form, int prec, double loc, int type)
 	}
 	break;
     case FORMAT_DDMMYY:
-	strcpy(format, "%d-%d-%d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
-	if (y >= 1900 && y < 2000) {
-	    y -= 1900;
-	}
+	strcpy(format, "%02d-%02d-%d");
+	jul_to_cal_and_time(loc, ROUND_DAY, &y, &m, &d, &h, &mm, &sec);
 	sprintf(s, format, d, m, y);
 	break;
     case FORMAT_MMDDYY:
-	strcpy(format, "%d-%d-%d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
-	if (y >= 1900 && y < 2000) {
-	    y -= 1900;
-	}
+	strcpy(format, "%02d-%02d-%d");
+	jul_to_cal_and_time(loc, ROUND_DAY, &y, &m, &d, &h, &mm, &sec);
 	sprintf(s, format, m, d, y);
 	break;
     case FORMAT_YYMMDD:
-	strcpy(format, "%d-%d-%d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
-	if (y >= 1900 && y < 2000) {
-	    y -= 1900;
-	}
+	strcpy(format, "%d-%02d-%02d");
+	jul_to_cal_and_time(loc, ROUND_DAY, &y, &m, &d, &h, &mm, &sec);
 	sprintf(s, format, y, m, d);
 	break;
     case FORMAT_MMYY:
-	strcpy(format, "%d-%d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
-	if (y >= 1900 && y < 2000) {
-	    y -= 1900;
-	}
+	strcpy(format, "%02d-%d");
+	jul_to_cal_and_time(loc, ROUND_MONTH, &y, &m, &d, &h, &mm, &sec);
 	sprintf(s, format, m, y);
 	break;
     case FORMAT_MMDD:
-	strcpy(format, "%d-%d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
+	strcpy(format, "%02d-%02d");
+	jul_to_cal_and_time(loc, ROUND_DAY, &y, &m, &d, &h, &mm, &sec);
 	sprintf(s, format, m, d);
 	break;
     case FORMAT_MONTHDAY:
-	strcpy(format, "%s-%d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
+	strcpy(format, "%s-%02d");
+	jul_to_cal_and_time(loc, ROUND_DAY, &y, &m, &d, &h, &mm, &sec);
 	if (m - 1 < 0 || m - 1 > 11) {
+	    sprintf(s, format, "???");
 	} else {
 	    sprintf(s, format, months[m - 1], d);
 	}
 	break;
     case FORMAT_DAYMONTH:
-	strcpy(format, "%d-%s");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
+	strcpy(format, "%02d-%s");
+	jul_to_cal_and_time(loc, ROUND_DAY, &y, &m, &d, &h, &mm, &sec);
 	if (m - 1 < 0 || m - 1 > 11) {
+	    sprintf(s, format, "???");
 	} else {
 	    sprintf(s, format, d, months[m - 1]);
 	}
 	break;
     case FORMAT_MONTHS:
 	strcpy(format, "%s");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
+	jul_to_cal_and_time(loc, ROUND_MONTH, &y, &m, &d, &h, &mm, &sec);
 	if (m - 1 < 0 || m - 1 > 11) {
+	    sprintf(s, format, "???");
 	} else {
 	    sprintf(s, format, months[m - 1]);
 	}
 	break;
-	break;
     case FORMAT_MONTHSY:
 	strcpy(format, "%s-%d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
+	jul_to_cal_and_time(loc, ROUND_MONTH, &y, &m, &d, &h, &mm, &sec);
 	if (m - 1 < 0 || m - 1 > 11) {
+	    sprintf(s, format, "???");
 	} else {
 	    sprintf(s, format, months[m - 1], y);
 	}
 	break;
-	break;
     case FORMAT_MONTHL:
 	strcpy(format, "%s");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
+	jul_to_cal_and_time(loc, ROUND_MONTH, &y, &m, &d, &h, &mm, &sec);
 	if (m - 1 < 0 || m - 1 > 11) {
+	    sprintf(s, format, "???");
 	} else {
 	    sprintf(s, format, monthl[m - 1]);
 	}
 	break;
-	break;
     case FORMAT_DAYOFWEEKS:
 	strcpy(format, "%s");
-	itmp = dayofweek(loc);
-	if ((itmp < 0) | (itmp > 6)) {
-	} else {
-	    sprintf(s, format, dayofweekstrs[dayofweek(loc)]);
-	}
+	sprintf(s, format, dayofweekstrs[dayofweek(loc + get_ref_date())]);
 	break;
     case FORMAT_DAYOFWEEKL:
 	strcpy(format, "%s");
-	itmp = dayofweek(loc);
-	if ((itmp < 0) | (itmp > 6)) {
-	} else {
-	    sprintf(s, format, dayofweekstrl[dayofweek(loc)]);
-	}
+	sprintf(s, format, dayofweekstrl[dayofweek(loc + get_ref_date())]);
 	break;
     case FORMAT_DAYOFYEAR:
 	strcpy(format, "%d");
-	sprintf(s, format, getndays(loc));
+        jul_to_cal_and_time(loc, ROUND_DAY, &y, &m, &d, &h, &mm, &sec);
+	sprintf(s, format,
+                1 + (int) (cal_to_jul(y, m, d) - cal_to_jul(y, 1, 1)));
 	break;
     case FORMAT_HMS:
 	strcpy(format, "%02d:%02d:%02d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
-	sprintf(s, format, h, mm, (int) sec);
+	jul_to_cal_and_time(loc, ROUND_SECOND, &y, &m, &d, &h, &mm, &sec);
+	sprintf(s, format, h, mm, sec);
 	break;
     case FORMAT_MMDDHMS:
-	strcpy(format, "%d-%d %02d:%02d:%02d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
-	if (y >= 1900 && y < 2000) {
-	    y -= 1900;
-	}
-	sprintf(s, format, m, d, h, mm, (int) sec);
+	strcpy(format, "%02d-%02d %02d:%02d:%02d");
+	jul_to_cal_and_time(loc, ROUND_SECOND, &y, &m, &d, &h, &mm, &sec);
+	sprintf(s, format, m, d, h, mm, sec);
 	break;
     case FORMAT_MMDDYYHMS:
-	strcpy(format, "%d-%d-%d %02d:%02d:%02d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
-	if (y >= 1900 && y < 2000) {
-	    y -= 1900;
-	}
-	sprintf(s, format, m, d, y, h, mm, (int) sec);
+	strcpy(format, "%02d-%02d-%d %02d:%02d:%02d");
+	jul_to_cal_and_time(loc, ROUND_SECOND, &y, &m, &d, &h, &mm, &sec);
+	sprintf(s, format, m, d, y, h, mm, sec);
 	break;
     case FORMAT_YYMMDDHMS:
-	strcpy(format, "%d-%d-%d %02d:%02d:%02d");
-	calcdate(loc, &m, &d, &y, &h, &mm, &sec);
-	if (y >= 1900 && y < 2000) {
-	    y -= 1900;
-	}
-	sprintf(s, format, y, m, d, h, mm, (int) sec);
+	strcpy(format, "%d-%02d-%02d %02d:%02d:%02d");
+	jul_to_cal_and_time(loc, ROUND_SECOND, &y, &m, &d, &h, &mm, &sec);
+	sprintf(s, format, y, m, d, h, mm, sec);
 	break;
     case FORMAT_DEGREESLON:
 	if (loc < 0.0) {
@@ -867,8 +734,8 @@ char *create_fstring(int form, int prec, double loc, int type)
 	    strcpy(format, "0 0'");
 	}
 	y = loc;
-	sec = (loc - y) * 60.0;
-	sprintf(s, format, y, prec, sec);
+	arcmin = (loc - y) * 60.0;
+	sprintf(s, format, y, prec, arcmin);
 	break;
     case FORMAT_DEGREESMMSSLON:
 	if (loc < 0.0) {
@@ -880,10 +747,10 @@ char *create_fstring(int form, int prec, double loc, int type)
 	    strcpy(format, "0 0' 0\"");
 	}
 	y = loc;
-	sec = (loc - y) * 3600.0;
-	m = sec / 60.0;
-	sec = (sec - m * 60);
-	sprintf(s, format, y, m, prec, sec);
+	arcsec = (loc - y) * 3600.0;
+	m = arcsec / 60.0;
+	arcsec = (arcsec - m * 60);
+	sprintf(s, format, y, m, prec, arcsec);
 	break;
     case FORMAT_MMSSLON:
 	if (loc < 0.0) {
@@ -895,10 +762,10 @@ char *create_fstring(int form, int prec, double loc, int type)
 	    strcpy(format, "0 0' 0\"");
 	}
 	y = loc;
-	sec = (loc - y) * 3600.0;
-	m = sec / 60.0;
-	sec = (sec - m * 60);
-	sprintf(s, format, m, prec, sec);
+	arcsec = (loc - y) * 3600.0;
+	m = arcsec / 60.0;
+	arcsec = (arcsec - m * 60);
+	sprintf(s, format, m, prec, arcsec);
 	break;
     case FORMAT_DEGREESLAT:
 	if (loc < 0.0) {
@@ -921,8 +788,8 @@ char *create_fstring(int form, int prec, double loc, int type)
 	    strcpy(format, "0 0'");
 	}
 	y = loc;
-	sec = (loc - y) * 60.0;
-	sprintf(s, format, y, prec, sec);
+	arcsec = (loc - y) * 60.0;
+	sprintf(s, format, y, prec, arcsec);
 	break;
     case FORMAT_DEGREESMMSSLAT:
 	if (loc < 0.0) {
@@ -934,10 +801,10 @@ char *create_fstring(int form, int prec, double loc, int type)
 	    strcpy(format, "0 0' 0\"");
 	}
 	y = loc;
-	sec = (loc - y) * 3600.0;
-	m = sec / 60.0;
-	sec = (sec - m * 60);
-	sprintf(s, format, y, m, prec, sec);
+	arcsec = (loc - y) * 3600.0;
+	m = arcsec / 60.0;
+	arcsec = (arcsec - m * 60);
+	sprintf(s, format, y, m, prec, arcsec);
 	break;
     case FORMAT_MMSSLAT:
 	if (loc < 0.0) {
@@ -949,15 +816,18 @@ char *create_fstring(int form, int prec, double loc, int type)
 	    strcpy(format, "0 0' 0\"");
 	}
 	y = loc;
-	sec = (loc - y) * 3600.0;
-	m = sec / 60.0;
-	sec = (sec - m * 60);
-	sprintf(s, format, m, prec, sec);
+	arcsec = (loc - y) * 3600.0;
+	m = arcsec / 60.0;
+	arcsec = (arcsec - m * 60);
+	sprintf(s, format, m, prec, arcsec);
 	break;
     default:
 	sprintf(s, format, prec, loc);
 	break;
     }
+
+    /* revert to POSIX */
+    set_locale_num(FALSE);
     
     return(s);
 }
@@ -1016,7 +886,7 @@ unsigned char reversebits(unsigned char inword)
     return (result);
 }
 
-char *copy_string(char *dest, char *src)
+char *copy_string(char *dest, const char *src)
 {
     if (src == dest) {
         ;
@@ -1030,6 +900,21 @@ char *copy_string(char *dest, char *src)
     return(dest);
 }
 
+char *concat_strings(char *dest, const char *src)
+{
+    if (src != NULL) {
+        if (dest == NULL) {
+            dest = copy_string(NULL, src);
+        } else {
+            dest = xrealloc(dest, (strlen(dest) + strlen(src) + 1)*SIZEOF_CHAR);
+            if (dest != NULL) {
+                strcat(dest, src);
+            }
+        }
+    }
+    return(dest);
+}
+
 void reverse_string(char *s)
 {
     char cbuf;
@@ -1046,3 +931,488 @@ void reverse_string(char *s)
         s[len - i - 1] = cbuf;
     }
 }
+
+int compare_strings(const char *s1, const char *s2)
+{
+    if (s1 == NULL && s2 == NULL) {
+        return TRUE;
+    } else if (s1 == NULL || s2 == NULL) {
+        return FALSE;
+    } else {
+        return (strcmp(s1, s2) == 0);
+    }
+}
+
+/* location of Grace home directory */
+static char grace_home[GR_MAXPATHLEN] = GRACE_HOME;	
+
+char *get_grace_home(void)
+{
+    return grace_home;
+}
+
+void set_grace_home(const char *dir)
+{
+    strncpy(grace_home, dir, GR_MAXPATHLEN - 1);
+}
+
+/* print command */
+static char print_cmd[GR_MAXPATHLEN] = GRACE_PRINT_CMD;	
+
+char *get_print_cmd(void)
+{
+    return print_cmd;
+}
+
+void set_print_cmd(const char *cmd)
+{
+    strncpy(print_cmd, cmd, GR_MAXPATHLEN - 1);
+}
+
+/* editor */
+static char grace_editor[GR_MAXPATHLEN] = GRACE_EDITOR;	
+
+char *get_editor(void)
+{
+    return grace_editor;
+}
+
+void set_editor(const char *cmd)
+{
+    strncpy(grace_editor, cmd, GR_MAXPATHLEN - 1);
+}
+
+static char help_viewer[GR_MAXPATHLEN] = GRACE_HELPVIEWER;	
+
+char *get_help_viewer(void)
+{
+    return help_viewer;
+}
+
+void set_help_viewer(const char *dir)
+{
+    strncpy(help_viewer, dir, GR_MAXPATHLEN - 1);
+}
+
+/* project file name */
+static char docname[GR_MAXPATHLEN] = NONAME;	
+
+char *get_docname(void)
+{
+    return docname;
+}
+
+void set_docname(const char *s)
+{
+    if (s != NULL) {
+        strncpy(docname, s, GR_MAXPATHLEN - 1);
+    } else {
+        strcpy(docname, NONAME);
+    }
+}
+
+/*
+ * stuff results, etc. into a text window
+ */
+void log_results(char *buf)
+{
+    char tmpbuf[512];
+    if (logwindow) {
+        strcpy(tmpbuf, buf);
+        if (tmpbuf[strlen(tmpbuf) - 1] != '\n') {
+            strcat(tmpbuf, "\n");
+        }
+        stufftext(tmpbuf, 1);
+    }
+}
+
+
+void errmsg(char *buf)
+{
+#ifdef NONE_GUI
+    fprintf(stderr, "%s\n", buf);
+#else
+    if (inwin) {
+        errwin(buf);
+    } else {
+        fprintf(stderr, "%s\n", buf);
+    }
+#endif
+}
+
+int yesnoterm(char *msg)
+{
+    return 1;
+}
+
+int yesno(char *msg, char *s1, char *s2, char *help_anchor)
+{
+    if (noask) {
+	return 1;
+    }
+#ifdef NONE_GUI
+    return (yesnoterm(msg));
+#else
+    if (inwin) {
+        return (yesnowin(msg, s1, s2, help_anchor));
+    } else {
+        return (yesnoterm(msg));
+    }
+#endif
+}
+ 
+
+int fexists(const char *to)
+{
+    struct stat stto;
+    char tbuf[256];
+
+    if (stat(to, &stto) == 0) {
+	sprintf(tbuf, "Overwrite %s?", to);
+	if (!yesno(tbuf, NULL, NULL, NULL)) {
+	    return (1);
+	}
+	return (0);
+    }
+    return (0);
+}
+
+void stufftext(char *s, int sp)
+{
+#ifdef NONE_GUI
+    printf("%s", s);
+#else
+    stufftextwin(s, sp);
+#endif
+}
+
+
+char *mybasename(const char *s)
+{
+    int start, end;
+    static char basename[GR_MAXPATHLEN];
+    
+    s = path_translate(s);
+    if (s == NULL) {
+        errmsg("Could not translate basename:");
+        return "???";
+    }
+    
+    end = strlen(s) - 1;
+    
+    /* root is a special case */
+    if (end == 0 && *s == '/'){
+        basename[0] = '/';
+        return basename;
+    }
+
+    /* strip trailing white space and slashes */
+    while (s[end] == '/' || s[end] == ' ' || s[end] == '\t') {
+        end--;
+    }
+    /* find start of basename */
+    start = end;
+    do {
+        start--;
+    } while (start >= 0 && s[start] != '/');
+
+    strncpy(basename, s + (start + 1), end - start);
+    basename[end - start] = '\0';
+    return basename;
+}
+
+static char workingdir[GR_MAXPATHLEN];
+
+int set_workingdir(const char *wd)
+{
+    char buf[GR_MAXPATHLEN];
+    
+    if (wd == NULL) {
+        getcwd(workingdir, GR_MAXPATHLEN - 1);
+        if (workingdir[strlen(workingdir)-1] != '/') {
+            strcat(workingdir, "/");
+        }
+        return GRACE_EXIT_SUCCESS;
+    }
+    
+    strncpy(buf, wd, GR_MAXPATHLEN - 1);
+    if (buf[0] == '~') {
+        expand_tilde(buf);
+    }
+    if (chdir(buf) >= 0) {
+        strncpy(workingdir, buf, GR_MAXPATHLEN - 1);
+        if (workingdir[strlen(workingdir)-1] != '/') {
+            strcat(workingdir, "/");
+        }
+	return GRACE_EXIT_SUCCESS;
+    } else {
+        return GRACE_EXIT_FAILURE;
+    }
+}
+
+char *get_workingdir(void)
+{
+    return workingdir;
+}
+
+static char *username = NULL;
+
+void init_username(void)
+{
+    char *s;
+
+/*
+ *     We don't use it for any kind of authentication, so why not let
+ *     user to customize her name? :)
+ */
+    s = getenv("LOGNAME");
+    if (s == NULL || s[0] == '\0') {
+        s = getlogin();
+        if (s == NULL || s[0] == '\0') {
+            s = "a user";
+        }
+    }
+    username = copy_string(username, s);
+}
+
+char *get_username(void)
+{
+    return username;
+}
+
+static char *userhome = NULL;
+
+void init_userhome(void)
+{
+    userhome = copy_string(NULL, getenv("HOME"));
+    if (userhome == NULL || userhome[strlen(userhome) - 1] != '/') {
+        userhome = concat_strings(userhome, "/");
+    }
+}
+
+char *get_userhome(void)
+{
+    return userhome;
+}
+
+/* TODO this needs some work */
+void expand_tilde(char *buf)
+{
+    char buf2[GR_MAXPATHLEN];
+
+    if (buf[0] == '~') {
+	if (strlen(buf) == 1) {
+            strcpy(buf, get_userhome());
+	} else if (buf[1] == '/') {
+            if (strlen(buf) > 2) {
+                strcpy(buf2, get_userhome());
+	        strcat(buf2, buf + 1);
+	        strcpy(buf, buf2);
+            } else {
+                strcpy(buf, get_userhome());
+            }
+	} else {
+	    char tmp[128], *pp = tmp, *q = buf + 1;
+	    struct passwd *pent;
+
+	    while (*q && (*q != '/')) {
+		*pp++ = *q++;
+	    }
+	    *pp = 0;
+	    if ((pent = getpwnam(tmp)) != NULL) {
+		strcpy(buf2, pent->pw_dir);
+		strcat(buf2, "/");
+		strcat(buf2, q);
+		strcpy(buf, buf2);
+	    } else {
+		errmsg("No user by that name");
+	    }
+	}
+    }
+}
+
+void echomsg(char *msg)
+{
+    if (logwindow) {
+	log_results(msg);
+    }
+    if (inwin) {
+#ifndef NONE_GUI
+        set_left_footer(msg);
+#endif
+    } else {
+        printf("%s\n", msg);
+    }
+}
+
+static void update_timestamp(void)
+{
+    struct tm tm;
+    time_t time_value;
+    char *str;
+
+    (void) time(&time_value);
+    tm = *localtime(&time_value);
+    str = asctime(&tm);
+    if (str[strlen(str) - 1] == '\n') {
+        str[strlen(str) - 1]= '\0';
+    }
+    set_plotstr_string(&timestamp, str);
+}
+
+void update_app_title(void)
+{
+#ifndef NONE_GUI
+    char buf[100];
+    
+    strcpy(buf, "Grace: ");
+    strncat(buf, mybasename(get_docname()), 80);
+    if (is_dirtystate()) {
+        strcat(buf, " (modified)");
+    }
+
+    set_title(buf);
+#endif
+}
+
+/*
+ * dirtystate routines
+ */
+
+static int dirtystate = 0;
+static int dirtystate_lock = FALSE;
+
+void set_dirtystate(void)
+{
+    if (dirtystate_lock == FALSE) {
+        dirtystate++;
+        update_timestamp();
+        update_app_title();
+
+/*
+ * TODO:
+ * 	if ( (dirtystate > SOME_LIMIT) || 
+ *           (current_time - autosave_time > ANOTHER_LIMIT) ) {
+ * 	    autosave();
+ * 	}
+ */
+    }
+}
+
+void clear_dirtystate(void)
+{
+    dirtystate = 0;
+    dirtystate_lock = FALSE;
+    update_app_title();
+}
+
+void lock_dirtystate(flag)
+{
+    dirtystate_lock = flag;
+}
+
+int is_dirtystate(void)
+{
+    return (dirtystate ? TRUE:FALSE);
+}
+
+int system_wrap(const char *string)
+{
+    return system(string);
+}
+
+void msleep_wrap(unsigned int msec)
+{
+    struct timeval timeout;
+    timeout.tv_sec = msec / 1000;
+    timeout.tv_usec = 1000 * (msec % 1000);
+    select(0, NULL, NULL, NULL, &timeout);    
+}
+
+static int need_locale = FALSE;
+static char *system_locale_string, *posix_locale_string;
+
+int init_locale(void)
+{
+    char *s;
+    s = setlocale(LC_NUMERIC, "");
+    if (s == NULL) {
+        /* invalid/unsupported locale */
+        return GRACE_EXIT_FAILURE;
+    } else if (!strcmp(s, "C")) {
+        /* don't enable need_locale, since the system locale is C */
+        return GRACE_EXIT_SUCCESS;
+    } else {
+        system_locale_string = copy_string(NULL, s);
+        s = setlocale(LC_NUMERIC, "C");
+        posix_locale_string = copy_string(NULL, s);
+        need_locale = TRUE;
+        return GRACE_EXIT_SUCCESS;
+    }
+}
+
+void set_locale_num(int flag)
+{
+#ifdef HAVE_SETLOCALE
+    if (need_locale) {
+        if (flag == TRUE) {
+            setlocale(LC_NUMERIC, system_locale_string);
+        } else {
+            setlocale(LC_NUMERIC, posix_locale_string);
+        }
+    }
+#endif
+}
+
+/*
+ * Build info stuff
+ */
+long bi_version_id(void)
+{
+    return BI_VERSION_ID;
+}
+
+char *bi_version_string(void)
+{
+    return BI_VERSION;
+}
+
+char *bi_system(void)
+{
+    return BI_SYSTEM;
+}
+
+char *bi_date(void)
+{
+    return BI_DATE;
+}
+
+char *bi_gui(void)
+{
+    return BI_GUI;
+}
+
+char *bi_t1lib(void)
+{
+    return BI_T1LIB;
+}
+
+char *bi_ccompiler(void)
+{
+    return BI_CCOMPILER;
+}
+
+#ifdef DEBUG
+static int debuglevel = 0;
+
+void set_debuglevel(int level)
+{
+    debuglevel = level;
+}
+
+int get_debuglevel(void)
+{
+    return debuglevel;
+}
+#endif
+
